
/* 
   Project:     SOFTSUSY 1.2
   File:        numerics.h
   Author:      Ben Allanach 
   Manual:      hep-ph/0104145 
   Webpage:     http://allanach.home.cern.ch/allanach/softsusy.html
   Description: Integration of ODEs by Runge Kutta, minimum finding and
                derivative calculation

   $Log: numerics.h,v $
   Revision 1.3  2001/07/18 14:42:51  allanach
   Added proper header info

   */

#ifndef NUMERICS_H
#define NUMERICS_H

#include "linalg.h"
void rungeKuttaStep(const DoubleVector & y, const DoubleVector & dydx, 
	     double x, double h, DoubleVector & yout, DoubleVector & yerr, 
	     DoubleVector (*derivs)(double, const DoubleVector &));

int odeStepper(DoubleVector & y, const DoubleVector & dydx, double *x, double
		htry, double eps, DoubleVector & yscal, double *hdid, 
		double *hnext,		
		DoubleVector (*derivs)(double, const DoubleVector &));

int integrateOdes(DoubleVector & ystart, double x1, double x2, double eps,
		  double h1, double hmin, 
		  DoubleVector (*derivs)(double, const DoubleVector &),
		  int (*rkqs)
		  (DoubleVector & y, const DoubleVector & dydx, double *x,
		   double htry, double eps, DoubleVector & yscal, double
		   *hdid, double *hnext, 
		   DoubleVector (*derivs)(double, const DoubleVector &)));

double calcDerivative(double (*func)(double), 
		     double x, double h, double *err);

double findMinimum(double ax, double bx, double cx, double (*f)(double),
		   double tol, double *xmin);

void shft2(double & a, double & b, double & c);
void shft3(double & a, double & b, double & c, double & d);

// For calculation of PV functions
double integrandThreshbn(double x);
double bIntegral(int n, double p, double m1, double m2, double mt);
DoubleVector dd(double x, const DoubleVector & y);

// Passarino-Veltman function definitions
double b0(double p, double m1, double m2, double q);
double b1(double p, double m1, double m2, double q);
double b22(double p,  double m1, double m2, double q);
double c0(double m1, double m2, double m3);
double d27(double m1, double m2, double m3, double m4);
double d0(double m1, double m2, double m3, double m4);

// inlined PV functions
inline double a0(double m, double q) {
  if (m == 0.0) return 0.0;
  return sqr(m) * (1.0 - log(sqr(m / q)));
}

inline double ffn(double p, double m1, double m2, double q) {
  return a0(m1, q) - 2 * a0(m2, q) - (2 * sqr(p) + 2 * sqr(m1) - 2 * sqr(m2)) *
    b0(p, m1, m2, q);
}

inline double gfn(double p, double m1, double m2, double q) {
  return (sqr(p) - sqr(m1) - sqr(m2)) * b0(p, m1, m2, q) - a0(m1, q) 
    - a0(m2, q); 
}

inline double hfn(double p, double m1, double m2, double q) {
  return 4.0 * b22(p, m1, m2, q) + gfn(p, m1, m2, q);
}

inline double b22bar(double p, double m1, double m2, double q) {
  return b22(p, m1, m2, q) - 0.25 * a0(m1, q) - 0.25 * a0(m2, q);
}

inline double fB(const Complex & x) {
  return (log(1.0 - x) - x * log(1.0 - 1.0 / x) - 1.0).real();
};

double integrandThreshbnr(double x);
Complex fnfn(double x);

#endif

