
/* 
   Project:     SOFTSUSY 1.2
   File:        softpars.cpp
   Author:      Ben Allanach 
   Manual:      hep-ph/0104145 
   Webpage:     http://allanach.home.cern.ch/allanach/softsusy.html

   $Log: softpars.cpp,v $
   Revision 1.2  2001/07/18 14:42:51  allanach
   Added proper header info

   */

#include "softpars.h"

const SoftParsMssm & SoftParsMssm::operator=(const SoftParsMssm & s) {
  if (this == &s) return *this;
  mGaugino = s.mGaugino;
  ua = s.ua;
  da = s.da;
  ea = s.ea;
  mQLsq = s.mQLsq;
  mURsq = s.mURsq;
  mDRsq = s.mDRsq;
  mLLsq = s.mLLsq;
  mSEsq = s.mSEsq;
  b = s.b;
  mH1sq = s.mH1sq;
  mH2sq = s.mH2sq;
  setSusy(s.displaySusy());
  setMu(s.displayMu());
  setLoops(s.displayLoops());
  setThresholds(s.displayThresholds());
  return *this;
}

DoubleMatrix SoftParsMssm::displayTrilinear(trilinears k) const {
  switch(k) {
  case UA: return ua; break;
  case DA: return da; break;
  case EA: return ea; break;
  }
  DoubleMatrix temp(3, 3);
  return temp;
}

// Will display zero if that's what the A term is regardless of what the
// Yukawa coupling is (even if it's zero).
double SoftParsMssm::displaySoftA(trilinears k, int i, int j) const {
  double am = displayTrilinear(k)(i, j);
  if (fabs(am) < EPS) return 0.0;
  
  if (fabs(displayYukawaElement(yukawa(k), i, j)) < 1.0e-10) {
    cout << "WARNING: asking for SoftParsMssm::displaySoftA(" << int(k) << "," 
	 << i << "," << j << "), where Yukawa coupling is " <<
      fabs(displayYukawaElement(yukawa(k), i, j)) << endl;
    exit(1);
  }
  double temp = displayTrilinear(k)(i, j) 
    / displayYukawaElement(yukawa(k), i, j); 
  return temp;
}

DoubleMatrix SoftParsMssm::displaySoftMassSquared(softMasses k) const {
  switch(k) {
  case mQl: return mQLsq; break;
  case mUr: return mURsq; break;
  case mDr: return mDRsq; break;
  case mLl: return mLLsq; break;
  case mEr: return mSEsq; break;
  }
  DoubleMatrix temp(3, 3);
  return temp;
}

DoubleVector SoftParsMssm::display() const {
  DoubleVector y(MssmSusy::display());
  y.setEnd(110);
  int i, j, k=32;
  for (i=1; i<=3; i++) {
    k++;
    y(k) = displayGaugino()(i);
  }
  
  for (i=1; i<=3; i++)    
    for (j=1; j<=3; j++) {
      k++;
      y(k) = displayTrilinear(UA)(i, j);
      y(k+9) = displayTrilinear(DA)(i, j);
      y(k+18) = displayTrilinear(EA)(i, j);
      y(k+27) = displaySoftMassSquared(mQl)(i, j);
      y(k+36) = displaySoftMassSquared(mUr)(i, j);
      y(k+45) = displaySoftMassSquared(mDr)(i, j);
      y(k+54) = displaySoftMassSquared(mLl)(i, j);
      y(k+63) = displaySoftMassSquared(mEr)(i, j);
    }
  y(108) = b;
  y(109) = mH1sq;
  y(110) = mH2sq;
  return y;
}

void SoftParsMssm::setSoftMassElement(softMasses k, int i, int j, 
					  double f) { 
  switch(k) {
  case mQl: mQLsq(i, j) = f; break;
  case mUr: mURsq(i, j) = f; break;
  case mDr: mDRsq(i, j) = f; break;
  case mLl: mLLsq(i, j) = f; break;
  case mEr: mSEsq(i, j) = f; break;
  }
}

void SoftParsMssm::setSoftMassMatrix(softMasses k, const DoubleMatrix & m) { 
  switch(k) {
  case mQl: mQLsq = m; break;
  case mUr: mURsq = m; break;
  case mDr: mDRsq = m; break;
  case mLl: mLLsq = m; break;
  case mEr: mSEsq = m; break;
  }
}

void SoftParsMssm::setTrilinearMatrix(trilinears k, const DoubleMatrix & m) { 
  switch(k) {
  case UA: ua = m; break;
  case DA: da = m; break;
  case EA: ea = m; break;
  }
}

void SoftParsMssm::setTrilinearElement(trilinears k, int i, int j, 
					  double m) { 
  switch(k) {
  case UA: ua(i, j) = m; break;
  case DA: da(i, j) = m; break;
  case EA: ea(i, j) = m; break;
  }
}

void SoftParsMssm::setAllGauginos(const DoubleVector & v) { 
  if (v.displayStart() != 1 || v.displayEnd() !=3) {
    cout << "Initialising SoftParsMssm::setAllGauginos with vector"
	 << v;
    exit(1);
  }
  mGaugino = v; 
}

void SoftParsMssm::setSusy(const MssmSusy & s) {
  setLoops(s.displayLoops());
  setThresholds(s.displayThresholds());
  setMu(s.displayMu());
  setYukawaMatrix(YU, s.displayYukawaMatrix(YU)); 
  setYukawaMatrix(YD, s.displayYukawaMatrix(YD)); 
  setYukawaMatrix(YE, s.displayYukawaMatrix(YE)); 
  setTanb(s.displayTanb());
  setSusyMu(s.displaySusyMu());
  setAllGauge(s.displayGauge());
}

void SoftParsMssm::set(const DoubleVector & y) {
  MssmSusy::set(y);
  int i, j, k=32;
  for (i=1; i<=3; i++) {
    k++;
    mGaugino(i) = y.display(k);
  }
  
  for (i=1; i<=3; i++)
    for (j=1; j<=3; j++) {
      k++;
      ua(i, j) = y.display(k);
      da(i, j) = y.display(k+9);
      ea(i, j) = y.display(k+18);
      mQLsq(i, j) = y.display(k+27);
      mURsq(i, j) = y.display(k+36);
      mDRsq(i, j) = y.display(k+45);
      mLLsq(i, j) = y.display(k+54);
      mSEsq(i, j) = y.display(k+63);
    }
  b = y.display(108);
  mH1sq = y.display(109);
  mH2sq = y.display(110);
}


// Outputs derivatives (DRbar scheme) in the form of dsoft
// thresholds = 0 and NOTHING is decoupled.
// thresholds = 2 and SUSY params/gauge couplings are decoupled at sparticle
// thresholds.
SoftParsMssm SoftParsMssm::beta2() const {
  //  susypars beta functions are defined in the
  //  Allanach, Dedes, Dreiner hep-ph/9906209 paper notation 
  //  W=  LL Y^E H1 ER + QL Y^D H1 DR + QL Y^U H2 UR + smu H1 H2, 
  //  (implicit epsilonAb phi^a phi^b a,b=Su(2) indices present where
  //  epsilon12 = +1, epsilon21 = -1, other components zero
  //  The following equations are for Barger, Berger and Ohmann's conventions
  //  with the same W: hep-ph/9209232, 9311269
  //  VTri = UR ua QL H2 + DR da QL H1 + ER ea LL H1 + m3 H1 H2 +hc
  //  VBi  = mH1sq |H1|^2 + mH2sq |H2|^2 + Q^* mQLsq Q + L^* mLLsq L +
  //          UR MuR UR^* + DR MDR DR^* + ER MER ER^*
  // Note in particular that ua(3,3) is NOT AT. AT = ua(3, 3) / ht.
  
  // Constants for gauge running
  static DoubleVector bBeta(3), cuBeta(3), cdBeta(3), ceBeta(3);
  static DoubleMatrix babBeta(3, 3);
  
  if (bBeta(1) < 1.0e-5) // Constants not set yet
    setBetas(babBeta, cuBeta, cdBeta, ceBeta, bBeta);
  
  // For calculational brevity: 
  static sBrevity a;
  // convert to beta functions
  static MssmSusy dsb;
  
  // calculate derivatives for full SUSY spectrum. Brevity calculations come
  // out encoded in a
  dsb = MssmSusy::beta(a);
  
  static DoubleVector g1(3);
  g1 = displayGauge();
  
  // To keep this a const function: TIME SAVINGS
  DoubleMatrix &u1=a.u1, &d1=a.d1, &e1=a.e1;
  DoubleMatrix &u2=a.u2, &d2=a.d2, &e2=a.e2,
    &u2t=a.u2t, &d2t=a.d2t, &e2t=a.e2t, &dt=a.dt, &ut=a.ut, &et=a.et;      
  double &uuT = a.uuT, &ddT = a.ddT, &eeT = a.eeT;
  DoubleVector &gsq=a.gsq; // &g3=a.g3, &g4=a.g4;
  
  static DoubleMatrix hu(3, 3), hd(3, 3), he(3, 3);
  static DoubleMatrix hut(3, 3), hdt(3, 3), het(3, 3);
  static DoubleMatrix hu2(3, 3), hd2(3, 3), he2(3, 3);
  static DoubleMatrix hu2t(3, 3), hd2t(3, 3), he2t(3, 3);
  static DoubleMatrix mq(3, 3), mu(3, 3), md(3, 3), ml(3, 3), me(3, 3);
  static DoubleVector mG(1, 3), msq(1, 3), gsqM(1, 3), gMsq(1, 3);
  
  hu = displayTrilinear(UA); hd = displayTrilinear(DA); 
  he = displayTrilinear(EA); 
  hut = hu.transpose(); hdt = hd.transpose(); het = he.transpose();
  hu2 = hu * hut; hd2 = hd * hdt; he2 = he * het;
  double huT = hu2.trace(), hdT = hd2.trace(), heT = he2.trace();
  hu2t = hut * hu; hd2t = hdt * hd; he2t = het * he;
  mq = displaySoftMassSquared(mQl); mu = displaySoftMassSquared(mUr);
  md = displaySoftMassSquared(mDr);
  ml = displaySoftMassSquared(mLl); me = displaySoftMassSquared(mEr); 
  double mqT = mq.trace(), muT = mu.trace(), mdT = md.trace(), meT =
    me.trace(), mlT = ml.trace();
  
  double huuT = (hu * ut).trace(), hddT = (hd * dt).trace(), 
    heeT = (he * et).trace(); 
  mG = mGaugino; msq = mG * mG; gsqM = gsq * mG; gMsq = gsq * msq;
  
  // derivatives of soft parameters
  static DoubleVector dmG(1, 3);
  static double dmH1sq, dmH2sq, dB;
  static DoubleMatrix dmq(3, 3), dmu(3, 3), dmd(3, 3), dme(3, 3), dml(3, 3);
  static DoubleMatrix dhu(3, 3), dhd(3, 3), dhe(3, 3);
  
  const double ONEO16Pisq = 1.0 / (16 * sqr(PI));
  if (displayLoops() > 0) {
    const static double sixteenO3 = 16.0 / 3.0, oneO15 = 1.0 /
      15.0;
    
    dmG = gsqM * 2.0 * bBeta;
    
    double curlyS = mH2sq - mH1sq + mqT - mlT - 2 * muT + mdT + meT;
    
    dB = 2.0 * (0.6 * gsqM(1) + 3 * gsqM(2) + 3 * (u1 * hu).trace()  + 
		3 * (d1 * hd).trace() + (e1 * he).trace());
    
    dmH1sq = 2.0 * 
      (-0.3 * gsq(1) * curlyS - 0.6 * gMsq(1) - 3.0 * gMsq(2) +
       3.0 * (mH1sq * ddT + (d2t * (mq + md)).trace() + hdT) +
       (mH1sq * eeT + (e2t * (ml + me)).trace() + heT));
    
    dmH2sq = 2.0 * 
      (0.3 * gsq(1) * curlyS - 0.6 * gMsq(1) - 3.0 * gMsq(2) +
       3.0 * (mH2sq * uuT + (u2t * (mq + mu)).trace() + huT));
    
    dmq = 2.0 *
      (0.1 * gsq(1) * curlyS - oneO15 * gMsq(1) - 3.0 * gMsq(2) -
       sixteenO3 * gMsq(3) +
       0.5 * (u2 * mq + mq * u2 + 2.0 * (u1 * mu * ut + mH2sq * u2 + hu2))
       + 0.5 * (d2 * mq + mq * d2 + 2.0 * (d1 * md * dt + mH1sq * d2 +
					   hd2)));
    
    dml = 2.0 *
      ( -0.3 * gsq(1) * curlyS - 0.6 * gMsq(1) - 3.0 * gMsq(2) + 
	0.5 * (e2 * ml + ml * e2 + 2.0 * (e1 * me * et + mH1sq * e2 + he2)));
    
    dmu = 2.0 *
      (-0.4 * gsq(1) * curlyS - 16 * oneO15 * gMsq(1) - sixteenO3 *
       gMsq(3) + 
       (u2t * mu + mu * u2t + 2.0 * (ut * mq * u1 + mH2sq * u2t + hu2t)));
    
    dmd = 2.0 *
      (0.2 * gsq(1) * curlyS - 4.0 * oneO15 * gMsq(1) - sixteenO3 *
       gMsq(3) + 
       (d2t * md + md * d2t + 2.0 * (dt * mq * d1 + mH1sq * d2t + hd2t)));
    
    dme = 2.0 *
      (0.6 * gsq(1) * curlyS - 36 * oneO15 * gMsq(1) +
       (e2t * me + me * e2t + 2.0 * (et * ml * e1 + mH1sq * e2t + he2t)));
    
    
    dhu = (-sixteenO3 * gsq(3) - 3 * gsq(2) - 13 * oneO15 * gsq(1))
      * hu
      + 2 * (13.0 * oneO15 * gsqM(1) + 3 * gsqM(2) + sixteenO3
	     * gsqM(3)) * u1 +
      4.0 * hu * u2t + 6.0 * huuT * u1 + 5.0 * u2 * hu + 3.0 *
      uuT * hu + 2.0 * hd * dt * u1 + d2 * hu;
    
    dhd = (-sixteenO3 * gsq(3) - 3 * gsq(2) - 7 * oneO15 * gsq(1)) *
      hd 
      + 2 * (7 * oneO15 * gsqM(1) + 3 * gsqM(2) + sixteenO3
	     * gsqM(3)) * d1 +
      4.0 * hd * d2t + 6.0 * hddT * d1 + 5.0 * d2 * hd + 3.0 *
      ddT * hd + 2.0 * hu * ut * d1 + u2 * hd + 2.0 * heeT * d1 + eeT
      * hd;
    
    dhe = (- 3 * gsq(2) - 1.8 * gsq(1)) * he
      + 2 * (1.8 * gsqM(1) + 3 * gsqM(2)) * e1 +
      4.0 * he * e2t + 2.0 * heeT * e1 + 5.0 * e2 * he + 
      eeT * he + 6.0 * hddT * e1 + 3 * ddT * he;
    
    // convert to proper derivatives: 
    dmG = dmG * ONEO16Pisq;
    dB = dB * ONEO16Pisq;
    dmH1sq = dmH1sq * ONEO16Pisq;
    dmH2sq = dmH2sq * ONEO16Pisq;
    dmq = dmq * ONEO16Pisq;
    dml = dml * ONEO16Pisq;
    dmu = dmu * ONEO16Pisq;
    dmd = dmd * ONEO16Pisq;
    dme = dme * ONEO16Pisq;
    dhu = dhu * ONEO16Pisq;
    dhd = dhd * ONEO16Pisq;
    dhe = dhe * ONEO16Pisq;
  }
  
  // two-loop contributions. I got these from hep-ph/9311340. WIth respect to
  // their notation, the Yukawas and h's are TRANSPOSED. Gaugino masses are
  // identical, as are soft masses. B(mV) = mu(BBO) B(BBO)
  if (displayLoops() > 1) {
    static DoubleVector dmG2(1, 3);
    /*      static double dmH1sq2, dmH2sq2, dB2;
	    static DoubleMatrix dmq2(3, 3), dMu2(3, 3), dMd2(3, 3), dme2(3, 3), 
	    dml2(3, 3);
	    static DoubleMatrix dHu2(3, 3), dHd2(3, 3), dHe2(3, 3);
	    */    
    const double oneO16Pif = sqr(ONEO16Pisq);
    
    dmG2 = 2.0 * gsq * 
      (babBeta * gsqM + mG * (babBeta * gsq) + 
       cuBeta * huuT - cuBeta * mG * uuT + 
       cdBeta * hddT - cdBeta * mG * ddT + 
       cdBeta * heeT - ceBeta * mG * eeT); // checked
    
    dmG = dmG + dmG2 * oneO16Pif;
    /*          dB = dB + dB2 * oneO16Pif;
	    dmH1sq = dmH1sq + dmH1sq2 * oneO16Pif;
	    dmH2sq = dmH2sq + dmH2sq2 * oneO16Pif;
	    dmq = dmq + dmq2 * oneO16Pif;
	    dml = dml + dml2 * oneO16Pif;
	    dmu = dmu + dmu2 * oneO16Pif;
	    dmd = dmd + dmd2 * oneO16Pif;
	    dmE = dmE + dme2 * oneO16Pif;
	    dHu = dHu + dHu2 * oneO16Pif;
	    dHd = dHd + dHd2 * oneO16Pif;
	    dHE = dHE + dHe2 * oneO16Pif;*/
  }

  // Check for non-perturbativeness
  /*  const double nonPertLimit = 5.0;
  if (displayYukawaElement(YU, 3, 3) > nonPertLimit ||
      displayYukawaElement(YD, 3, 3)> nonPertLimit) {
    return SoftParsMssm();
    }*/
    
  
  SoftParsMssm dsoft(dsb, dmG, dhu, dhd, dhe, dmq, dmu,
		      dmd, dml, dme, dB, dmH1sq, dmH2sq, displayMu(),
		      displayLoops(), displayThresholds());
  
  return dsoft;
}

// Outputs derivatives vector y[109] for SUSY parameters: interfaces to
// integration routines
DoubleVector SoftParsMssm::beta() const {
  // calculate the derivatives
  static SoftParsMssm dsoft; dsoft = beta2();
  
  return dsoft.display(); // convert to a long vector
}

/* 
   Give it a SUSY object and a value of M3/2, and it will return a soft
   object with AMSB soft breaking terms. Note that the sleptons will be
   tachyonic, ie nothing has been done to fix that problem.
   Note that in the following, we are neglecting all Yukawa couplings except
   that of the third family.
   
   THE CURRENT STATE OF PLAY:
   trilinear and gauge interactions will generalise to two loops OK
   trilinear interactions have full flavour structure
   soft scalar masses are ONE LOOP ONLY, with 3rd family approximation and no
   particular flavour structure. However, should you require full flavour
   structure (for example for FCNC constraints), this should be relatively
   easy to incorporate. Two loop additions are possible, but a pain.
   */
void SoftParsMssm::addAmsb(double m32) {
  MssmSusy run(displaySusy());
  const double ONEO16pisq = 1.0 / (16. * sqr(PI));
    
  double 
    yt   = run.displayYukawaElement(YU, 3, 3), 
    yb   = run.displayYukawaElement(YD, 3, 3), 
    ytau = run.displayYukawaElement(YE, 3, 3),
    g1   = run.displayGauge()(1), 
    g2   = run.displayGauge()(2), 
    g3   = run.displayGauge()(3); 
  
  // gauge parts of beta functions
  double Xu = -13.0 / 15.0 * sqr(g1) - 3.0 * sqr(g2) - 16.0 / 3.0 * sqr(g3);
  double Xd = -7.0 / 15.0 * sqr(g1) - 3.0 * sqr(g2) - 16.0 / 3.0 * sqr(g3);
  double Xe = -9 / 5.0 * sqr(g1) - 3.0 * sqr(g2);
  
  // Yukawa parts
  double betayt   = yt *   (Xu + 6.0 * sqr(yt) + sqr(yb));
  double betayb   = yb *   (Xd + 6.0 * sqr(yb) + sqr(yt) + sqr(ytau));
  double betaytau = ytau * (Xe + 3.0 * sqr(yb) + 4.0 * sqr(ytau));
  
  const double ONEO16pif = sqr(ONEO16pisq);

  // Higgs
  double mhusq = ONEO16pif * sqr(m32) * 
    (-99. / 50. * sqr(sqr(g1)) - 1.5 * sqr(sqr(g2)) + 3.0 * yt * betayt);
  double mhdsq = ONEO16pif * sqr(m32) * 
    (-99. / 50. * sqr(sqr(g1)) - 1.5 * sqr(sqr(g2)) + 3.0 * yb * betayb + ytau
     * betaytau);
  
  // third generation
  double mstoprsq = ONEO16pif * sqr(m32) * 
    (-88. / 25. * sqr(sqr(g1)) + 8.0 * sqr(sqr(g3)) + 2.0 * yt * betayt);
  double msbottomrsq = ONEO16pif * sqr(m32) * 
    (-22. / 25. * sqr(sqr(g1)) + 8.0 * sqr(sqr(g3)) + 2.0 * yb * betayb);
  double mQl3sq = ONEO16pif * sqr(m32) * 
    (-11. / 50. * sqr(sqr(g1)) - 1.5 * sqr(sqr(g2)) + 8.0 * sqr(sqr(g3)) + yb *
     betayb + yt * betayt);
  
  double mL3sq = ONEO16pif * sqr(m32) * 
    (-99. / 50. * sqr(sqr(g1)) - 1.5 * sqr(sqr(g2)) + ytau
     * betaytau);
  double mstaursq = ONEO16pif * sqr(m32) * 
    (-198. / 25. * sqr(sqr(g1)) + 2.0 * ytau * betaytau);
  
  // other generations
  double mursq = ONEO16pif * sqr(m32) * 
    (-88. / 25. * sqr(sqr(g1)) + 8.0 * sqr(sqr(g3)));
  double mdrsq = ONEO16pif * sqr(m32) * 
    (-22. / 25. * sqr(sqr(g1)) + 8.0 * sqr(sqr(g3)));
  double mQlsq = ONEO16pif * sqr(m32) * 
    (-11. / 50. * sqr(sqr(g1)) - 1.5 * sqr(sqr(g2)) + 8.0 * sqr(sqr(g3)));
  double mLsq = ONEO16pif * sqr(m32) * 
    (-99. / 50. * sqr(sqr(g1)) - 1.5 * sqr(sqr(g2)));
  double mersq = ONEO16pif * sqr(m32) * 
    (-198. / 25. * sqr(sqr(g1)));
  
  mQLsq(1, 1) = mQLsq(1, 1) + mQlsq;
  mQLsq(2, 2) = mQLsq(2, 2) + mQlsq;
  mQLsq(3, 3) = mQLsq(3, 3) + mQl3sq;
  mURsq(1, 1) = mURsq(1, 1) + mursq;
  mURsq(2, 2) = mURsq(2, 2) + mursq;
  mURsq(3, 3) = mURsq(3, 3) + mstoprsq;
  mDRsq(1, 1) = mDRsq(1, 1) + mdrsq;
  mDRsq(2, 2) = mDRsq(2, 2) + mdrsq;
  mDRsq(3, 3) = mDRsq(3, 3) + msbottomrsq;
  mLLsq(1, 1) = mLLsq(1, 1) + mLsq;
  mLLsq(2, 2) = mLLsq(2, 2) + mLsq;
  mLLsq(3, 3) = mLLsq(3, 3) + mL3sq;
  mSEsq(1, 1) = mSEsq(1, 1) + mersq;
  mSEsq(2, 2) = mSEsq(2, 2) + mersq;
  mSEsq(3, 3) = mSEsq(3, 3) + mstaursq;
  
  mH1sq = mH1sq + mhdsq;
  mH2sq = mH2sq + mhusq;
  
  // For calculational brevity: 
  static sBrevity a;
  static MssmSusy dsb;
  
  // calculate derivatives for full SUSY spectrum. Brevity calculations come
  // out encoded in a
  dsb = MssmSusy::beta(a);
  
  // AMSB spectrum
  DoubleVector amsbGaugino(3);
  DoubleMatrix temp(3, 3);
  
  int i; for (i=1; i<=3; i++)
    amsbGaugino(i) = dsb.displayGauge()(i) * m32 / run.displayGauge()(i);
  mGaugino = mGaugino + amsbGaugino;
  
  ua = ua - dsb.displayYukawaMatrix(YU) * m32;
  da = da - dsb.displayYukawaMatrix(YD) * m32;
  ea = ea - dsb.displayYukawaMatrix(YE) * m32;
  
  return;
}

// Reads in universal boundary conditions at the current scale:
// m0, M1/2, A0, B and sign of mu
void SoftParsMssm::universal(double m0,  double m12,  double a0,  double mu,
			      double b) {
  standardSugra(m0, m12, a0);  
  setSusyMu(mu);
  setB(b);
}

void SoftParsMssm::universalScalars(double m0) {
  // scalar masses
  DoubleMatrix ID(3, 3), mm0(3, 3);
  int i; for (i=1; i<=3; i++) ID(i, i) = 1.0;
  mm0 = ID * sqr(m0);
  setSoftMassMatrix(mQl, mm0); setSoftMassMatrix(mUr, mm0);
  setSoftMassMatrix(mDr, mm0); setSoftMassMatrix(mLl, mm0);
  setSoftMassMatrix(mEr, mm0);
  setMh1Squared(sqr(m0)); setMh2Squared(sqr(m0));
}

void SoftParsMssm::universalGauginos(double m12) {  
  // gaugino masses
  int i; for (i=1; i<=3; i++) setGauginoMass(i, m12);
}

void SoftParsMssm::universalTrilinears(double a0)  {  
  // trilinears
  setTrilinearMatrix(UA, a0 * displayYukawaMatrix(YU)); 
  setTrilinearMatrix(DA, a0 * displayYukawaMatrix(YD));
  setTrilinearMatrix(EA, a0 * displayYukawaMatrix(YE));
}

// Input m0, NOT m0 squared.
void SoftParsMssm::standardSugra(double m0,  double m12, double a0) {
  if (m0 < 0.0) {
    cout << "m0^2=" << m0 << " passed to universal boundary" <<
      "conditions illegally negative.";
    exit(1);
  }
  universalScalars(m0);
  universalGauginos(m12);
  universalTrilinears(a0);
}

#define HR "---------------------------------------------------------------\n"

ostream & operator <<(ostream &left, const SoftParsMssm &s) {
  left << HR;
  left << "SUSY breaking MSSM parameters\n";
  left << " UA" << s.displayTrilinear(UA) 
       << " UD" << s.displayTrilinear(DA) 
       << " UE" << s.displayTrilinear(EA); 
  left << " mQLsq" << s.displaySoftMassSquared(mQl) 
       << " mURsq" << s.displaySoftMassSquared(mUr) 
       << " mDRsq" << s.displaySoftMassSquared(mDr) 
       << " mLLsq" << s.displaySoftMassSquared(mLl) 
       << " mSEsq" << s.displaySoftMassSquared(mEr);
  left << "B: " << s.displayB() << " mH1sq: " << s.displayMh1Squared() << 
    " mH2sq: " << s.displayMh2Squared() << '\n';
  left << "Gaugino masses" << s.displayGaugino();
  left << s.displaySusy();
  return left;
}

#undef HR

istream & operator >>(istream &left, SoftParsMssm &s) {
  char c[70];

  left >> c >> c >> c >> c >> c;
  DoubleMatrix ua(3, 3), da(3, 3), ea(3, 3);
  left >> c >> ua
       >> c >> da
       >> c >> ea;
  s.setTrilinearMatrix(UA, ua);
  s.setTrilinearMatrix(DA, da);
  s.setTrilinearMatrix(EA, ea);
  DoubleMatrix mqlsq(3, 3), mursq(3, 3), mdrsq(3, 3), mllsq(3, 3), mersq(3, 3);
  left >> c >> mqlsq
       >> c >> mursq
       >> c >> mdrsq
       >> c >> mllsq
       >> c >> mersq;
  s.setSoftMassMatrix(mQl, mqlsq); 
  s.setSoftMassMatrix(mUr, mursq); 
  s.setSoftMassMatrix(mDr, mdrsq); 
  s.setSoftMassMatrix(mLl, mllsq); 
  s.setSoftMassMatrix(mEr, mersq); 
  double B, mh1sq, mh2sq;
  left >> c >> B >> c >> mh1sq >> c >> mh2sq;
  s.setB(B); s.setMh1Squared(mh1sq); s.setMh2Squared(mh2sq);
  DoubleVector mg(3);
  left >> c >> mg; 
  s.setAllGauginos(mg);
  MssmSusy ss;
  left >> ss;   s.setSusy(ss);
  return left;
}

