
/* 
   Project:     SOFTSUSY 1.2
   File:        softsusy.cpp
   Author:      Ben Allanach 
   Manual:      hep-ph/0104145 
   Webpage:     http://allanach.home.cern.ch/allanach/softsusy.html

   $Log: softsusy.cpp,v $
   Revision 1.7  2001/07/31 10:40:11  allanach
   Bug fix: radiative corrections to tachyons are now trapped

   Revision 1.6  2001/07/30 14:08:47  allanach
   Added ISAWIG and SSRUN interface

   Revision 1.5  2001/07/26 14:20:22  allanach
   Added isajet interface

   Revision 1.4  2001/07/18 14:42:51  allanach
   Added proper header info

   */

#include "softsusy.h"

const static double sw2 = 1.0 - sqr(MW / MZ),
  guL = 0.5 - 2.0 * sw2 / 3.0, 
  gdL = -0.5 + sw2 / 3.0,
  geL = -0.5 + sw2,
  guR = 2.0 * sw2 / 3.0, 
  gdR = -sw2 / 3.0,
  geR = -sw2; 

const MssmSoftsusy & MssmSoftsusy::operator=(const MssmSoftsusy & s) {
  if (this == &s) return *this;
  setSoftPars(s.displaySoftPars());
  physpars = s.physpars;
  problem = s.problem;
  m32 = s.m32;
  minV = s.minV;
  higgsVevMs = s.higgsVevMs;
  dataSet = s.dataSet;
  setMu(s.displayMu());
  setLoops(s.displayLoops());
  setThresholds(s.displayThresholds());
  return *this;
}

// Returns mu from rewsb requirement (TREE LEVEL is default, without deltaT2)
// returns 1 if there's a problem. Call at MSusy
int MssmSoftsusy::rewsbMu(int sgnMu, double & mu) const {
  int flag = 0;
  if (fabs(sgnMu) != 1.0) {
    cerr << "Error: sign mu = " << sgnMu << "\n";
    exit(1);
  }
  double mH1sq = displayMh1Squared(), mH2sq = displayMh2Squared(), tanb =
    displayTanb(), tanb2 =  sqr(tanb);
  
  // One-loop relation
  double musq = (mH1sq - mH2sq * tanb2) / (tanb2 - 1.0) - 0.5 * sqr(MZ);
  
  if (musq < 0.0) {
    mu = sgnMu * sqrt(fabs(musq)); flag = 1; // mu has incorrect sign
  }
  else
    mu = sgnMu * sqrt(musq); 
  return flag;
}

// Returns B from rewsb requirement (TREE LEVEL is default if no deltaT2)
// returns 1 if mu < 1.0e-9
int MssmSoftsusy::rewsbB(double mu, double & b) const {
  int flag = 0;

  if (fabs(mu) < 1.0e-9) 
    { flag = 1; b = 0.0; }
  else
    b =  0.5 * 
	 (displayMh1Squared() + displayMh2Squared() - displayTadpole2Ms() -
	  displayTadpole1Ms() + 2.0 * sqr(mu)) * 
	 sin(2 * atan(displayTanb())) /
      mu; 
  
  // Following means no good rewsb
  if (b * mu < 0.0) flag = 1;
  
  return flag;
}

// Predicts tan beta once mu and soft terms are predicted at low energy
// Useful for fine-tuning calculation. Call at MSusy only.
double MssmSoftsusy::predTanb() const  {
  double sin2t = 2.0 * displaySusyMu() * displayB() / 
    (displayMh1Squared() - displayTadpole1Ms() + 
     displayMh2Squared() - displayTadpole2Ms() + 2.0 *
     sqr(displaySusyMu())); 
  
  // Note: we want to take inverse sine so that fundamental domain is greater
  // than pi/4. sin(pi - 2 beta)=sin 2 beta should achieve this.
  // we also use tan (pi/2 - theta) = 1/tan(theta)
  double theta;
  if (fabs(sin2t) < 1.0) theta = asin(sin2t) * 0.5;
  else return 0.0;
  
  return 1.0 / tan(theta);
}

// Goes into mSUGRA
// From hep-ph/9606211's appendix. It should be done at MSusy to minimize the
// 1-loop contributions. Only call if you've calculated physpars
// inputs are running top/bottom masses
// Call at MSusy only
void MssmSoftsusy::setTadpole1Ms(double mt, double sinthDRbar) { // checked

  double g = displayGaugeCoupling(2), 
    costhDRbar = cos(asin(sinthDRbar)),
    tanb = displayTanb(), cosb = cos(atan(tanb)), 
    ht = displayYukawaElement(YU, 3, 3),
    mu = -displaySusyMu(), q = displayMu(),
    hb = displayYukawaElement(YD, 3, 3);
  
  // sneutrino coupling
  double lSnu = g * MZ / costhDRbar * 0.5 * cosb;
  
  // stop couplings
  DoubleMatrix lTS1Lr(2, 2), lTS112(2, 2), rotate(2, 2);
  lTS1Lr(1, 1) = g * MZ / costhDRbar * guL * cosb;
  lTS1Lr(1, 2) = ht * mu / sqrt(2.0);
  lTS1Lr(2, 1) = lTS1Lr(1, 2);
  lTS1Lr(2, 2) = g * MZ / costhDRbar * guR * cosb;
  rotate = rot2d(physpars.thetat);
  
  lTS112 = rotate * lTS1Lr * rotate.transpose();
  
  // sbottom couplings
  DoubleMatrix lBS1Lr(2, 2), lBS112(2, 2);
  double mA = physpars.mhiggs(2), mH0 = physpars.mhiggs(3), mHp =
    physpars.mhiggs(4);
  
  double beta = atan(displayTanb());
  double vev = displayHiggsVevMs();
  double mb = displayYukawaElement(YD, 3, 3) * vev * cos(beta) / sqrt(2.0);
  double mtau = displayYukawaElement(YE, 3, 3) * vev * cos(beta) / sqrt(2.0);

  lBS1Lr(1, 1) = g * MZ / costhDRbar * gdL * cosb + sqrt(2.0) * mb * hb;
  lBS1Lr(1, 2) = displayTrilinear(DA)(3, 3) / sqrt(2.0);
  lBS1Lr(2, 1) = lBS1Lr(1, 2);
  lBS1Lr(2, 2) = g * MZ / costhDRbar * gdR * cosb + sqrt(2.0) * mb * hb;
  
  rotate = rot2d(physpars.thetab);
  
  lBS112 = rotate * lBS1Lr * rotate.transpose();
  
  // stau couplings
  DoubleMatrix lTauS1Lr(2, 2), lTauS112(2, 2);
  double htau = displayYukawaElement(YE, 3, 3);
  
  lTauS1Lr(1, 1) = g * MZ / costhDRbar * geL * cosb + sqrt(2.0) * mtau *
    htau;
  lTauS1Lr(1, 2) = displayTrilinear(EA)(3, 3) / sqrt(2.0);
  lTauS1Lr(2, 1) = lTauS1Lr(1, 2);
  lTauS1Lr(2, 2) = g * MZ / costhDRbar * geR * cosb + sqrt(2.0) * mtau *
    htau;
  
  rotate = rot2d(physpars.thetatau);
  
  lTauS112 = rotate * lTauS1Lr * rotate.transpose();
  
  double delta = 0.0;
  
  // bottom quark, ignore others - factor (10^-2)^3 down
  delta = delta - 6.0 * sqr(displayYukawaElement(YD, 3, 3)) *
    a0(mb, q);

  double gO2mwcosb = g / (2.0 * MW * cosb);
  // third generation squarks
  delta = delta + 3.0 * gO2mwcosb * lTS112(1, 1) *
    a0(physpars.mu(1, 3), q);
  delta = delta + 3.0 * gO2mwcosb* lTS112(2, 2) *
    a0(physpars.mu(2, 3), q);
  delta = delta + 3.0 * gO2mwcosb * lBS112(1, 1) *
    a0(physpars.md(1, 3), q);
  delta = delta + 3.0 * gO2mwcosb * lBS112(2, 2) *
    a0(physpars.md(2, 3), q);
  // third generation sleptons
  delta = delta + gO2mwcosb * lTauS112(1, 1) *
    a0(physpars.me(1, 3), q);
  delta = delta + gO2mwcosb * lTauS112(2, 2) *
    a0(physpars.me(2, 3), q);
  
  delta = delta + gO2mwcosb * lSnu * a0(physpars.msnu(3), q);
  
  int family; 
  double gsqMZO2mwcw = g * MZ * gO2mwcosb * cosb / costhDRbar;
  // first two families of squarks
  for (family = 1; family <=2; family++) {
    delta = delta + 3.0 * gsqMZO2mwcw * 
      ( guL * a0(physpars.mu(1, family), q) + 
	guR * a0(physpars.mu(2, family), q)); 
    delta = delta + 3.0 * gsqMZO2mwcw * 
      ( gdL * a0(physpars.md(1, family), q) +
	gdR * a0(physpars.md(2, family), q));
  }
  
  // sleptons
  for (family = 1; family <=2; family++) {
    delta = delta + gsqMZO2mwcw * 
      (geL * a0(physpars.me(1, family), q) +
       geR * a0(physpars.me(2, family), q));
    delta = delta + gO2mwcosb * lSnu * a0(physpars.msnu(family), q);
  }
  
  // Higgs
  double alpha = displayPhys().thetaH, sina2 = sqr(sin(alpha)), 
    cosa2 = 1.0 - sina2, cos2b = cos(2.0 * atan(tanb)), 
    costhDRbar2 = sqr(costhDRbar), 
    mh = physpars.mhiggs(1), sin2a = sin(2.0 * alpha);
  delta = delta - sqr(g) * cos2b / (8.0 * costhDRbar2) *
    (a0(mA, q) + 2.0 * a0(mHp, q)) +
    sqr(g) * a0(mHp, q) * 0.5 +
    sqr(g) / (8.0 * costhDRbar2) * a0(mh, q) *
    (3.0 * sina2 - cosa2 + sin2a * tanb) +
    sqr(g) / (8.0 * costhDRbar2) * a0(mH0, q) *
    (3.0 * cosa2 - sina2 - sin2a * tanb); 
  
  // Neutralinos
  ComplexMatrix n(4, 4);
  DoubleVector mneut(4);
  mpzNeutralinos(n, mneut); // gets to Matchev+Pierce's conventions

  double tanthDRbar = tan(acos(costhDRbar));
  for (family = 1; family <= 4; family++)
    delta = delta - sqr(g) * mneut(family) / (MW * cosb) *
      (n(family, 3) * (n(family, 2) - n(family, 1) * tanthDRbar)).real() * 
      a0(mneut(family), q);
  
  // Charginos
  ComplexMatrix u(2, 2), v(2, 2); DoubleVector mch(2); 
  mpzCharginos(u, v, mch);
  for (family=1; family<=2; family++)
    delta = delta - sqrt(2.0) * sqr(g) / (MW * cosb)
      * mch(family) * (v(family, 1) * u(family, 2)).real()
	 * a0(mch(family), q);
  
  // Weak bosons
  delta = delta + 3.0 * sqr(g) / 4.0 * 
    (2.0 * a0(MW, q) + a0(MZ, q) / costhDRbar2)
    + sqr(g) * cos2b / (8.0 * costhDRbar2) * (2.0 * a0(MW, q) + a0(MZ, q));
  
  physpars.t1OV1Ms = delta / (16.0 * sqr(PI));
}

// From hep-ph/9311269's appendix. It should be done at MSusy to minimize the
// 1-loop contributions. Only call if you've calculated physpars
// inputs are running top/bottom masses. Call at MSusy
void MssmSoftsusy::setTadpole2Ms(double mt, double sinthDRbar) {// CHECKED
  if (physpars.mu(1, 3) == 0.0 || physpars.mu(2, 3) == 0.0) {
    cerr << "Trying to calculate tadpole without having first calculated";
    cerr << " the physical masses.\n";
    exit(1);
  }
  
  double g = displayGaugeCoupling(2), 
    costhDRbar = cos(asin(sinthDRbar)), 
    tanb = displayTanb(), 
    sinb = sin(atan(tanb)), 
    ht = displayYukawaElement(YU, 3, 3),
    mu = -displaySusyMu(), q = displayMu(),
    hb = displayYukawaElement(YD, 3, 3);
  
  // Stop couplings
  DoubleMatrix lTS2Lr(2, 2), lTS212(2, 2), rotate(2, 2);
  
  lTS2Lr(1, 1) = - g * MZ / costhDRbar * guL * sinb + sqrt(2.0) * ht * mt;
  lTS2Lr(1, 2) = displayTrilinear(UA)(3, 3) / sqrt(2.0);
  lTS2Lr(2, 1) = lTS2Lr(1, 2);
  lTS2Lr(2, 2) = - g * MZ / costhDRbar * guR * sinb + sqrt(2.0) * ht * mt;
  
  rotate = rot2d(physpars.thetat);
  
  lTS212 = rotate * lTS2Lr * rotate.transpose();
  
  // sneutrino coupling
  double lSnu = - g * MZ / costhDRbar * 0.5 * sinb;
  
  // sbottom couplings
  DoubleMatrix lBS2Lr(2, 2), lBS212(2, 2);
  double mA = physpars.mhiggs(2), mH0 = physpars.mhiggs(3), mHp =
    physpars.mhiggs(4);
  
  lBS2Lr(1, 1) = - g * MZ / costhDRbar * gdL * sinb;
  lBS2Lr(1, 2) = hb * mu / sqrt(2.0);
  lBS2Lr(2, 1) = lBS2Lr(1, 2);
  lBS2Lr(2, 2) = - g * MZ / costhDRbar * gdR * sinb;
  
  rotate = rot2d(physpars.thetab);
  
  lBS212 = rotate * lBS2Lr * rotate.transpose();
  
  // stau couplings
  DoubleMatrix lTauS2Lr(2, 2), lTauS212(2, 2);
  double htau = displayYukawaElement(YE, 3, 3);
  
  lTauS2Lr(1, 1) = - g * MZ / costhDRbar * geL * sinb;
  lTauS2Lr(1, 2) = htau * mu / sqrt(2.0);
  lTauS2Lr(2, 1) = lTauS2Lr(1, 2);
  lTauS2Lr(2, 2) = - g * MZ / costhDRbar * geR * sinb;
  
  rotate = rot2d(physpars.thetatau);
  
  lTauS212 = rotate * lTauS2Lr * rotate.transpose();
  
  double delta = 0.0;
  
  // top quark, ignore others - factor (10^-2)^3 down
  delta = delta - 6.0 * sqr(displayYukawaElement(YU, 3, 3)) *
    a0(mt, displayMu());
  // third generation squarks
  delta = delta + 3.0 * g * lTS212(1, 1) / (2.0 * MW * sinb) *
    a0(physpars.mu(1, 3), q);
  delta = delta + 3.0 * g * lTS212(2, 2) / (2.0 * MW * sinb) *
    a0(physpars.mu(2, 3), q);
  delta = delta + 3.0 * g * lBS212(1, 1) / (2.0 * MW * sinb) *
    a0(physpars.md(1, 3), q);
  delta = delta + 3.0 * g * lBS212(2, 2) / (2.0 * MW * sinb) *
    a0(physpars.md(2, 3), q);
  // third generation sleptons
  delta = delta + g * lTauS212(1, 1) / (2.0 * MW * sinb) *
    a0(physpars.me(1, 3), q);
  delta = delta + g * lTauS212(2, 2) / (2.0 * MW * sinb) *
    a0(physpars.me(2, 3), q);
  delta = delta + g * lSnu / (2.0 * MW * sinb) * a0(physpars.msnu(3), q);
  
  int family; 
  // first two families of squarks
  for (family = 1; family <=2; family++)
    {
      delta = delta + 3.0 * g * 
	(- g * MZ / costhDRbar * guL * sinb * a0(physpars.mu(1, family), q) 
	 - g * MZ / costhDRbar * guR * sinb * a0(physpars.mu(2, family), q)) 
	/ (2.0 * MW * sinb);
      delta = delta + 3.0 * g * 
	(- g * MZ / costhDRbar * gdL * sinb * a0(physpars.md(1, family), q) 
	 - g * MZ / costhDRbar * gdR * sinb * a0(physpars.md(2, family), q)) 
	/ (2.0 * MW * sinb);	
    }
  
  // sleptons
  for (family = 1; family <=2; family++)
    {
      delta = delta + g * 
	(- g * MZ / costhDRbar * geL * sinb * a0(physpars.me(1, family), q) 
	 - g * MZ / costhDRbar * geR * sinb * a0(physpars.me(2, family), q)) 
	/ (2.0 * MW * sinb);
      delta = delta + g * lSnu * a0(physpars.msnu(family), q)
	/ (2.0 * MW * sinb);
    }
  
  // Higgs
  double alpha = displayPhys().thetaH, sina2 = sqr(sin(alpha)), cosa2 = 1.0 -
    sina2, cos2b = cos(2.0 * atan(tanb)), costhDRbar2 = sqr(costhDRbar), 
    mh = physpars.mhiggs(1), sin2a = sin(2.0 * alpha);
  delta = delta + sqr(g) * cos2b / (8.0 * costhDRbar2) *
    (a0(mA, displayMu()) + 2.0 * a0(mHp, q)) +
    sqr(g) * a0(mHp, q) * 0.5 +
    sqr(g) / (8.0 * costhDRbar2) * a0(mh, q) *
    (3.0 * cosa2 - sina2 + sin2a / tanb) +
    sqr(g) / (8.0 * costhDRbar2) * a0(mH0, q) *
    (3.0 * sina2 - cosa2 - sin2a / tanb);
  
  // Neutralinos
  ComplexMatrix n(4, 4); DoubleVector mneut(4); 
  mpzNeutralinos(n, mneut); // gets to Matchev+Pierce's conventions
  double tanthDRbar = tan(acos(costhDRbar));
  for (family = 1; family <= 4; family++)
    delta = delta + sqr(g) * mneut(family) / (MW * sinb) *
      (n(family, 4) * (n(family, 2) - n(family, 1) * tanthDRbar)).real() * 
      a0(mneut(family), q); 
  
  // Charginos
  ComplexMatrix u(2, 2), v(2, 2); DoubleVector mch(2); 
  mpzCharginos(u, v, mch);
  for (family = 1; family <= 2; family++)
    delta = delta - sqrt(2.0) * sqr(g) * mch(family) / (MW * sinb)
      * (v(family, 2) * u(family, 1)).real() * a0(mch(family), q);
  
  // Weak bosons
  delta = delta + 3.0 * sqr(g) / 4.0 * 
    (2.0 * a0(MW, q) + a0(MZ, q) / costhDRbar2)
    - sqr(g) * cos2b / (8.0 * costhDRbar2) * (2.0 * a0(MW, q) + a0(MZ, q));
  
  physpars.t2OV2Ms = delta / (16.0 * sqr(PI));
}

// Apply at scale MSusy: checked 19.12.2000
// Displays PHYSICAL MZ, ie MZ(q) - piZz^T(q)
double MssmSoftsusy::predMzsq(double & tanb) const {
  tanb = predTanb(); 
  
  double MZsq = 2.0 *
    ((displayMh1Squared() - displayTadpole1Ms() - 
      (displayMh2Squared() - displayTadpole2Ms()) *
      sqr(tanb)) / 
     (sqr(tanb) - 1.0) - sqr(displaySusyMu())) - piZZT(MZ, displayMu());

  return MZsq;
}

// Used to get useful information into ftCalc
static MssmSoftsusy *tempSoft1;
static int ftFunctionality;
static DoubleVector ftPars(3); 
void (*ftBoundaryCondition)(MssmSoftsusy &, const DoubleVector &);

double ftCalc(double x) {
  // Stores running parameters in a vector
  DoubleVector storeObject(tempSoft1->display());
  double initialMu = tempSoft1->displayMu();
  sPhysical savePhys(tempSoft1->displayPhys());

  if (PRINTOUT > 1) cout << "#";
  
  if (ftFunctionality <= ftPars.displayEnd()) {
    ftPars(ftFunctionality) = x;
    ftBoundaryCondition(*tempSoft1, ftPars);
    if (PRINTOUT > 1) cout << "p" << ftFunctionality << "=";
  }
  else if (ftFunctionality == ftPars.displayEnd() + 1) { 
    tempSoft1->setSusyMu(x); if (PRINTOUT > 1) cout << "mu= "; 
  }
  else if (ftFunctionality == ftPars.displayEnd() + 2) { 
    tempSoft1->setB(x); if (PRINTOUT > 1) cout << "B= "; 
  } 
  else if (ftFunctionality == ftPars.displayEnd() + 3) {
    tempSoft1->setYukawaElement(YU, 3, 3, x); if (PRINTOUT > 1) cout << "ht= ";
  }
  else {
    cout << "MssmSoftsusy:ftCalc called with incorrect functionality=" <<
      ftFunctionality << endl;
    exit(1);
  }
  
  double referenceMzsq, predTanb;
  
  tempSoft1->runto(tempSoft1->calcMs());
  tempSoft1->physical(3);
  
  // Strictly speaking, we are using a top mass that is NOT consistent with
  // the new set of sparticle masses. To use the correct one is however quite
  // a pain. We take it to suffice...
  tempSoft1->setHiggsVevMs(tempSoft1->getVev());
  double mt = tempSoft1->calcRunningMt();
  double sinthDRbar = tempSoft1->calcSinthdrbar();
  tempSoft1->setTadpole2Ms(mt, sinthDRbar); 
  tempSoft1->setTadpole1Ms(mt, sinthDRbar);
  
  referenceMzsq = tempSoft1->predMzsq(predTanb);  
  
  if (PRINTOUT > 1) cout << x << " MZ=" << sqrt(fabs(referenceMzsq)) 
			 << " tanb=" << predTanb << "\n";

  // Restore initial parameters at correct scale
  tempSoft1->setMu(initialMu);
  tempSoft1->set(storeObject);
  tempSoft1->setPhys(savePhys);

  return referenceMzsq;
}

// Give it a GUT scale object consistent with rewsb
// and it'll return the fine tuning by varying m32, mu and B at the high scale
double MssmSoftsusy::it1par(int numPar, const DoubleVector & bcPars) {
  
  double ftParameter = 0.0, err, h = 0.01;
  
  tempSoft1 = this;

  // Stores running parameters in a vector
  DoubleVector storeObject(display());
  double initialMu = displayMu();
  sPhysical savePhys(displayPhys());
  
  ftFunctionality = numPar;
  double x;
  
  // Defines starting value to calculate derivative from
  if (numPar > 0 && numPar <= bcPars.displayEnd()) {
    x = bcPars.display(numPar); h = 0.01 * x; 
  }
  else if (numPar == bcPars.displayEnd() + 1) {//mu
    x = displaySusyMu(); h = 0.01 * x; 
  }
  else if (numPar == bcPars.displayEnd() + 2) {//B
    x = displayB(); h = 0.01 * x; 
  }
  else if (numPar == bcPars.displayEnd() + 3) {//ht  
    x = displayYukawaMatrix(YU)(3, 3); h = x * 0.0005; 
  }
  else {
    cout << "it1par called with functionality " << ftFunctionality << 
      " out of range.\n";
    exit(1);
  }
  
  ftPars.setEnd(bcPars.displayEnd()); ftPars = bcPars;
  
  // Fine tuning is a / MZ^2 d MZ^2 / d a for any parameter a
  double derivative;
  if (fabs(x) < 1.0e-10) {
    ftParameter = 0.0; derivative = 0.0; err = 0.0;
  }
  else {
    derivative = calcDerivative(ftCalc, x, h, &err);
    ftParameter = x * derivative / sqr(MZ);
  }
  
  if (PRINTOUT > 1)
    cout << "derivative=" << derivative << " error=" << err << "\n";
  
  // High error: if can't find a derivative, error comes back with 1.0e30
  if (ftParameter > EPS && fabs(err / derivative) > 1.0) 
    return 6.66e66;

  // Restore initial parameters at correct scale
  setMu(initialMu);
  set(storeObject);
  setPhys(savePhys);

  return fabs(ftParameter);
}

// Pass it an object and it'll return the fine tuning parameter for
// dilaton domination (or NaN in the case of difficulties)
DoubleVector MssmSoftsusy::fineTune
(void (*boundaryCondition)(MssmSoftsusy &, const DoubleVector &),
 const DoubleVector  & bcPars, double mx) {

  // Stores running parameters in a vector
  DoubleVector storeObject(display());
  double initialMu = displayMu();
  sPhysical savePhys(displayPhys());
  
  runto(mx);

  ftBoundaryCondition = boundaryCondition;
  
  int numPars = bcPars.displayEnd() + 3;
  
  DoubleVector tempFineTuning(numPars);
  
  int i; for (i = 1; i<= numPars; i++) {
    tempFineTuning(i) = it1par(i, bcPars);
    // flag problem FT calculation with NaN...too inaccurate
    if (tempFineTuning(i) > 1.0e66) tempFineTuning(i) = asin(2.0);
  }
  
  // Restore initial parameters at correct scale
  setMu(initialMu);
  set(storeObject);
  setPhys(savePhys);

  return tempFineTuning;
}

// Obtains solution of one-loop effective potential minimisation via iteration
// technique
// err is 1 if no iteration reached
// 2 if incorrect rewsb
void MssmSoftsusy::iterateMu(double & muold, int sgnMu,
			     double mt, int maxTries, double pizzMS,
			     double sinthDRbar, double tol, int & err) {
  static int numTries = 0;
  static double munew = 0.0;
  
  if (numTries - 1 > maxTries) { 
    if (PRINTOUT) cout << "iterateMu reached maxtries\n"; 
    numTries = 0; munew = 0.0;
    err = 1; return;
  }
  // How close to convergence are we?
  double c = 1.0 - minimum(fabs(muold), fabs(munew)) / 
    maximum(fabs(muold), fabs(munew));
  
  if (c < tol) { 
    muold = munew;
    numTries = 0; munew = 0.0;
    if (PRINTOUT > 2) cout << " mu converged\n";
    return; 
  }
  
  numTries = numTries + 1;
  
  muold = munew;
  
  double mH1sq = displayMh1Squared(), mH2sq = displayMh2Squared(), 
    tanb = displayTanb(), tanb2 = sqr(tanb);
  
  double treeLevelMusq = (mH1sq - mH2sq * tanb2) / (tanb2 - 1.0) - 0.5 *
    sqr(MZ);
  
  setSusyMu(munew);
  
  // calculate the new one-loop tadpoles with old value of mu
  setTadpole2Ms(mt, sinthDRbar);
  setTadpole1Ms(mt, sinthDRbar);   
  
  double oneLoopMusq = treeLevelMusq - 0.5 * pizzMS + 
    (displayTadpole2Ms() * sqr(tanb) - displayTadpole1Ms()) /
    (sqr(tanb) - 1.0); 
  
  // Error in rewsb
  if (oneLoopMusq < 0.0) {
    err = 2; 
    if (PRINTOUT > 1) cout << " mu^2<0 ";
  }
  
  munew = sgnMu * sqrt(fabs(oneLoopMusq));

  if (PRINTOUT > 2) cout << " mu=" << munew;
  
  iterateMu(muold, sgnMu, mt, maxTries, pizzMS, sinthDRbar, tol, err);
}

void MssmSoftsusy::rewsbTreeLevel(int sgnMu) {
  double mu, b;
  if (rewsbMu(sgnMu, mu)) flagMusqwrongsign(true);
  else flagMusqwrongsign(false);
  setSusyMu(mu);
  if (rewsbB(mu, b)) flagB(true);
  else flagB(false); 
  
  setB(b);
  
  if ((displayMh1Squared() + 2.0 * sqr(displaySusyMu()) +
       displayMh2Squared() - 2.0 * fabs(displaySusyMu() *
					  displayB())) < 0.0 )
    flagHiggsufb(true);
  else 
    flagHiggsufb(false);
  
  return;
}

// Organises rewsb: call it at the low scale MS^2 = sqrt(0.5 * (mT1^2 +
// mT2^2)) is best, or below if it's decoupled from there. 
// Call with zero, or no mt if you want tree level
void MssmSoftsusy::rewsb(int sgnMu, double mt) {
  double munew, bnew;
  
  double sinthDRbarMS = calcSinthdrbar();

  setTadpole1Ms(mt, sinthDRbarMS);  
  setTadpole2Ms(mt, sinthDRbarMS); 
  
  munew = displaySusyMu();
  
  // Iterate to get a self-consistent mu solution
  int maxTries = 20, err = 0;
  double tol = EPS * 1.0e-6;
  
  setHiggsVevMs(getVev()); 
  
  double pizztMS = piZZT(MZ, displayMu());
  iterateMu(munew, sgnMu, mt, maxTries, pizztMS, sinthDRbarMS,
	    tol, err); 
  
  if (err == 2) flagMusqwrongsign(true);
  else flagMusqwrongsign(false); 
  if (err == 1) flagNoMuConvergence(true);
  else
    setSusyMu(munew);
  
  if (rewsbB(munew, bnew) == 0) flagB(false);
  else flagB(true);
  
  setB(bnew);
  
  if ((displayMh1Squared() + 2.0 * sqr(displaySusyMu()) +
       displayMh2Squared() - 2.0 * fabs(displaySusyMu() *
					  displayB())) < 0.0 )
    flagHiggsufb(true);
  else 
    flagHiggsufb(false);
}

#define HR "----------------------------------------------------------\n"

ostream & operator <<(ostream &left, const MssmSoftsusy &s) {
  left << HR;
  left << "SUSY breaking MSSM parameters at Q=" << s.displayMu() << endl;
  left << " UA" << s.displayTrilinear(UA) 
       << " UD" << s.displayTrilinear(DA) 
       << " UE" << s.displayTrilinear(EA); 
  left << " mQLsq" << s.displaySoftMassSquared(mQl) 
       << " mURsq" << s.displaySoftMassSquared(mUr) 
       << " mDRsq" << s.displaySoftMassSquared(mDr) 
       << " mLLsq" << s.displaySoftMassSquared(mLl) 
       << " mSEsq" << s.displaySoftMassSquared(mEr);
  left << "B: " << s.displayB() << " mH1sq: " << s.displayMh1Squared() << 
    " mH2sq: " << s.displayMh2Squared() << '\n';
  left << "Gaugino masses" << s.displayGaugino();
  left << "Gravitino mass M3/2: " << s.displayGravitino() << endl;
  left << s.displayPhys();
  left << "Higgs VEV: " << s.displayHiggsVevMs() << endl;  
  left << "Data set:\n" << s.displayDataSet();
  double mass; int posi, posj, id;
  id = s.lsp(mass, posi, posj);
  left << "lsp is ";
  recogLsp(id, posj);
  left << " of mass " << mass << " GeV\n";
  left << s.displaySusy();
  left << s.displayProblem();
  return left;
}

#undef HR

istream & operator >>(istream &left, MssmSoftsusy &s) {
  char c[70];
  left >> c >> c >> c >> c >> c;
  DoubleMatrix ua(3, 3), da(3, 3), ea(3, 3);
  left >> c >> ua
       >> c >> da
       >> c >> ea;
  s.setTrilinearMatrix(UA, ua);
  s.setTrilinearMatrix(DA, da);
  s.setTrilinearMatrix(EA, ea);
  DoubleMatrix mqlsq(3, 3), mursq(3, 3), mdrsq(3, 3), mllsq(3, 3), mersq(3, 3);
  left >> c >> mqlsq
       >> c >> mursq
       >> c >> mdrsq
       >> c >> mllsq
       >> c >> mersq;
  s.setSoftMassMatrix(mQl, mqlsq); 
  s.setSoftMassMatrix(mUr, mursq); 
  s.setSoftMassMatrix(mDr, mdrsq); 
  s.setSoftMassMatrix(mLl, mllsq); 
  s.setSoftMassMatrix(mEr, mersq); 
  double B, mh1sq, mh2sq;
  left >> c >> B >> c >> mh1sq >> c >> mh2sq;
  s.setB(B); s.setMh1Squared(mh1sq); s.setMh2Squared(mh2sq);
  DoubleVector mg(3);
  left >> c >> mg; 
  s.setAllGauginos(mg);
  double m32;
  left >> c >> c >> c >> m32; // m32 not supported at present
  sPhysical sp; double vev;
  left >> sp;
  s.setPhys(sp);
  left >> c >> c >> vev;  
  s.setHiggsVevMs(vev);
  QedQcd qq;
  left >> c >> c >> qq;
  s.setData(qq);
  left >> c >> c >> c >> c >> c >> c >> c; 
  MssmSusy ss;
  left >> ss;   s.setSusy(ss);
  return left;
}

// Gives a summary of important properties of a SUSY object: mu, B, mH0,
// mChi0, lightest stau, lightest mGluino, lightest stop, lightest chargino,
// lightest tau sneutrino, lightest sbottom, down squark, up squark and
// selectron masses, minimum of potential (if calculated) and fine-tuning
// parameter (if passed)
void printShortInitialise() {
  cout <<
    "     mu     " << "     B      " << 
    "   mstau1   " << "   msbott   " << "  mstop1    " <<
    " mSquark   " << "   mSlep   " << "  msnu1     " <<
    "    mhl0    " << "   mchi10   " << " mchargino1 " <<
    "  mGluino  " << flush;
  }

// Prints mu, B and important spectral information
void MssmSoftsusy::printShort() const {
  
  const double problemFlag = -100.0;

  double mu, b;
  mu = displaySusyMu(); b = displayB(); 
  int pos;
  sPhysical s(this->displayPhys());
  
  if (displayProblem().muSqWrongSign || displayProblem().b ||
      displayProblem().higgsUfb || displayProblem().noRhoConvergence) 
    mu = -1.0 *  mu;
  if (displayProblem().tachyon) b = -1.0 * b;
  
  if (displayProblem().nonperturbative || displayProblem().noMuConvergence) {
    cout << " ";
    int i; for (i=1; i<=12; i++) cout << problemFlag << " ";
  }
    else {
      cout << " " << mu << " " << b << " "
	   << minimum(s.me(1, 3), s.me(2, 3)) << " " 
	   << minimum(s.md(1, 3), s.md(2, 3)) << " " 
	   << minimum(s.mu(1, 3), s.mu(2, 3)) << " " 
	 << minimum(minimum(s.md(1, 1), s.md(2, 1)), 
		    minimum(s.mu(1, 1), s.mu(2, 1)))
	   << " " << minimum (s.me(1, 1), s.me(2, 1)) << " "
	   << s.msnu.min(pos) << " " 
	   << s.mhiggs(1) << " " 
	   << (s.mneut.apply(fabs)).min(pos) << " "
	   << (s.mch.apply(fabs)).min(pos) << " "
	   << s.mGluino << " ";
    }
  
  cout << flush;
}

void MssmSoftsusy::printLong() {
  // output:
  //  1  2  3      4   5   6   7    8 9  10 11 12    13    14     
  // mu  B mH1sq mH2sq g1 g2 mt(mt) V mh mA mH mH+ alphaH msnu3
  //  15     16     17    18     19    20     21   
  // msnu1 mstopL mstopR msupL msupR msbotL msbotR
  //  22   23   24     25     26     27    28     29      30    
  // msdL msdR mstauL mstauR mselL mselR thetat thetab thetatau
  // 31   32   33   34      35     36    37     38     39      
  // mgl mch1 mch2 thetaL thetaR mneut1 mneut2 mneut3 mneut4
  //   40     41    42  
  // sinthW t1ov1 t2ov2 
  double mu = displaySusyMu();
  if (displayProblem().muSqWrongSign || displayProblem().b ||
      displayProblem().higgsUfb) 
    mu = -1.0 *  mu;
  //  if (displayProblem().noConvergence) 
  // mu = asin(-2.0);
  
  if (displayProblem().tachyon) {
    sPhysical s(displayPhys());
    s.me = -1.0 * s.me;
    s.md = -1.0 * s.md;
    s.mu = -1.0 * s.mu;
    s.msnu = -1.0 * s.msnu;	
    s.mhiggs = -1.0 * s.mhiggs;
    setPhys(s);
  }
  
  cout <<" " << mu << " " 
       << displayB() << " " 
       << displayMh1Squared() << " " <<
    displayMh2Squared() << " " << 
    displayGaugeCoupling(1) << " " <<
    displayGaugeCoupling(2) << " " <<
    calcRunningMt() << " " <<
    displayHiggsVevMs() << " " <<
    displayPhys().mhiggs(1) << " " <<
    displayPhys().mhiggs(2) << " " <<
    displayPhys().mhiggs(3) << " " <<
    displayPhys().mhiggs(4) << " " <<
    displayPhys().thetaH << " " <<
    displayPhys().msnu(3) << " " <<
    displayPhys().msnu(1) << " " <<
    displayPhys().mu(1, 3) << " " <<
    displayPhys().mu(2, 3) << " " <<
    displayPhys().mu(1, 1) << " " <<
    displayPhys().mu(2, 1) << " " <<
    displayPhys().md(1, 3) << " " <<
    displayPhys().md(2, 3) << " " <<
    displayPhys().md(1, 1) << " " <<
    displayPhys().md(2, 1) << " " <<
    displayPhys().me(1, 3) << " " <<
    displayPhys().me(2, 3) << " " <<
    displayPhys().me(1, 1) << " " <<
    displayPhys().me(2, 1) << " " <<
    displayPhys().thetat << " " <<
    displayPhys().thetab << " " <<
    displayPhys().thetatau << " " <<
    displayPhys().mGluino << " " <<
    displayPhys().mch(1) << " " <<
    displayPhys().mch(2) << " " <<
    displayPhys().thetaL << " " <<
    displayPhys().thetaR << " " <<
    displayPhys().mneut(1) << " " <<
    displayPhys().mneut(2) << " " <<
    displayPhys().mneut(3) << " " <<
    displayPhys().mneut(4) << " " <<
    calcSinthdrbar() << " " <<
    displayTadpole1Ms() << " " <<
    displayTadpole2Ms() << " ";

  if (displayProblem().test()) cout << "%" << displayProblem();
  cout << endl << flush;
}

// returns sqrt(f) for f>0 or -sqrt(|f|) for f<0
inline double ccbSqrt(double f){ return sqrt(fabs(f)); }

// checked: thetaH is defined to be the 'alpha' from Haber.
// following reproduces FeynHiggsFast.f results for the same inputs
// accuracy: 0=tree, 1=1 loop top corrections, 2=all one loop corrections,
// 3=up to two loop corrections
// mt inputted should be the POLE MASS, alphas should prob be at top scale
void MssmSoftsusy::higgs(int accuracy, double piwwtMS, double pizztMS) {
  double mt = displayDataSet().displayPoleMt(), 
    mb = displayDataSet().displayMass(mBottom), 
    alphas = displayDataSet().displayAlpha(ALPHAS); 
  const static double root2 = sqrt(2.0), pisq = sqr(PI);
  
  double tanb = displayTanb();
  double beta = atan(tanb), sinb = sin(beta), cosb = cos(beta);
  double sinb2 = sqr(sinb), cosb2 = sqr(cosb), mz2 = sqr(MZ), mt2 = sqr(mt),
    mt4 = sqr(mt2);
  double mu = displaySusyMu();
  
  // tree level: there'll be trouble if B has the opp sign to mu
  double mAsq = displayB() * mu / (sinb * cosb);
  if (mAsq < 0.0) {
    flagTachyon(true);
    if (PRINTOUT) cout << " mA^2(tree)=" << mAsq;
    mAsq = fabs(mAsq);
  }
  
  double costh = sqr(MW / MZ), sinth = 1 - costh; 
  DoubleMatrix mH(2, 2);
  mH(1, 1) = mAsq * sinb2 + mz2 * cosb2;
  mH(1, 2) = - sinb * cosb * (mAsq + mz2); 
  mH(2, 2) = mAsq * cosb2 + mz2 * sinb2; 
  mH(2, 1) = mH(1 ,2); 
  
  /*       one loop contributions:       */
  if (accuracy > 0) {
    // brevity: been checked
    double ms = 
      pow(displaySoftMassSquared(mQl)(3, 3) * displaySoftMassSquared(mUr)(3, 3)
	  + mt2 * (displaySoftMassSquared(mQl)(3, 3)
		   + displaySoftMassSquared(mUr)(3, 3)) +
	  sqr(mt2), 0.25), 
      ms2 = sqr(ms), ms4 = sqr(ms2), ms6 = ms4 * ms2, ms8 = sqr(ms4);
    double mtOMs2 = sqr(mt / ms), mtOMs4 = sqr(mtOMs2), mtOMs6 =
      mtOMs2 * mtOMs4, mtOMs8 = sqr(mtOMs4);
    double mzOMt2 = mz2 / mt2, mzOMt4 = sqr(mzOMt2);
    double logMtOMssq = log(mtOMs2);
    double mLR = displaySoftA(UA, 3, 3) - mu / tanb, mLR2 = sqr(mLR), mLR4
      = sqr(mLR2), mLR6 = mLR2 * mLR4, mLR8 = sqr(mLR4);
    const static double lambda = 1.0 / 8.0 - sinth / 3.0 + 4.0 / 9.0 *
      sqr(sinth);
    
    // Following are t,stop corrections from hep-ph/9903404: checked, but
    // note that mu and B have opposite signs to my notation from that paper
    DoubleMatrix sigma(2, 2);
    sigma(1, 1) = GMU * root2 * sqr(mz2) * lambda * cosb2 * logMtOMssq 
      / pisq;
    sigma(1, 2) = -GMU * root2 * mz2 / (pisq * tanb) * 
      (-3.0 / 8.0 * mt2 + mz2 * lambda * sinb2) * logMtOMssq;
    sigma(2, 2) = GMU * root2 / (8.0 * pisq * sinb2) * mt4 * 
      (-2.0 * mzOMt2 + 1.1 * mzOMt4 +
       (12.0 - 6.0 * mzOMt2 * sinb2 + 8.0 * mzOMt4 * lambda *
	sqr(sinb2)) * logMtOMssq + mLR2 / ms2 * 
       (-12.0 + 4.0 * mzOMt2 + 6.0 * mtOMs2) + mLR4 / ms4 * 
       (1.0 - 4.0 * mtOMs2 + 3.0 * mtOMs4) + mLR6 / ms6 * 
       (0.6 * mtOMs2 - 2.4 * mtOMs4 + 2.0 * mtOMs6) + mLR8 / ms8 * 
       (3.0 / 7.0 * mtOMs4 - 12.0 / 7.0 * mtOMs6 + 1.5 * mtOMs8)); 
    
    if (accuracy > 1) {
      // Following are non-t/stop corrections from hep-ph/9609331: checked
      double Pb, Pu, Pd, Pl, Pg, Pgp, P2h, P2hp, P1h, Pf;
      double sinth2 = sqr(sinth);
      Pb = 3.0 * (1.0 - 4.0 * sinth / 3.0 + 8.0 / 9.0 * sinth2);
      Pu = 3.0 * (2.0 - 4.0 * sinth  + 64.0 / 9.0 * sinth2);
      Pd = 3.0 * (2.0 - 4.0 * sinth  + 16.0 / 9.0 * sinth2);
      Pl = 3.0 * (2.0 - 4.0 * sinth + 8.0 * sinth2);
      Pf = Pu + Pd + Pl;
      Pg = -44.0 + 106.0 * sinth - 62.0 * sinth2;
      Pgp = 10.0 + 34.0 * sinth - 26.0 * sinth2;
      P2h = -10.0 + 2.0 * sinth - 2.0 * sinth2;
      P2hp = 8.0 - 22.0 * sinth + 10.0 * sinth2;
      double cos2b2 = sqr(cos(2.0 * beta)), cos2b4 = sqr(cos2b2);
      P1h = -9.0 * cos2b4 + (1.0 - 2.0 * sinth + 2.0 * sinth2) * cos2b2;
      
      // non bottom-mixing corrections: checked
      // Following was obtained from FeynHiggsFast.f
      double msusy2 = ms2 - mt2;
      double logMsScale = log(msusy2 / mz2);
      // Note if B undefined or zero, will have a problem!
      double logA0Scale = log(fabs(mAsq) / mz2);
      if (fabs(mAsq) < EPSTOL) logA0Scale = 0.0;
      double mz4 = sqr(mz2), cosb4 = sqr(cosb2), mb2 = sqr(mb), mb4 =
	sqr(mb2);
      double howie11 = root2 * GMU * cosb2 * mz4 / (24.0 * pisq);
      double howie12 = howie11 * tanb, howie22 = howie12 * tanb;
      
      sigma(1, 1) = sigma(1, 1) - howie11 * 
	(
	 (36.0 * mb4 / (mz4 * cosb4) - 18.0 * mb2 / (mz2 * cosb2) 
	  + Pb + Pf + Pg + P2h) * logMsScale 
	 + (P1h - P2h) * logA0Scale
	 );
      sigma(1, 2) = sigma(1, 2) + howie12 * 
	(
	 (- 9.0 * mb2 / (mz2 * cosb2) + Pb + Pf + Pgp + P2hp) 
	 * logMsScale - (P1h + P2hp) * logA0Scale
	 );
      sigma(2, 2) = sigma(2, 2) - howie22 * 
	(
	 (Pb + Pf + Pg + P2h) * logMsScale + (P1h - P2h) * logA0Scale
	 ); 
      
      // sbottom mixing corrections: checked
      double mblr = displaySoftA(DA, 3, 3) - mu * tanb, 
	mblr2 = sqr(mblr);
      double mu2 = sqr(mu);
      
      sigma(1, 1) = sigma(1, 1) -
	(root2 * GMU / PI * 3.0 * 
	 (-(mb2 / msusy2 * mz2 * (mLR + mu / tanb) * 
	    (mblr + (mLR + mu / tanb) / 3.0)) + 4.0 / cosb2 *
	  mb4 * mblr / msusy2 * (mLR + mu / tanb) * 
	  (1.0 - (mblr / msusy2 * (mLR + mu / tanb)) / 12.0))) / 
	(8.0 * PI);
      sigma(1, 2) = sigma(1, 2)
	+ ((root2 * GMU / PI * 3.0 * 
	    (-4.0 / cosb2 * mb4 * mblr / msusy2
	     * mu * (1.0 - (mblr / msusy2 * (mLR + mu / tanb)) / 6.0) -
	     mb2 / msusy2 * mz2 * (mblr * (mLR + 2.0 * mu / tanb ) + 
				   (mu2 + sqr(mLR + mu / tanb)) / 3.0) *
	     tanb)) / (16.0 * PI));
      sigma(2, 2) = sigma(2, 2) -
	(root2 * GMU / (8.0 * pisq) * 3.0 * 
	 (- ((1.0 / cosb2 * mb4 * mblr2 / sqr(msusy2) * mu2) / 3.0) - mb2
	  / msusy2
	  * mu * mz2 * tanb * (mblr + (mu * tanb) / 3.0))); 
      
      /*         two loop contributions: checked  from hep-ph/9903404   */
      if (accuracy > 2) {
	sigma(2, 2) = sigma(2, 2) - 
	  (root2 * GMU / (PI * pisq) * alphas * mt4 / sinb2 * 
	   ((6.0 * mLR) / ms - (12.0 * mLR4) / (17.0 * ms4) + 
	    (2.0 - (3.0 * mLR2) / ms2) * log(ms2 / mt2) - 
	    3.0 * sqr(log(ms2 / mt2))
	    - 4.0 + 8.0 * mLR2 / ms2));
      }      
    }
    sigma(2, 1) = sigma(1, 2);
    
    mH = mH - sigma;
  }
  
  DoubleVector temp(2);  
  double theta;
  
  temp = mH.sym2by2(theta);
  if (temp(1) < 0.0 || temp(2) < 0.0) flagTachyon(true);
  temp = temp.apply(ccbSqrt);
  
  // Definitions are such that theta should diagonalise the matrix like
  // O = [ cos  sin ]
  //     [-sin  cos ] 
  // and
  // O m O^T = [ m2^2      ]
  //           [      m1^2 ]
  // where m1 < m2, therefore if they come out in the wrong order, 
  // add pi/2 onto theta.
  if (temp(2) > temp(1)) { 
    theta = theta + PI * 0.5; 
    if (PRINTOUT > 3) {
      cout << "Changing alpha from " << theta - PI * 0.5;
      cout << " to " << theta << " resulting in:";
      DoubleMatrix O(rot2d(theta));
      cout << O * mH * O.transpose() << flush;
    }
  }
  
  physpars.mhiggs(1) = minimum(temp(2), temp(1));
  physpars.mhiggs(3) = maximum(temp(1), temp(2)); 
  
  double mA = displayPhys().mhiggs(2);
  double poleMasq = (displayMh2Squared() - displayMh1Squared() )
    / cos(2.0 * beta) - sqr(MZ);
  
  double piaa = 0.0;
  if (accuracy > 0)
    {
      piaa = piAA(mA, displayMu());
      poleMasq = 
	(displayMh2Squared() - displayTadpole2Ms() - displayMh1Squared() +
	 displayTadpole1Ms()) / cos(2.0 * beta) 
	- sqr(MZ) - pizztMS - piaa +
	sqr(sin(beta)) * displayTadpole1Ms() + sqr(cos(beta)) *
	displayTadpole2Ms();
    }

  // FUTURE WORK: could incorporate more accurate calculations on charged
  // Higgs from   hep-ph/9606211: include piH+H-
  double poleMhc = 
    poleMasq + sqr(MW) + piaa + piwwtMS;
  
  physpars.mhiggs(4) = ccbSqrt(poleMhc);
  physpars.mhiggs(2) = ccbSqrt(poleMasq);
  
  // two-loop Yukawa correction to mh^2: checked
  if (accuracy > 2) {
    double mstop1 = physpars.mu(1, 3);
    double mstop2 = physpars.mu(2, 3);
    double mstop1sq = sqr(mstop1), mstop2sq = sqr(mstop2);
    double t = 0.5 * log(mstop1sq * mstop2sq / mt4);
    double X = ( sqr( (mstop2sq - mstop1sq) / (4.0 * mt2) * 
		      sqr(sin(2.0 * physpars.thetat)) ) * 
		 (2.0 - (mstop2sq + mstop1sq) / (mstop2sq - mstop1sq) *
		  log(mstop2sq / mstop1sq)) +
		 (mstop2sq - mstop1sq) / (2.0 * mt2) * 
		 sqr(sin(2.0 * physpars.thetat)) * log(mstop2sq / mstop1sq)
		 );
    physpars.mhiggs(1) = sqrt(sqr(physpars.mhiggs(1)) + 9.0 / 
			      (16.0 * sqr(pisq)) *
			      sqr(GMU) * mt2 * mt4 * (X * t + sqr(t)));
  } 
  physpars.thetaH = theta;
}

void MssmSoftsusy::addCharginoLoop(DoubleMatrix & mass) {
  double g2 = displayGaugeCoupling(2), 
    ht = displayYukawaElement(YU, 3, 3),
    hb = displayYukawaElement(YD, 3, 3),
    q = displayMu(), mu = displaySusyMu(), M2 = displayGaugino()(2), 
    MA = physpars.mhiggs(2), tanb = displayTanb(); // brevity 
  
  // From hep-ph/9606211, approx formulae only, both checked! Results are a
  // bit inaccurate.... Note that a0,b0,b1 do NOT depend on the sign of the
  // masses/p input, so don't have to change sign of mu in these terms.

  // quark/squark loops
  double quarkSquarkM2 = - sqr(g2) / (16.0 * sqr(PI)) *
    (9.0 * b1(M2, 0.0, sqrt(displaySoftMassSquared(mQl)(1, 1)), q) +
     3.0 * b1(M2, 0.0, sqrt(displaySoftMassSquared(mLl)(1, 1)), q) -
     mu / M2 * sin(2.0 * atan(tanb)) *
     (b0(M2, mu, MA, q) - b0(M2, mu, MZ, q)) +
     b1(M2, mu, MA, q) + b1(M2, mu, MZ, q));

  // Gauge loops
  double gaugeBosonM2 = 
    sqr(g2) / (4.0 * sqr(PI)) * 
    (2.0 * b0(M2, M2, MW, q) - b1(M2, M2, MW, q));

  double deltaM2oM2 = quarkSquarkM2 + gaugeBosonM2;

  double deltaMuoMu = - 3.0 / (32.0 * sqr(PI)) *
    ((sqr(hb) + sqr(ht)) * 
     b1(mu, 0.0, sqrt(displaySoftMassSquared(mQl)(3, 3)), q) + 
     sqr(ht) * 
     b1(mu, 0.0, sqrt(displaySoftMassSquared(mUr)(3, 3)), q) + sqr(hb) * 
     b1(mu, 0.0, sqrt(displaySoftMassSquared(mDr)(3, 3)), q))
    - 3.0 * sqr(g2) / (64.0 * sqr(PI)) * 
    (b1(mu, M2, MA, q) + b1(mu, M2, MZ, q) + 2.0 * b1(mu, mu, MZ, q) - 4.0 *
     b0(mu, mu, MZ, q));
  
  mass(1, 1) = mass(1, 1) * (1.0 + deltaM2oM2);
  mass(2, 2) = mass(2, 2) * (1.0 + deltaMuoMu);
  
  // These are the proper corrections:
  /*
    double p = 100;
    DoubleMatrix sigmaL(2, 2);
    
    DoubleMatrix aPsicUp(2, 2), aPsicDown(2, 2), aPsicLep(2, 2);
    DoubleVector aPsicTop(2), aPsicBot(2), aPsicTau(2);
    DoubleVector aPsicTopm(2), aPsicBotm(2), aPsicTaum(2);
    aPsicTop(2) = -ht; aPsicBot(2) = -hb; aPsicTau(2) = -htau;
    int fam; for (fam=1; fam<=2; fam++)
    {
    aPsicDown(fam, 1) = g2;       aPsicLep(fam, 1) = g2; 
    }
    aPsicBot = g2; aPsicTau = g2;
    aPsicTopm = rot2d(thetat) * aPsicTop;
    aPsicBotm = rot2d(thetab) * aPsicBot;
    aPsicTaum = rot2d(thetatau) * aPsicTau;
    
    int i; for (i=1; i<=2; i++)
    {
    sigmaL = sigmaL + 
    1.5 * outerProduct(aPsicTopm, aPsicTopm) * 
    b1(p, mtop, physpars.mu(i, 3), q) + 
    1.5 * outerProduct(aPsicBotm, aPsicBotm) * 
    b1(p, mbot, physpars.md(i, 3), q) + 
    1.5 * outerProduct(aPsicTaum, aPsicTaum) * 
    b1(p, mtau, physpars.msnu(i, 3), q);
    }
    sigmaL(1, 1) = sigmaL(1, 1) + 
    1.5 * sqr(g) * 
    (b1(p, md, physpars.mu(1, 1), q) + b1(p, me, physpars.msnu(1), q) +
    b1(p, ms, physpars.mu(1, 2), q) + b1(p, mmu, physpars.msnu(2), q) +
    b1(p, mtau
    */
  
  
  
}

// checked
void MssmSoftsusy::charginos(int accuracy, double piwwtMS) {
  double tanb = displayTanb(), smu = displaySusyMu();
  DoubleMatrix mCh(2, 2);
  
  // Store current object in vector
  DoubleVector storeObject(display());
  double muInitial = displayMu();

  // In order to get M2(mch1), mu(mch2)
  if (physpars.mch(1) == 0.0) physpars.mch(1) = displayGaugino()(2);
  runto(displayGaugino()(2));
  double m2 = displayGaugino()(2); 
  runto(fabs(smu));
  smu = displaySusyMu();

  // restore object at initial scale.
  setMu(muInitial);
  set(storeObject);

  // tree level mass matrix
  mCh(1, 1) = m2;
  mCh(2, 1) = sqrt(2.0) * sqrt(sqr(MW) + piwwtMS) * cos(atan(tanb)); 
  mCh(1, 2) = mCh(2, 1) * tanb;
  mCh(2, 2) = smu;

  if (accuracy > 0) addCharginoLoop(mCh);

  physpars.mch = mCh.asy2by2(physpars.thetaL, physpars.thetaR);
}

double MssmSoftsusy::thet(double a, double b, double c = 0.0) {
  double yy = maximum(sqr(a), sqr(b));
  yy = maximum(yy, sqr(c));
  return log(yy / sqr(displayMu()));
}

void MssmSoftsusy::addNeutralinoLoop(DoubleMatrix & mass) {
  double gp = displayGaugeCoupling(1) * sqrt(0.6), 
    g2 = displayGaugeCoupling(2), ht = displayYukawaElement(YU, 3, 3), 
    hb = displayYukawaElement(YD, 3, 3),
    q = displayMu(), mu = -displaySusyMu(), M2 = displayGaugino()(2), M1 =
    displayGaugino()(1), MA = physpars.mhiggs(2), tanb = displayTanb();
  // brevity 
  
  double msq1 = sqrt(displaySoftMassSquared(mQl)(1, 1)), 
    mslep1 = sqrt(displaySoftMassSquared(mLl)(1, 1));
  
  // From hep-ph/9606211, approx formulae only, both checked!
  double deltaM1 = sqr(gp) / (16.0 * sqr(PI)) * 
    (11.0 * b1(M1, 0.0, msq1, q) + 9.0 * b1(M1, 0.0, mslep1, q) + mu / M1 * sin
     (2.0 * atan(tanb)) * (b0(M1, mu, MA, q) - b0(M1, mu, MZ, q)) 
     + b1(M1, mu, MA, q) + b1(M1, mu, MZ, q));
  
  double deltaM2 = - sqr(g2) / (16.0 * sqr(PI)) *
    (9.0 * b1(M2, 0.0, sqrt(displaySoftMassSquared(mQl)(1, 1)), q) +
     3.0 * b1(M2, 0.0, sqrt(displaySoftMassSquared(mLl)(1, 1)), q) +
     mu / M2 * sin(2.0 * atan(tanb)) *
     (b0(M2, mu, MA, q) - b0(M2, mu, MZ, q)) +
     b1(M2, mu, MA, q) + b1(M2, mu, MZ, q)
     + 4.0 * b1(M2, M2, MW, q) - 8.0 * b0(M2, M2, MW, q));
  
  double deltaMu = - 3.0 / (32.0 * sqr(PI)) *
    ((sqr(hb) + sqr(ht)) * 
     b1(mu, 0.0, sqrt(displaySoftMassSquared(mQl)(3, 3)), q) + 
     sqr(ht) * 
     b1(mu, 0.0, sqrt(displaySoftMassSquared(mUr)(3, 3)), q) + sqr(hb) * 
     b1(mu, 0.0, sqrt(displaySoftMassSquared(mDr)(3, 3)), q)) 
    - 3.0 * sqr(g2) / (64.0 * sqr(PI)) * 
    (b1(mu, M2, MA, q) + b1(mu, M2, MZ, q) + 2.0 * b1(mu, mu, MZ, q) - 4.0 *
     b0(mu, mu, MZ, q));
  
  mass(1, 1) = mass(1, 1) * (1.0 + deltaM1);
  mass(2, 2) = mass(2, 2) * (1.0 + deltaM2);
  mass(3, 4) = mass(3, 4) * (1.0 + deltaMu);
}

// mixNeut set to diagonal = mixNeut^T mNeutralino mixNeut: checked
void MssmSoftsusy::neutralinos(int accuracy, double piwwtMS, double pizztMS) {
  double tanb = displayTanb();
  double cosb = cos(atan(tanb));
  DoubleMatrix mNeut(4, 4);
  
  // Store current object in vector
  DoubleVector storeObject(display());
  double muInitial = displayMu();

  // In order to get M2(mch1), mu(mch2)
  runto(displayGaugino()(1));
  double m1 = displayGaugino()(1);
  runto(displayGaugino()(2));
  double m2 = displayGaugino()(2); 
  double smu = displaySusyMu();
  runto(fabs(smu));
  smu = displaySusyMu(); 

  // restore object at initial scale.
  setMu(muInitial);
  set(storeObject);

  // tree level
  mNeut(1, 1) = m1;
  mNeut(2, 2) = m2;
  mNeut(1, 3) = - sqrt(fabs(sqr(MZ) + pizztMS)) * cosb * 
    calcSinthdrbar();
  mNeut(1, 4) = - mNeut(1, 3) * tanb;
  mNeut(2, 3) = sqrt(fabs(sqr(MW) + piwwtMS)) * cosb;

  mNeut(2, 4) = - mNeut(2, 3) * tanb;
  mNeut(3, 4) = - smu;

  if (accuracy > 0) addNeutralinoLoop(mNeut);
  
  mNeut.symmetrise();
  if (mNeut.diagonaliseSym(physpars.mixNeut, physpars.mneut) > EPS *
      1.0e-3) 
    { cout << "accuracy bad in neutralino diagonalisation"<< flush;
    exit(1); }

}

// One loop corrections to gluino pole mass: hep-ph/9606211
// BUG fixed to use g3 at current scale 8.1.2001
// Changed to resummed version 11.05.2001
void MssmSoftsusy::gluino(int accuracy) {

  if (accuracy == 0) {
    physpars.mGluino = fabs(displayGaugino()(3));
    return;
  }

  // Store current object in vector
  DoubleVector storeObject(display());
  double muInitial = displayMu();

  if (physpars.mGluino == 0.0) physpars.mGluino = fabs(displayGaugino()(3));
  runto(fabs(physpars.mGluino)); // will calculate mass at mgluino

  double p = fabs(displayGaugino()(3)), Q = displayMu();

  // SUSY QCD radiative correction (gluon/gluino)
  double delta = 15.0 + 9.0 * log(sqr(Q / p));
  
  // Quark/squark correction
  delta = delta -
    (b1(p, dataSet.displayMass(mUp), physpars.mu(1, 1), Q) +
     b1(p, dataSet.displayMass(mCharm), physpars.mu(1, 2), Q) +
     b1(p, dataSet.displayMass(mTop), physpars.mu(1, 3), Q) + 
     b1(p, dataSet.displayMass(mUp), physpars.mu(2, 1), Q) + 
     b1(p, dataSet.displayMass(mCharm), physpars.mu(2, 2), Q) + 
     b1(p, dataSet.displayMass(mTop), physpars.mu(2, 3), Q) + 
     b1(p, dataSet.displayMass(mDown), physpars.md(1, 1), Q) +
     b1(p, dataSet.displayMass(mStrange), physpars.md(1, 2), Q) +
     b1(p, dataSet.displayMass(mBottom), physpars.md(1, 3), Q) +
     b1(p, dataSet.displayMass(mDown), physpars.md(2, 1), Q) +
     b1(p, dataSet.displayMass(mStrange), physpars.md(2, 2), Q) + 
     b1(p, dataSet.displayMass(mBottom), physpars.md(2, 3), Q) );

  // Third family mixing contribution:
  delta = delta - dataSet.displayMass(mTop) * sin(2.0 * physpars.thetat) *
    (b0(p, dataSet.displayMass(mTop), physpars.mu(1, 3), Q) - 
     b0(p, dataSet.displayMass(mTop), physpars.mu(2, 3), Q)) / p;
  delta = delta - dataSet.displayMass(mBottom) * sin(2.0 * physpars.thetab) *
    (b0(p, dataSet.displayMass(mBottom), physpars.md(1, 3), Q) - 
    b0(p, dataSet.displayMass(mBottom), physpars.md(2, 3), Q)) / p; 

  delta = delta * sqr(displayGaugeCoupling(3)) / (16.0 * sqr(PI));

  physpars.mGluino = fabs(displayGaugino()(3) / (1.0 - delta)); // Resummed
  //physpars.mGluino = mGaugino(3) * (1.0 + delta); // Non-resummed

  // restore object at initial scale.
  setMu(muInitial);
  set(storeObject);
}

// Formulae from hep-ph/9801365: checked but should be checked again!
// Implicitly calculates at the current scale.

double MssmSoftsusy::calcRunningMt() const {

  // For brevity
  double    thetaWDRbar = asin(calcSinthdrbar());
  double    cw2DRbar    = sqr(cos(thetaWDRbar));
  double    mtpole      = displayDataSet().displayPoleMt();
  const double  costh   = (MW / MZ);
  const double    cw2   = sqr(costh) ;
  const double    sw2   = (1.0 - cw2);

  double    ht      = displayYukawaMatrix(YU)(3, 3);
  double    hb      = displayYukawaMatrix(YD)(3, 3);
  double    mstop1  = displayPhys().mu(1,3);
  double    mstop2  = displayPhys().mu(2,3);
  double    mg      = displayPhys().mGluino;
  double    thetat  = displayPhys().thetat ;
  double    mH      = displayPhys().mhiggs(3); 
  double    alpha   = displayPhys().thetaH ;
  double    g       = displayGauge()(2);
  double    mh0     = displayPhys().mhiggs(1);
  double    mA      = displayPhys().mhiggs(2);
  double    beta    = atan(displaySusy().displayTanb());
  double    mb      = displayDataSet().displayMass(mBottom) ;
  double    mHc     = displayPhys().mhiggs(4);
  double    gp      = displayGaugeCoupling(1) * sqrt(0.6);
  double    thetab  = displayPhys().thetab;

  double resigmat = 0.0; 
  
  double qcd = 0.0, stopGluino = 0.0, twoLoopQcd = 0.0 , higgs = 0.0; 

  // 1 loop QCD only -- DRbar 10% correction
  qcd = - (5.0 + 6.0 * log(displayMu() / mtpole)) * 4.0 *
    sqr(displayGaugeCoupling(3)) / 3.0;
  resigmat = resigmat + qcd;
  
  // stop/gluino correction 6% correction
  stopGluino = 4.0 * sqr(displayGaugeCoupling(3)) / 3.0 *
    (b1(mtpole, mg, mstop1, displayMu()) + 
     b1(mtpole, mg, mstop2, displayMu()) -
     sin(2.0 * thetat) * mg / mtpole *  
     (b0(mtpole, mg, mstop1, displayMu()) - 
      b0(mtpole, mg, mstop2, displayMu())));
  resigmat = resigmat + stopGluino;
  
  // 2 loop QCD --> MSbar only...bit dodgey!
  twoLoopQcd = - 1.11 * sqr(sqr(displayGaugeCoupling(3)));
  resigmat = resigmat + twoLoopQcd;
  
  // rest are extra bits from Matchev et al: 2% corrections
  double alphaMZ = displayDataSet().displayAlpha(ALPHA),
    q = displayMu();
  double gtL = 0.5 - 2 * sw2 / 3.0, gtR = 2 * sw2 / 3.0;
  
  higgs = sqr(ht) / 2.0 * 
    (sqr(sin(alpha)) * (b1(mtpole, mtpole, mH, q) + b0(mtpole, mtpole, mH, q))
     + sqr(cos(alpha)) * (b1(mtpole, mtpole, mh0, q) + 
			  b0(mtpole, mtpole, mh0, q))
     + sqr(cos(beta)) * (b1(mtpole, mtpole, mA, q) - 
			 b0(mtpole, mtpole, mA, q)) 
     + sqr(sin(beta)) * (b1(mtpole, mtpole, MZ, q) - 
			 b0(mtpole, mtpole, MZ, q))) + 
    0.5 * ( (sqr(hb) * sqr(sin(beta)) + sqr(ht) * sqr(cos(beta))) * 
	    b1(mtpole, mb, mHc, q) +
	    (sqr(g) + sqr(hb) * sqr(cos(beta)) + sqr(ht) * sqr(sin(beta))) * 
	    b1(mtpole, mb, MW, q)) +
    sqr(hb) * sqr(cos(beta)) * (b0(mtpole, mb, mHc, q) 
				- b0(mtpole, mb, MW, q)) -
    (alphaMZ * 16.0 * PI) / 9.0 * (5.0 + 6.0 * log(q / mtpole)) +
    sqr(g) / cw2DRbar * ( (sqr(gtL) + sqr(gtR)) * b1(mtpole, mtpole, MZ, q) +
			  4.0 * gtL * gtR * b0(mtpole, mtpole, MZ, q) );
  resigmat = resigmat + higgs;
  
  DoubleMatrix neutralinoContribution(4, 2);
  // Neutralino contribution
  DoubleVector aPsi0TStopr(4), bPsi0TStopr(4), aPsi0TStopl(4),
    bPsi0TStopl(4); 
  aPsi0TStopr(1) = - 4 * gp / (3.0 * sqrt(2.0));
  bPsi0TStopl(1) = gp / (3.0 * sqrt(2.0));
  bPsi0TStopl(2) = g / sqrt(2.0);
  aPsi0TStopl(4) = ht;
  bPsi0TStopr(4) = ht;

  ComplexVector aChi0TStopl(4), bChi0TStopl(4), aChi0TStopr(4),
    bChi0TStopr(4);

  ComplexMatrix n(4, 4); DoubleVector mneut(4);
  mpzNeutralinos(n, mneut); // gets to Matchev+Pierce's conventions  

  aChi0TStopl = n.complexConjugate() * aPsi0TStopl;
  bChi0TStopl = n * bPsi0TStopl;
  aChi0TStopr = n.complexConjugate() * aPsi0TStopr;
  bChi0TStopr = n * bPsi0TStopr;

  ComplexMatrix aNeutTStop(4, 2), bNeutTStop(4, 2);
  DoubleMatrix fNeutTStop(4, 2), gNeutTStop(4, 2);

  int i, j;
  DoubleMatrix O(2, 2);
  ComplexVector tt(2), t1(2), t2(2);
  O = rot2d(thetat);
  for (i=1; i<=4; i++) {
    tt(1) = aChi0TStopl(i); tt(2) = aChi0TStopr(i);      
    t1 = O * tt;

    tt(1) = bChi0TStopl(i); tt(2) = bChi0TStopr(i);      
    t2 = O * tt;    
    for (j=1; j<=2; j++) {
      aNeutTStop(i, j) = t1(j);
      bNeutTStop(i, j) = t2(j);
      // functions of couplings needed for loops
      fNeutTStop(i, j) = sqr(aNeutTStop(i, j).mod()) + 
	sqr(bNeutTStop(i, j).mod());

      gNeutTStop(i, j) = 2.0 * 
	(aNeutTStop(i, j) * bNeutTStop(i, j).conj()).real(); 
      
      neutralinoContribution(i, j) = (fNeutTStop(i, j) * 
	 b1(mtpole, mneut(i), displayPhys().mu(j, 3), q) + 
	 gNeutTStop(i, j) * mneut(i) /  mtpole *  
	 b0(mtpole, mneut(i), displayPhys().mu(j, 3), q)) * 0.5;

      resigmat = resigmat + neutralinoContribution(i, j);
    }
  }

  // Chargino contribution  
  DoubleVector bPsicTSbotl(2), bPsicTSbotr(2), aPsicTSbotl(2); 

  bPsicTSbotl(1) = g;
  bPsicTSbotr(2) = - hb;
  aPsicTSbotl(2) = -ht;
  
  DoubleVector aPsicTSbotr(2), aPsicCSbotl(2);
  ComplexVector aChicTSbotr(2), aChicTSbotl(2), bChicTSbotl(2),
      bChicTSbotr(2);
  ComplexMatrix aChTSbot(2, 2), bChTSbot(2, 2);
  DoubleMatrix fChTSbot(2, 2), gChTSbot(2, 2); 

  ComplexMatrix u(2, 2), v(2, 2); DoubleVector mch(2);
  mpzCharginos(u, v, mch);

  aChicTSbotl = v.complexConjugate() * aPsicTSbotl;
  bChicTSbotl = u * bPsicTSbotl;
  aChicTSbotr = v.complexConjugate() * aPsicTSbotr;
  bChicTSbotr = u * bPsicTSbotr;
       
  double charginoContribution = 0.0;

  for(i=1; i<=2; i++) {  
    O = rot2d(thetab);      
    tt(1) = aChicTSbotl(i); tt(2) = aChicTSbotr(i);
    t1 = O * tt;
    tt(1) = bChicTSbotl(i); tt(2) = bChicTSbotr(i);      
    t2 = O * tt;
    for (j=1; j<=2; j++) {
	aChTSbot(i, j) = t1(j);
	bChTSbot(i, j) = t2(j);

      	fChTSbot(i, j) = sqr(aChTSbot(i, j).mod()) + 
	  sqr(bChTSbot(i, j).mod());
	gChTSbot(i, j) = 2.0 * (aChTSbot(i, j) * 
				bChTSbot(i, j).conj()).real(); 
      
      	charginoContribution = charginoContribution + 
	  (fChTSbot(i, j) * 
	   b1(mtpole, mch(i), fabs(displayPhys().md(j, 3)),
	      q) +
	   gChTSbot(i, j) * mch(i) / mtpole * 
	   b0(mtpole, mch(i), displayPhys().md(j, 3), q)) * 0.5;
    }            
  }

  resigmat = resigmat + charginoContribution;
  
  resigmat = resigmat * mtpole / (16.0 * sqr(PI));  

  return mtpole + resigmat;

}

// Using approximation of Matchev+Pierce - include contributions proportional
// to g and enhanced by tan beta or mu
double MssmSoftsusy::calcRunningMtau() const {
  double delta = sqr(displayGauge()(2)) / (16 * sqr(PI)) *
    (-displaySusyMu()) * displayGaugino()(2) * displayTanb() /
    (sqr(displaySusyMu()) - sqr(displayGaugino()(2))) *
    (b0(0.0, displayGaugino()(2), displayPhys().msnu(3), displayMu()) -
     b0(0.0, -displaySusyMu(), displayPhys().msnu(3), displayMu()));
  
  return displayDataSet().displayMass(mTau) * (1.0 - delta);
}

double MssmSoftsusy::calcRunningMb() const {
  if (displayMu() != MZ) {
    cout << "MssmSoftsusy::calcRunningMb called with mu=" <<
      displayMu() << endl; 
    exit(1);
  }
  
  double mbMZ = displayDataSet().displayMass(mBottom)
  	, alphasMZ = sqr(displayGaugeCoupling(3)) / (4.0 * PI);
  
  double    msbot1  = displayPhys().md(1,3);
  double    msbot2  = displayPhys().md(2,3);
  double    mstop1  = displayPhys().mu(1,3);
  double    mstop2  = displayPhys().mu(2,3);
  double    smu     = -displaySusyMu() ;
  double    tanb    = displayTanb() ;
  double    mg      = displayPhys().mGluino;
  double    thetab = displayPhys().thetab;
  double    thetat = displayPhys().thetat ;
  double    ht      = displayYukawaMatrix(YU)(3, 3);
  double    M2      = displayGaugino()(2);
  double    g       = displayGauge()(2);

  double p = PMBOTTOM;
  
  double deltaSquarkGluino = - alphasMZ / (3.0 * PI) *
    (b1(p, mg, msbot1, displayMu()) + 
     b1(p, mg, msbot2, displayMu()) - 
     sin(2.0 * thetab) * mg / PMBOTTOM *  
     (b0(p, mg, msbot1, displayMu()) - 
      b0(p, mg, msbot2, displayMu())));

  double deltaSquarkChargino = sqr(ht) / (16 * sqr(PI)) * smu *
    (displaySoftA(UA, 3, 3) * tanb + smu) / (sqr(mstop2) - sqr(mstop1)) *
    (b0(p, smu, mstop2, displayMu()) - 
     b0(p, smu, mstop1, displayMu())) 
    + sqr(g) / (16 * sqr(PI)) * 
    (smu * M2 * tanb / (sqr(smu) - sqr(M2)) *
     (sqr(cos(thetat)) * b0(p, M2, mstop2, displayMu()) +
      sqr(sin(thetat)) * b0(p, M2, mstop1, displayMu()) - 
      sqr(cos(thetat)) * b0(p, smu, mstop2, displayMu()) -
      sqr(sin(thetat)) * b0(p, smu, mstop1, displayMu())));

  return mbMZ * (1 - deltaSquarkGluino - deltaSquarkChargino);
}

void MssmSoftsusy::treeStop(DoubleMatrix & mass, double mtrun, double pizztMS, 
			    double sinthDRbarMS, int family) {
  const double cu = 2.0 / 3.0;
  double mz2 = sqr(MZ) + pizztMS, mt2 = sqr(mtrun);
  double beta = atan(displayTanb()), mu = displaySusyMu(),
    c2b = cos(2 * beta), tanb = displayTanb(), sinth2 = sqr(sinthDRbarMS);

  mass(1, 1) = displaySoftMassSquared(mQl)(family, family) +
    mz2 * (0.5 - cu * sinth2) * c2b;
  mass(2, 2) = displaySoftMassSquared(mUr)(family, family) +
    mz2 * cu * sinth2 * c2b;

  if (family != 3) mass(1, 2) = 0.0;
  else {
    mass(1, 2) = mtrun * (displaySoftA(UA, 3, 3) - mu / tanb);
    mass(1, 1) = mass(1, 1) + mt2;
    mass(2, 2) = mass(2, 2) + mt2;
  }

  mass(2, 1) = mass(1, 2);
}

void MssmSoftsusy::addSquarkCorrection(DoubleMatrix & mass) {

// No point adding radiative corrections to tachyonic particles
  if (mass(1, 1) < 0.0 || mass(2, 2) < 0.0) { 
    flagTachyon(true);
    return;
  }

  DoubleVector x(2), delta(2);
  int i; for (i=1; i<=2; i++)  
    { 
      x(i)= sqr(displayPhys().mGluino) / mass(i, i);
      
      delta(i) = sqr(displayGaugeCoupling(3)) / (6.0 * sqr(PI)) * 
	(1.0 + 3.0 * x(i) + sqr(x(i) - 1.0) * log(fabs(x(i) - 1.0))
	 - sqr(x(i)) * log(x(i)) + 2.0 * x(i) * 
	 log(sqr(displayMu()) / mass(i, i)));
      mass(i, i) = mass(i, i) * (1.0 + delta(i));
    }  
}

// As in BPMZ appendix, minus weak boson loops.
// Checked 21.06.01
void MssmSoftsusy::addStopCorrection(DoubleMatrix & mass, double mt) {

// No point adding radiative corrections to tachyonic particles
  if (mass(1, 1) < 0.0 || mass(2, 2) < 0.0) { 
    flagTachyon(true);
    return;
  }

  // one-loop correction matrix
  DoubleMatrix piSq(2, 2); // Self-energy matrix
	
  // brevity
  double    sinthDrbar = calcSinthdrbar();
  double    costhDrbar = sqrt(1.0 - sqr(sinthDrbar));
  double    alpha   = displayPhys().thetaH;
  double    g	    = displayGaugeCoupling(2);
  double    gp      = displayGaugeCoupling(1) * sqrt(0.6);
  double    beta    = atan(displayTanb());

  DoubleVector msbot(2);
  msbot(1) = displayPhys().md(1, 3);
  msbot(2) = displayPhys().md(2, 3);
  double    thetat  = displayPhys().thetat;
  double    thetab  = displayPhys().thetab;
  double    ct      = cos(thetat) ;
  double    st      = sin(thetat) ;
  double    cb      = cos(thetab) ;
  double    sb      = sin(thetab) ;
  DoubleVector mstop(2);
  mstop(1)          = displayPhys().mu(1, 3);
  mstop(2)          = displayPhys().mu(2, 3);
  double    mg      = displayPhys().mGluino;
  double    smu     = -displaySusyMu();
  double q = displayMu(), g3sq = sqr(displayGaugeCoupling(3)), 
    ht = displayYukawaElement(YU, 3, 3), 
    hb = displayYukawaElement(YD, 3, 3), 
    mb = displayDataSet().displayMass(mBottom),
    htsq = sqr(displayYukawaElement(YU, 3, 3)), 
    sinb = sin(beta), cosb = cos(beta), 
    hbsq = sqr(displayYukawaElement(YD, 3, 3));

  double p = calcMs();

  // define Higgs vector in 't-Hooft Feynman gauge, and couplings:
  DoubleVector higgsm(4), higgsc(2);
  higgsm(1) = displayPhys().mhiggs(3);
  higgsm(2) = displayPhys().mhiggs(1);
  higgsm(3) = MZ;
  higgsm(4) = displayPhys().mhiggs(2);
  higgsc(1) = displayPhys().mhiggs(4);
  higgsc(2) = MW;

  DoubleVector dnu(4), dnd(4), cn(4);
  cn(1)  = - cos(2.0 * displayPhys().thetaH);
  cn(2)  = - cn(1);
  cn(3)  = - cos(2.0 * beta);
  cn(4)  = - cn(3);
  dnu(1) = sqr(sin(displayPhys().thetaH));
  dnu(2) = sqr(cos(displayPhys().thetaH));
  dnu(3) = sqr(sinb);
  dnu(4) = sqr(cosb);
  dnd(1) = dnu(2);
  dnd(2) = dnu(1);
  dnd(3) = dnu(4);
  dnd(4) = dnu(3);

  DoubleMatrix lsStopLStopLR(4, 2), lsStopLStop12(4, 2);
  DoubleMatrix lsStopRStopLR(4, 2), lsStopRStop12(4, 2);
  // Order (s1 s2 G A, L R)
  lsStopLStopLR(1, 1) = g * MZ * guL * cosb / costhDrbar;
  lsStopLStopLR(1, 2) = ht * smu / sqrt(2.0);
  lsStopLStopLR(3, 2) = ht / sqrt(2.0) * 
    (smu * cosb + displaySoftA(UA, 3, 3) * sinb);
  lsStopLStopLR(4, 2) = -ht / sqrt(2.0) * 
    (smu * sinb - displaySoftA(UA, 3, 3) * cosb);
  lsStopLStopLR(2, 1) = -g * MZ * guL * sinb / costhDrbar
    + sqrt(2.0) * ht * mt;
  lsStopLStopLR(2, 2) = ht * displaySoftA(UA, 3, 3) / sqrt(2.0);

  lsStopRStopLR(1, 1) = lsStopLStopLR(1, 2);
  lsStopRStopLR(1, 2) = g * MZ * guR * cosb / costhDrbar;
  lsStopRStopLR(2, 1) = lsStopLStopLR(2, 2);
  lsStopRStopLR(2, 2) = -g * MZ * guR * sinb / costhDrbar
    + sqrt(2.0) * ht * mt;
  lsStopRStopLR(3, 1) = lsStopLStopLR(3, 2);
  lsStopRStopLR(4, 1) = -ht / sqrt(2.0) * 
    (smu * sinb - displaySoftA(UA, 3, 3) * cosb);

  // Mix stops up
  int i;
  DoubleVector temp(2), temp2(2);
  for (i=1; i<=4; i++) {
    temp(1) = lsStopLStopLR(i, 1);
    temp(2) = lsStopLStopLR(i, 2);
    temp2 = rot2d(thetat) * temp;
    lsStopLStop12(i, 1) = temp2(1);
    lsStopLStop12(i, 2) = temp2(2);
    temp(1) = lsStopRStopLR(i, 1);
    temp(2) = lsStopRStopLR(i, 2);
    temp2 = rot2d(thetat) * temp;
    lsStopRStop12(i, 1) = temp2(1);
    lsStopRStop12(i, 2) = temp2(2);
 }

  DoubleMatrix lHStopLStop12(lsStopLStop12), lHStopRStop12(lsStopRStop12);
  // Mix CP-even Higgses up
  for (i=1; i<=2; i++) { // i is the s1/s2 label
    temp(1) = lsStopLStop12(1, i);
    temp(2) = lsStopLStop12(2, i);
    temp2 = rot2d(alpha) * temp;
    lHStopLStop12(1, i) = temp2(1);
    lHStopLStop12(2, i) = temp2(2);
    temp(1) = lsStopRStop12(1, i);
    temp(2) = lsStopRStop12(2, i);
    temp2 = rot2d(alpha) * temp;
    lHStopRStop12(1, i) = temp2(1);
    lHStopRStop12(2, i) = temp2(2);
  }

  // Charged Higgs Feynman rules
  DoubleMatrix lChHstopLsbotLR(2, 2); // (H+ G+, L R) basis
  lChHstopLsbotLR(1, 1) = g * MW * sin(2.0 * beta) / sqrt(2.0) 
    - ht * mt * cosb - hb * mb * sinb;
  lChHstopLsbotLR(1, 2) = hb * (smu * cosb - displaySoftA(DA, 3, 3) * sinb);
  lChHstopLsbotLR(2, 1) = -g * MW * cos(2.0 * beta) / sqrt(2.0) 
    - ht * mt * sinb + hb * mb * cosb;
  lChHstopLsbotLR(2, 2) = hb * (smu * sinb + displaySoftA(DA, 3, 3) * cosb);
  DoubleMatrix lChHstopLsbot12(2, 2);
  temp(1) = lChHstopLsbotLR(1, 1);
  temp(2) = lChHstopLsbotLR(1, 2);
  temp2 = rot2d(thetab) * temp;
  lChHstopLsbot12(1, 1) = temp2(1);
  lChHstopLsbot12(1, 2) = temp2(2);
  temp(1) = lChHstopLsbotLR(2, 1);
  temp(2) = lChHstopLsbotLR(2, 2);
  temp2 = rot2d(thetab) * temp;
  lChHstopLsbot12(2, 1) = temp2(1);
  lChHstopLsbot12(2, 2) = temp2(2);

  DoubleMatrix lChHstopRsbotLR(2, 2); // (H+ G+, L R) basis
  lChHstopRsbotLR(1, 1) = ht * (smu * sinb - displaySoftA(UA, 3, 3) * cosb);
  lChHstopRsbotLR(1, 2) = -ht * mb * cosb - hb * mt * sinb;
  lChHstopRsbotLR(2, 1) = -ht * (smu * cosb + displaySoftA(UA, 3, 3) * sinb);
  DoubleMatrix lChHstopRsbot12(2, 2);
  temp(1) = lChHstopRsbotLR(1, 1);
  temp(2) = lChHstopRsbotLR(1, 2);
  temp2 = rot2d(thetab) * temp;
  lChHstopRsbot12(1, 1) = temp2(1);
  lChHstopRsbot12(1, 2) = temp2(2);
  temp(1) = lChHstopRsbotLR(2, 1);
  temp(2) = lChHstopRsbotLR(2, 2);
  temp2 = rot2d(thetab) * temp;
  lChHstopRsbot12(2, 1) = temp2(1);
  lChHstopRsbot12(2, 2) = temp2(2);

  // Neutralino Feynman rules
  DoubleVector aPsi0TStopr(4), bPsi0TStopr(4), aPsi0TStopl(4),
    bPsi0TStopl(4); 
  aPsi0TStopr(1) = - 4.0 * gp / (3.0 * sqrt(2.0));
  bPsi0TStopl(1) = gp / (3.0 * sqrt(2.0));
  bPsi0TStopl(2) = g / sqrt(2.0);
  aPsi0TStopl(4) = ht;
  bPsi0TStopr(4) = ht;

  ComplexVector aChi0TStopl(4), bChi0TStopl(4), aChi0TStopr(4),
    bChi0TStopr(4);

  ComplexMatrix n(4, 4); DoubleVector mneut(4);
  mpzNeutralinos(n, mneut); // gets to Matchev+Pierce's conventions  

  aChi0TStopl = n.complexConjugate() * aPsi0TStopl;
  bChi0TStopl = n * bPsi0TStopl;
  aChi0TStopr = n.complexConjugate() * aPsi0TStopr;
  bChi0TStopr = n * bPsi0TStopr;

  DoubleVector gChi0TopStopLL(4), fChi0TopStopLL(4);
  DoubleVector gChi0TopStopLR(4), fChi0TopStopLR(4);
  DoubleVector gChi0TopStopRR(4), fChi0TopStopRR(4);
  for (i=1; i<=4; i++) {
    fChi0TopStopLL(i) = (aChi0TStopl(i) * aChi0TStopl(i).conj() + 
      bChi0TStopl(i) * bChi0TStopl(i).conj()).real();
    gChi0TopStopLL(i) = (bChi0TStopl(i).conj() * aChi0TStopl(i) + 
      bChi0TStopl(i) * aChi0TStopl(i).conj()).real();
    fChi0TopStopRR(i) = (aChi0TStopr(i) * aChi0TStopr(i).conj() + 
      bChi0TStopr(i) * bChi0TStopr(i).conj()).real();
    gChi0TopStopRR(i) = (bChi0TStopr(i).conj() * aChi0TStopr(i) + 
      bChi0TStopr(i) * aChi0TStopr(i).conj()).real();
    fChi0TopStopLR(i) = (aChi0TStopr(i) * aChi0TStopl(i).conj() + 
      bChi0TStopr(i) * bChi0TStopl(i).conj()).real();
    gChi0TopStopLR(i) = (bChi0TStopl(i).conj() * aChi0TStopr(i) + 
      bChi0TStopr(i) * aChi0TStopl(i).conj()).real();
  }

  // Chargino Feynman Rules
  DoubleVector bPsicBStopl(2), bPsicBStopr(2), aPsicBStopl(2), 
    aPsicBStopr(2); 

  aPsicBStopl(1) = g;
  aPsicBStopr(2) = -ht;
  bPsicBStopl(2) = -hb;
  
  DoubleVector aPsicCSbotl(2);
  ComplexVector aChicBStopr(2), aChicBStopl(2), bChicBStopl(2),
      bChicBStopr(2);
  ComplexMatrix aChBStop(2, 2), bChBStop(2, 2);

  ComplexMatrix u(2, 2), v(2, 2); DoubleVector mch(2);
  mpzCharginos(u, v, mch);

  aChicBStopl = v.complexConjugate() * aPsicBStopl;
  bChicBStopl = u * bPsicBStopl;
  aChicBStopr = v.complexConjugate() * aPsicBStopr;
  bChicBStopr = u * bPsicBStopr;

  DoubleVector fChBStopLL(2), gChBStopLL(2) ;
  DoubleVector fChBStopLR(2), gChBStopLR(2); 
  DoubleVector fChBStopRR(2), gChBStopRR(2); 
  for (i=1; i<=2; i++) {
    fChBStopLL(i) = (aChicBStopl(i).conj() * aChicBStopl(i) +
		      bChicBStopl(i).conj() * bChicBStopl(i)).real();
    gChBStopLL(i) = (bChicBStopl(i).conj() * aChicBStopl(i) +
		      aChicBStopl(i).conj() * bChicBStopl(i)).real();
    fChBStopLR(i) = (aChicBStopl(i).conj() * aChicBStopr(i) +
		      bChicBStopl(i).conj() * bChicBStopr(i)).real();
    gChBStopLR(i) = (bChicBStopl(i).conj() * aChicBStopr(i) +
		      aChicBStopl(i).conj() * bChicBStopr(i)).real();
    fChBStopRR(i) = (aChicBStopr(i).conj() * aChicBStopr(i) +
		      bChicBStopr(i).conj() * bChicBStopr(i)).real();
    gChBStopRR(i) = (bChicBStopr(i).conj() * aChicBStopr(i) +
		      aChicBStopr(i).conj() * bChicBStopr(i)).real();
  }

  // Corrections themselves start here
  // Full corrections from BPMZ w/ g=g'=e=0
  DoubleMatrix strong(2, 2), stop(2, 2), sbottom(2, 2), 
    higgs(2, 2), chargino(2, 2), neutralino(2, 2);
  double a0t1 = a0(mstop(1), q), a0t2 = a0(mstop(2), q);
  double ft1 = ffn(p, mstop(1), 0.0, q), ft2 = ffn(p, mstop(2), 0.0, q);
  double ggt = gfn(p, mg, mt, q);
  strong(1, 1) = 4.0 * g3sq / 3.0 *
    (2.0 * ggt + sqr(ct) * (ft1 + a0t1) + sqr(st) * (ft2 + a0t2));
  strong(2, 2) = 4.0 * g3sq / 3.0 *
    (2.0 * ggt + sqr(st) * (ft1 + a0t1) + sqr(ct) * (ft2 + a0t2));
  strong(1, 2) = 4.0 * g3sq / 3.0 *
    (4.0 * mg * mt * b0(p, mg, mt, q) + st * ct * (ft1 - a0t1 - ft2 + a0t2));

  stop(1, 1) = htsq * (sqr(st) * a0t1 + sqr(ct) * a0t2);
  stop(2, 2) = htsq * (sqr(ct) * a0t1 + sqr(st) * a0t2);
  stop(1, 2) = htsq * ct * st * 3.0 * (a0t1 - a0t2);

  sbottom(1, 1) = 
    hbsq * (sqr(sb) * a0(msbot(1), q) + sqr(cb) * a0(msbot(2), q));
  sbottom(2, 2) = 
    htsq * (sqr(cb) * a0(msbot(1), q) + sqr(sb) * a0(msbot(2), q));

  for (i=1; i<=4; i++)
    higgs = higgs + 0.5 * htsq * dnu(i) * a0(higgsm(i), q);
  for (i=3; i<=4; i++) {
    double a0p = a0(higgsc(i - 2), q);
    higgs(1, 1) = higgs(1, 1) + hbsq * dnu(i) * a0p;
    higgs(2, 2) = higgs(2, 2) + htsq * dnd(i) * a0p;
  }
  int j; for(i=1; i<=4; i++)
    for (j=1; j<=2; j++) {
      double b0p = b0(p, higgsm(i), mstop(j), q);
      higgs(1, 1) = higgs(1, 1) + sqr(lHStopLStop12(i, j)) * b0p;
      higgs(1, 2) = higgs(1, 2) + 
	lHStopLStop12(i, j) * lHStopRStop12(i, j) * b0p;
      higgs(2, 2) = higgs(2, 2) + sqr(lHStopRStop12(i, j)) * b0p;
    }
  for(i=1; i<=2; i++)
    for (j=1; j<=2; j++) {
      double b0p = b0(p, higgsc(i), msbot(j), q); 
      higgs(1, 1) = higgs(1, 1) + sqr(lChHstopLsbot12(i, j)) * b0p;
      higgs(1, 2) = higgs(1, 2) + 
	lChHstopLsbot12(i, j) * lChHstopRsbot12(i, j) * b0p;
      higgs(2, 2) = higgs(2, 2) + sqr(lChHstopRsbot12(i, j)) * b0p;
    }

  for (i=1; i<=2; i++) {
    double one = gfn(p, mch(i), mb, q);
    double two = mch(i) * mb * b0(p, mch(i), mb, q) * 2.0;
    chargino(1, 1) = chargino(1, 1) + fChBStopLL(i) * one -
      gChBStopLL(i) * two;
    chargino(1, 2) = chargino(1, 2) + fChBStopLR(i) * one -
      gChBStopLR(i) * two;
    chargino(2, 2) = chargino(2, 2) + fChBStopRR(i) * one - 
      gChBStopRR(i) * two;
  }

  for (i=1; i<=4; i++) {
    double one = gfn(p, mneut(i), mt, q);
    double two = 2.0 * mneut(i) * mt * b0(p, mneut(i), mt, q);
    neutralino(1, 1) = neutralino(1, 1) +
      fChi0TopStopLL(i) * one - gChi0TopStopLL(i) * two;
    neutralino(2, 2) = neutralino(2, 2) +
      fChi0TopStopRR(i) * one - gChi0TopStopRR(i) * two;
    neutralino(1, 2) = neutralino(1, 2) +
      fChi0TopStopLR(i) * one - gChi0TopStopLR(i) * two;
  }

  // Have neglected loops involving photons or electroweak bosons.
  piSq = 1.0 / (16.0 * sqr(PI)) * 
    (strong + stop + sbottom + higgs + chargino + neutralino);

  piSq(2, 1) = piSq(1, 2);
  
  mass = mass - piSq;	  
}

// Checked 25.05.2001
void MssmSoftsusy::addSbotCorrection(DoubleMatrix & mass, double mt) {

// No point adding radiative corrections to tachyonic particles
  if (mass(1, 1) < 0.0 || mass(2, 2) < 0.0) { 
    flagTachyon(true);
    return;
  }

  // one-loop correction matrix
  DoubleMatrix piSq(2, 2); // Self-energy matrix
	
  // brevity
  double    sinthDrbar = calcSinthdrbar();
  double    costhDrbar = sqrt(1.0 - sqr(sinthDrbar));
  double    alpha   = displayPhys().thetaH;
  double    g	    = displayGaugeCoupling(2);
  double    gp      = displayGaugeCoupling(1) * sqrt(0.6);
  double    beta    = atan(displayTanb());

  DoubleVector msbot(2);
  msbot(1) = displayPhys().md(1, 3);
  msbot(2) = displayPhys().md(2, 3);
  double    thetat  = displayPhys().thetat;
  double    thetab  = displayPhys().thetab;
  double    ct      = cos(thetat) ;
  double    st      = sin(thetat) ;
  double    cb      = cos(thetab) ;
  double    sb      = sin(thetab) ;
  DoubleVector mstop(2);
  mstop(1)          = displayPhys().mu(1, 3);
  mstop(2)          = displayPhys().mu(2, 3);
  double    mg      = displayPhys().mGluino;
  double    smu     = -displaySusyMu();
  double q = displayMu(), g3sq = sqr(displayGaugeCoupling(3)), 
    ht = displayYukawaElement(YU, 3, 3), 
    hb = displayYukawaElement(YD, 3, 3), 
    mb = displayDataSet().displayMass(mBottom),
    htsq = sqr(displayYukawaElement(YU, 3, 3)), 
    sinb = sin(beta), cosb = cos(beta), 
    hbsq = sqr(displayYukawaElement(YD, 3, 3));

  double    p       = sqrt(msbot(1) * msbot(2));

  // define Higgs vector in 't-Hooft Feynman gauge, and couplings:
  DoubleVector higgsm(4), higgsc(2);
  higgsm(1) = displayPhys().mhiggs(3);
  higgsm(2) = displayPhys().mhiggs(1);
  higgsm(3) = MZ;
  higgsm(4) = displayPhys().mhiggs(2);
  higgsc(1) = displayPhys().mhiggs(4);
  higgsc(2) = MW;

  DoubleVector dnu(4), dnd(4), cn(4);
  cn(1)  = - cos(2.0 * displayPhys().thetaH);
  cn(2)  = - cn(1);
  cn(3)  = - cos(2.0 * beta);
  cn(4)  = - cn(3);
  dnu(1) = sqr(sin(displayPhys().thetaH));
  dnu(2) = sqr(cos(displayPhys().thetaH));
  dnu(3) = sqr(sinb);
  dnu(4) = sqr(cosb);
  dnd(1) = dnu(2);
  dnd(2) = dnu(1);
  dnd(3) = dnu(4);
  dnd(4) = dnu(3);

  DoubleMatrix lsSbotLSbotLR(4, 2), lsSbotLSbot12(4, 2);
  DoubleMatrix lsSbotRSbotLR(4, 2), lsSbotRSbot12(4, 2);
  // Order (s1 s2 G A, L R)
  lsSbotLSbotLR(1, 1) = g * MZ * gdL * cosb / costhDrbar + 
    sqrt(2.0) * hb * mb;
  lsSbotLSbotLR(1, 2) = hb * displaySoftA(DA, 3, 3) / sqrt(2.0);
  lsSbotLSbotLR(3, 2) = -hb / sqrt(2.0) * 
    (smu * sinb + displaySoftA(DA, 3, 3) * cosb);
  lsSbotLSbotLR(4, 2) = -hb / sqrt(2.0) * 
    (smu * cosb - displaySoftA(DA, 3, 3) * sinb);
  lsSbotLSbotLR(2, 1) = -g * MZ * gdL * sinb / costhDrbar;
  lsSbotLSbotLR(2, 2) = hb * smu / sqrt(2.0);

  lsSbotRSbotLR(1, 1) = lsSbotLSbotLR(1, 2);
  lsSbotRSbotLR(1, 2) = g * MZ * gdR * cosb / costhDrbar +
    sqrt(2.0) * hb * mb;
  lsSbotRSbotLR(2, 1) = lsSbotLSbotLR(2, 2);
  lsSbotRSbotLR(2, 2) = -g * MZ * gdR * sinb / costhDrbar;
  lsSbotRSbotLR(3, 1) = lsSbotLSbotLR(3, 2);
  lsSbotRSbotLR(4, 1) = lsSbotLSbotLR(4, 2);

  // Mix sbots up
  int i;
  DoubleVector temp(2), temp2(2);
  for (i=1; i<=4; i++) {
    temp(1) = lsSbotLSbotLR(i, 1);
    temp(2) = lsSbotLSbotLR(i, 2);
    temp2 = rot2d(thetab) * temp;
    lsSbotLSbot12(i, 1) = temp2(1);
    lsSbotLSbot12(i, 2) = temp2(2);
    temp(1) = lsSbotRSbotLR(i, 1);
    temp(2) = lsSbotRSbotLR(i, 2);
    temp2 = rot2d(thetab) * temp;
    lsSbotRSbot12(i, 1) = temp2(1);
    lsSbotRSbot12(i, 2) = temp2(2);
 }

  DoubleMatrix lHSbotLSbot12(lsSbotLSbot12), lHSbotRSbot12(lsSbotRSbot12);
  // Mix CP-even Higgses up
  for (i=1; i<=2; i++) { // i is the s1/s2 label
    temp(1) = lsSbotLSbot12(1, i);
    temp(2) = lsSbotLSbot12(2, i);
    temp2 = rot2d(alpha) * temp;
    lHSbotLSbot12(1, i) = temp2(1);
    lHSbotLSbot12(2, i) = temp2(2);
    temp(1) = lsSbotRSbot12(1, i);
    temp(2) = lsSbotRSbot12(2, i);
    temp2 = rot2d(alpha) * temp;
    lHSbotRSbot12(1, i) = temp2(1);
    lHSbotRSbot12(2, i) = temp2(2);
  }


  // Charged Higgs Feynman rules
  DoubleMatrix lChHsbotLstopLR(2, 2); // (H+ G+, L R) basis
  lChHsbotLstopLR(1, 1) = g * MW * sin(2.0 * beta) / sqrt(2.0) 
    - ht * mt * cosb - hb * mb * sinb;
  lChHsbotLstopLR(1, 2) = ht * (smu * sinb - displaySoftA(UA, 3, 3) * cosb);
  lChHsbotLstopLR(2, 1) = -g * MW * cos(2.0 * beta) / sqrt(2.0) 
    - ht * mt * sinb + hb * mb * cosb;
  lChHsbotLstopLR(2, 2) = -ht * (smu * cosb + displaySoftA(UA, 3, 3) * sinb);
  DoubleMatrix lChHsbotLstop12(2, 2);
  temp(1) = lChHsbotLstopLR(1, 1);
  temp(2) = lChHsbotLstopLR(1, 2);
  temp2 = rot2d(thetat) * temp;
  lChHsbotLstop12(1, 1) = temp2(1);
  lChHsbotLstop12(1, 2) = temp2(2);
  temp(1) = lChHsbotLstopLR(2, 1);
  temp(2) = lChHsbotLstopLR(2, 2);
  temp2 = rot2d(thetat) * temp;
  lChHsbotLstop12(2, 1) = temp2(1);
  lChHsbotLstop12(2, 2) = temp2(2);

  DoubleMatrix lChHsbotRstopLR(2, 2); // (H+ G+, L R) basis
  lChHsbotRstopLR(1, 1) = hb * (smu * cosb - displaySoftA(DA, 3, 3) * sinb);
  lChHsbotRstopLR(1, 2) = -ht * mb * cosb - hb * mt * sinb;
  lChHsbotRstopLR(2, 1) = hb * (smu * sinb + displaySoftA(DA, 3, 3) * cosb);
  DoubleMatrix lChHsbotRstop12(2, 2);
  temp(1) = lChHsbotRstopLR(1, 1);
  temp(2) = lChHsbotRstopLR(1, 2);
  temp2 = rot2d(thetat) * temp;
  lChHsbotRstop12(1, 1) = temp2(1);
  lChHsbotRstop12(1, 2) = temp2(2);
  temp(1) = lChHsbotRstopLR(2, 1);
  temp(2) = lChHsbotRstopLR(2, 2);
  temp2 = rot2d(thetat) * temp;
  lChHsbotRstop12(2, 1) = temp2(1);
  lChHsbotRstop12(2, 2) = temp2(2);

  // Neutralino Feynman rules
  DoubleVector aPsi0BSbotr(4), bPsi0BSbotr(4), aPsi0BSbotl(4),
    bPsi0BSbotl(4); 
  aPsi0BSbotr(1) = 2.0 * gp / (3.0 * sqrt(2.0));
  bPsi0BSbotl(1) = gp / (3.0 * sqrt(2.0));
  bPsi0BSbotl(2) = -g / sqrt(2.0);
  aPsi0BSbotl(3) = hb;
  bPsi0BSbotr(3) = hb;

  ComplexVector aChi0BSbotl(4), bChi0BSbotl(4), aChi0BSbotr(4),
    bChi0BSbotr(4);

  ComplexMatrix n(4, 4); DoubleVector mneut(4);
  mpzNeutralinos(n, mneut); // gets to Matchev+Pierce's conventions  

  aChi0BSbotl = n.complexConjugate() * aPsi0BSbotl;
  bChi0BSbotl = n * bPsi0BSbotl;
  aChi0BSbotr = n.complexConjugate() * aPsi0BSbotr;
  bChi0BSbotr = n * bPsi0BSbotr;

  DoubleVector gChi0BotSbotLL(4), fChi0BotSbotLL(4);
  DoubleVector gChi0BotSbotLR(4), fChi0BotSbotLR(4);
  DoubleVector gChi0BotSbotRR(4), fChi0BotSbotRR(4);
  for (i=1; i<=4; i++) {
    fChi0BotSbotLL(i) = (aChi0BSbotl(i) * aChi0BSbotl(i).conj() + 
      bChi0BSbotl(i) * bChi0BSbotl(i).conj()).real();
    gChi0BotSbotLL(i) = (bChi0BSbotl(i).conj() * aChi0BSbotl(i) + 
      bChi0BSbotl(i) * aChi0BSbotl(i).conj()).real();
    fChi0BotSbotRR(i) = (aChi0BSbotr(i) * aChi0BSbotr(i).conj() + 
      bChi0BSbotr(i) * bChi0BSbotr(i).conj()).real();
    gChi0BotSbotRR(i) = (bChi0BSbotr(i).conj() * aChi0BSbotr(i) + 
      bChi0BSbotr(i) * aChi0BSbotr(i).conj()).real();
    fChi0BotSbotLR(i) = (aChi0BSbotr(i) * aChi0BSbotl(i).conj() + 
      bChi0BSbotr(i) * bChi0BSbotl(i).conj()).real();
    gChi0BotSbotLR(i) = (bChi0BSbotl(i).conj() * aChi0BSbotr(i) + 
      bChi0BSbotr(i) * aChi0BSbotl(i).conj()).real();
  }

  // Chargino Feynman Rules
  DoubleVector bPsicTSbotl(2), bPsicTSbotr(2), aPsicTSbotl(2), 
    aPsicTSbotr(2); 

  bPsicTSbotl(1) = g;
  bPsicTSbotr(2) = -hb;
  aPsicTSbotl(2) = -ht;
  
  ComplexVector aChicTSbotr(2), aChicTSbotl(2), bChicTSbotl(2),
      bChicTSbotr(2);
  ComplexMatrix aChTSbot(2, 2), bChTSbot(2, 2);

  ComplexMatrix u(2, 2), v(2, 2); DoubleVector mch(2);
  mpzCharginos(u, v, mch);

  aChicTSbotl = v.complexConjugate() * aPsicTSbotl;
  bChicTSbotl = u * bPsicTSbotl;
  aChicTSbotr = v.complexConjugate() * aPsicTSbotr;
  bChicTSbotr = u * bPsicTSbotr;

  DoubleVector fChTSbotLL(2), gChTSbotLL(2) ;
  DoubleVector fChTSbotLR(2), gChTSbotLR(2); 
  DoubleVector fChTSbotRR(2), gChTSbotRR(2); 
  for (i=1; i<=2; i++) {
    fChTSbotLL(i) = (aChicTSbotl(i).conj() * aChicTSbotl(i) +
		      bChicTSbotl(i).conj() * bChicTSbotl(i)).real();
    gChTSbotLL(i) = (bChicTSbotl(i).conj() * aChicTSbotl(i) +
		      aChicTSbotl(i).conj() * bChicTSbotl(i)).real();
    fChTSbotLR(i) = (aChicTSbotl(i).conj() * aChicTSbotr(i) +
		      bChicTSbotl(i).conj() * bChicTSbotr(i)).real();
    gChTSbotLR(i) = (bChicTSbotl(i).conj() * aChicTSbotr(i) +
		      aChicTSbotl(i).conj() * bChicTSbotr(i)).real();
    fChTSbotRR(i) = (aChicTSbotr(i).conj() * aChicTSbotr(i) +
		      bChicTSbotr(i).conj() * bChicTSbotr(i)).real();
    gChTSbotRR(i) = (bChicTSbotr(i).conj() * aChicTSbotr(i) +
		      aChicTSbotr(i).conj() * bChicTSbotr(i)).real();
  }
  
  // Corrections themselves start here
  // Full corrections from BPMZ w/ g=g'=e=0
  DoubleMatrix strong(2, 2), stop(2, 2), sbottom(2, 2), 
    higgs(2, 2), chargino(2, 2), neutralino(2, 2);
  double a0t1 = a0(msbot(1), q), a0t2 = a0(msbot(2), q);
  double ft1 = ffn(p, msbot(1), 0.0, q), ft2 = ffn(p, msbot(2), 0.0, q);
  double ggt = gfn(p, mg, mb, q);
  strong(1, 1) = 4.0 * g3sq / 3.0 *
    (2.0 * ggt + sqr(cb) * (ft1 + a0t1) + sqr(sb) * (ft2 + a0t2));
  strong(2, 2) = 4.0 * g3sq / 3.0 *
    (2.0 * ggt + sqr(sb) * (ft1 + a0t1) + sqr(cb) * (ft2 + a0t2));
  strong(1, 2) = 4.0 * g3sq / 3.0 *
    (4.0 * mg * mb * b0(p, mg, mb, q) + sb * cb * (ft1 - a0t1 - ft2 + a0t2));

  stop(1, 1) = hbsq * (sqr(sb) * a0t1 + sqr(cb) * a0t2);
  stop(2, 2) = hbsq * (sqr(cb) * a0t1 + sqr(sb) * a0t2);
  stop(1, 2) = hbsq * cb * sb * 3.0 * (a0t1 - a0t2);

  sbottom(1, 1) = 
    htsq * (sqr(st) * a0(mstop(1), q) + sqr(ct) * a0(mstop(2), q));
  sbottom(2, 2) = 
    hbsq * (sqr(ct) * a0(mstop(1), q) + sqr(st) * a0(mstop(2), q));

  for (i=1; i<=4; i++)
    higgs = higgs + 0.5 * hbsq * dnd(i) * a0(higgsm(i), q);
  for (i=3; i<=4; i++) {
    double a0p = a0(higgsc(i - 2), q);
    higgs(1, 1) = higgs(1, 1) + htsq * dnd(i) * a0p;
    higgs(2, 2) = higgs(2, 2) + hbsq * dnu(i) * a0p;
  }
  int j; for(i=1; i<=4; i++)
    for (j=1; j<=2; j++) {
      double b0p = b0(p, higgsm(i), msbot(j), q);
      higgs(1, 1) = higgs(1, 1) + sqr(lHSbotLSbot12(i, j)) * b0p;
      higgs(1, 2) = higgs(1, 2) + 
	lHSbotLSbot12(i, j) * lHSbotRSbot12(i, j) * b0p;
      higgs(2, 2) = higgs(2, 2) + sqr(lHSbotRSbot12(i, j)) * b0p;
    }
  for(i=1; i<=2; i++)
    for (j=1; j<=2; j++) {
      double b0p = b0(p, higgsc(i), mstop(j), q); 
      higgs(1, 1) = higgs(1, 1) + sqr(lChHsbotLstop12(i, j)) * b0p;
      higgs(1, 2) = higgs(1, 2) + 
	lChHsbotLstop12(i, j) * lChHsbotRstop12(i, j) * b0p;
      higgs(2, 2) = higgs(2, 2) + sqr(lChHsbotRstop12(i, j)) * b0p;
    }

  for (i=1; i<=2; i++) {
    double one = gfn(p, mch(i), mt, q);
    double two = mch(i) * mt * b0(p, mch(i), mt, q) * 2.0;
    chargino(1, 1) = chargino(1, 1) + fChTSbotLL(i) * one -
      gChTSbotLL(i) * two;
    chargino(1, 2) = chargino(1, 2) + fChTSbotLR(i) * one -
      gChTSbotLR(i) * two;
    chargino(2, 2) = chargino(2, 2) + fChTSbotRR(i) * one - 
      gChTSbotRR(i) * two;
  }

  for (i=1; i<=4; i++) {
    double one = gfn(p, mneut(i), mb, q);
    double two = 2.0 * mneut(i) * mb * b0(p, mneut(i), mb, q);
    neutralino(1, 1) = neutralino(1, 1) +
      fChi0BotSbotLL(i) * one - gChi0BotSbotLL(i) * two;
    neutralino(2, 2) = neutralino(2, 2) +
      fChi0BotSbotRR(i) * one - gChi0BotSbotRR(i) * two;
    neutralino(1, 2) = neutralino(1, 2) +
      fChi0BotSbotLR(i) * one - gChi0BotSbotLR(i) * two;
  }

  // Have neglected loops involving photons or electroweak bosons.
  piSq = 1.0 / (16.0 * sqr(PI)) * 
    (strong + stop + sbottom + higgs + chargino + neutralino);

  piSq(2, 1) = piSq(1, 2);

  mass = mass - piSq;	
}

void MssmSoftsusy::doStop(double mt, double pizztMS, double sinthDRbarMS, int
			  accuracy) { 
  int family; for (family = 1; family <= 3; family++) {

    DoubleMatrix mStopSquared(2, 2);
    treeStop(mStopSquared, mt, pizztMS, sinthDRbarMS, family);

    // one loop corrections 
    if (family == 3 && accuracy > 0) addStopCorrection(mStopSquared, mt);
    if (family != 3 && accuracy > 0) addSquarkCorrection(mStopSquared); 
    
    double theta;
    DoubleVector 
      physicalStopMassesSquared(mStopSquared.sym2by2(theta));
    DoubleVector physicalStopMasses(physicalStopMassesSquared.apply(ccbSqrt));
    
    if (family == 3) physpars.thetat = theta;

    physpars.mu(1, family) = physicalStopMasses(1);
    physpars.mu(2, family) = physicalStopMasses(2);
  }
}

void MssmSoftsusy::treeSbot(DoubleMatrix & mass, double mbrun, double pizztMS, 
			    double sinthDRbarMS, int family) {
  const double cd = 1.0 / 3.0;
  double mz2 = sqr(MZ) + pizztMS, mb2 = sqr(mbrun);
  double beta = atan(displayTanb()), mu = displaySusyMu(),
    c2b = cos(2 * beta), tanb = displayTanb(), sinth2 = sqr(sinthDRbarMS);

  mass(1, 1) = displaySoftMassSquared(mQl)(family, family) +
      mz2 * (-0.5 + cd * sinth2) * c2b;
  mass(2, 2) = displaySoftMassSquared(mDr)(family, family) -
      mz2 * cd * sinth2 * c2b;

  if (family != 3) mass(1, 2) = 0.0;
  else {
    mass(1, 2) = mbrun * (displaySoftA(DA, 3, 3) - mu * tanb);
    mass(1, 1) = mass(1, 1) + mb2;
    mass(2, 2) = mass(2, 2) + mb2;
  }

  mass(2, 1) = mass(1, 2);
}

void MssmSoftsusy::doSbot(double mb, double pizztMS, double
			    sinthDRbarMS, int accuracy, double mt) {
  int family; for (family = 1; family <= 3; family++) {

    DoubleMatrix mSbotSquared(2, 2);
    treeSbot(mSbotSquared, mb, pizztMS, sinthDRbarMS, family);

    if (family == 3 && accuracy > 0) addSbotCorrection(mSbotSquared,
						       mt); 
    if (family != 3 && accuracy > 0) addSquarkCorrection(mSbotSquared);
    
    double theta;
    DoubleVector 
      physicalSbotMassesSquared(mSbotSquared.sym2by2(theta));
    DoubleVector physicalSbotMasses(physicalSbotMassesSquared.apply(ccbSqrt));
    
    if (family == 3) {physpars.thetab = theta;
    }
    physpars.md(1, family) = physicalSbotMasses(1);
    physpars.md(2, family) = physicalSbotMasses(2);
  }
}

void MssmSoftsusy::treeStau(DoubleMatrix & mass, double mtaurun, double
			    pizztMS, double sinthDRbarMS, int family) {
  double mz2 = sqr(MZ) + pizztMS, mtau2 = sqr(mtaurun);
  double beta = atan(displayTanb()), mu = displaySusyMu(),
    c2b = cos(2 * beta), tanb = displayTanb(), sinth2 = sqr(sinthDRbarMS);

  mass(1, 1) = displaySoftMassSquared(mLl)(family, family) -
    mz2 * (0.5 - sinth2) * c2b;
  mass(2, 2) = displaySoftMassSquared(mEr)(family, family) -
    mz2 * sinth2 * c2b;

  if (family != 3) mass(1, 2) = 0.0;
  else {
    mass(1, 2) = mtaurun * (displaySoftA(EA, 3, 3) - mu * tanb);
    mass(1, 1) = mass(1, 1) + mtau2;
    mass(2, 2) = mass(2, 2) + mtau2;
  }

  mass(2, 1) = mass(1, 2);
}

void MssmSoftsusy::doStau(double mtau, double pizztMS, double
			    sinthDRbarMS, int accuracy) {
  int family; for (family = 1; family <= 3; family++) {
    DoubleMatrix mStauSquared(2, 2);
    treeStau(mStauSquared, mtau, pizztMS, sinthDRbarMS, family);
    //  if (family == 3 && accuracy > 0) addStauCorrection(mStauSquared);
  
    double theta;
    DoubleVector 
      physicalStauMassesSquared(mStauSquared.sym2by2(theta));
    DoubleVector physicalStauMasses(physicalStauMassesSquared.apply(ccbSqrt));

    if (family == 3) physpars.thetatau = theta;

    physpars.me(1, family) = physicalStauMasses(1);
    physpars.me(2, family) = physicalStauMasses(2);
  }
}

void MssmSoftsusy::doSnu(double pizztMS) {
  double mSnuSquared;
  int family; for (family = 1; family <= 3; family++) {
    treeSnu(mSnuSquared, pizztMS, family);
    //  if (family == 3 && accuracy > 0) addSnuCorrection(mSnuSquared);
    
    physpars.msnu(family) = ccbSqrt(mSnuSquared);
  }
}

void MssmSoftsusy::treeSnu(double & mSnuSquared, double pizztMS, int family) {
  double mz2 = sqr(MZ) + pizztMS;
  double beta = atan(displayTanb());
  double c2b = cos(2.0 * beta);

  mSnuSquared = displaySoftMassSquared(mLl)(family, family) + 0.5 * mz2
      * c2b;
}

// Organises calculation of physical quantities such as sparticle masses etc
// Call AT MSusy
void MssmSoftsusy::physical(int accuracy) {
  double sinthDRbarMS, piwwtMS, pizztMS;

  if (accuracy == 0) {
    sinthDRbarMS = 0.0;
    piwwtMS = 0.0;
    pizztMS = 0.0;
  }
  else {
  sinthDRbarMS = calcSinthdrbar();
  piwwtMS = piWWT(MW, displayMu());
  pizztMS = piZZT(MZ, displayMu());
  }

  // Running masses at MSUSY
  double beta = atan(displayTanb());
  double vev = displayHiggsVevMs();
  double mt = displayYukawaElement(YU, 3, 3) * vev * sin(beta) / sqrt(2.0);
  double mb = displayYukawaElement(YD, 3, 3) * vev * cos(beta) / sqrt(2.0);
  double mtau = displayYukawaElement(YE, 3, 3) * vev * cos(beta) / sqrt(2.0);

  // Sfermion masses: all three families in each
  doStop(mt, pizztMS, sinthDRbarMS, accuracy); 
  doSbot(mb, pizztMS, sinthDRbarMS, accuracy, mt); 
  doStau(mtau, pizztMS, sinthDRbarMS, accuracy); 
  doSnu(pizztMS);
  
  // Charginos/neutralinos/higgs
  higgs(accuracy, piwwtMS, pizztMS);   
  gluino(accuracy); 
  charginos(accuracy, piwwtMS); 
  neutralinos(accuracy, piwwtMS, pizztMS);
}

// For a given trial value of the log of field H2, gives the value of the
// potential at the minimum. The following global variables must be set before
// it is called:
static double unificationScale, minTol;
extern double minimufb3(double lnH2) {

  // Save initial parameters
  double initialMu = tempSoft1->displayMu();
  DoubleVector savePars(tempSoft1->display());

  MssmSoftsusy temp(*tempSoft1);

  double mx = unificationScale; 
  double mu = -temp.displaySusyMu();
  
  int family = 1; // family could be 2 as well if they're very different
  
  // field VEVS
  double htau = temp.displayYukawaMatrix(YE)(3, 3);
  double h2 =  - exp(lnH2), 
    eR = sqrt(fabs(mu * h2 / htau)),
    Lisq = -4.0 * tempSoft1->displaySoftMassSquared(mLl)(family, family) / 
    (0.6 * sqr(tempSoft1->displayGauge()(1)) +
     sqr(tempSoft1->displayGauge()(2))) + sqr(h2) + sqr(eR);
  
  // qhat is scale at which one-loop potential corrections are smallest:
  // iterate
  double qhat = getQhat(minTol,eR, h2, Lisq, mx, temp);
  if (qhat < 0.0) {
    cout << "Error: qhat returned negative in ufb3min" << endl;
    exit(1);
  }
  
  double vufb3 = ufb3fn(mu, htau, h2, family, temp);
  if (PRINTOUT > 1) cout << vufb3 << endl;

  // Restore initial parameters
  tempSoft1->setMu(initialMu);
  tempSoft1->set(savePars);

  return vufb3;  
};

double ufb3fn(double mu, double htau, double h2, int family, MssmSoftsusy
	      temp) { 
  double vufb3 = 0.0;
  // potential value for these VEVs
  if (fabs(h2) > 
      sqrt(sqr(mu) / (4.0 * sqr(htau)) + 
	   4.0 * temp.displaySoftMassSquared(mLl)(family, family) /  
	   (0.6 * sqr(temp.displayGauge()(1)) +
	    sqr(temp.displayGauge()(2)))) - fabs(mu) / 
      (0.6 * sqr(temp.displayGauge()(1)) +
       sqr(temp.displayGauge()(2))))
    vufb3 = 
      sqr(h2) * (temp.displayMh2Squared() +
		 temp.displaySoftMassSquared(mLl)(family, family)) + 
      fabs(mu * h2) / htau * 
      (temp.displaySoftMassSquared(mLl)(3, 3) +
       temp.displaySoftMassSquared(mEr)(3, 3) 
       + temp.displaySoftMassSquared(mLl)(family, family)) -
      2.0 * sqr(temp.displaySoftMassSquared(mLl)(family, family)) / 
      (0.6 * sqr(temp.displayGauge()(1)) +
       sqr(temp.displayGauge()(2)));
  else
    vufb3 = 
      sqr(h2) * temp.displayMh2Squared() + 
      fabs(mu * h2) / htau * 
      (temp.displaySoftMassSquared(mLl)(3, 3) +
       temp.displaySoftMassSquared(mEr)(3, 3)) +  
      1.0 / 8.0 * (0.6 * sqr(temp.displayGauge()(1)) +
		   sqr(temp.displayGauge()(2))) * 
      sqr(sqr(h2) + fabs(mu * h2) / htau);
  
  if (PRINTOUT > 1) cout << vufb3 << endl;
  return vufb3;
}

// For ufb3direction, returns scale at which one-loop corrections are smallest
double getQhat(double inminTol,double eR, double h2, double Lisq, double mx,
		MssmSoftsusy & temp) {
  double oldQhat = -1.0e16;
  int maxNum = 40;
  
  int d; for (d = 1; d <= maxNum; d++)     {
    double qhat = 
      maximum(maximum(maximum(temp.displayGauge()(2) * eR, 
		     temp.displayGauge()(2) * fabs(h2)), 
		temp.displayGauge()(2) * sqrt(fabs(Lisq))),
	   temp.displayYukawaMatrix(YU)(3, 3) * fabs(h2));
    // Run all paramaters to that scale
    if (qhat < mx) temp.runto(qhat);
    else temp.runto(mx); 
    if (PRINTOUT > 1) cout << qhat << " ";
    
    if (fabs((qhat - oldQhat) / qhat) < inminTol) return qhat;
    oldQhat = qhat;
  }
  // Return NOB if no convergence on qhat
  return -6.66e66;
}

// Input mx the scale up to which you search for minima
// Returns minimum value of potential along that direction
// Does ufbs truly properly but takes ages.
double MssmSoftsusy::ufb3sl(double mx) {

  // Save initial parameters
  double initialMu = displayMu();
  DoubleVector parSave(display());

  unificationScale = mx;
  // Running should be faster from high scales on the whole
  runto(mx);
  
  double ax = log(1.0e3), bx = 0.5*(log(mx) + log(1.0e3)), cx = log(mx);
  
  double tol = 1.0e-3, lnx; minTol = tol * 10.0;
  // Numerical recipes routine to determine minimum of potential specified in
  // minimufb3
  double Vmin = findMinimum(ax, bx, cx, minimufb3, tol, &lnx);

  // Restore initial parameters
  setMu(initialMu);
  set(parSave);
  
  return Vmin;
}

// Does SUSY (and other) threshold corrections to alphaS
// Input alphas in MSbar and it returns it in DRbar scheme. From hep-ph/9606211
double MssmSoftsusy::qcdSusythresh(double alphasMSbar, double q) const {
  double mt = displayDataSet().displayPoleMt();
  double deltaAlphas = alphasMSbar / (2.0 * PI) *
    (0.5 - 2.0 / 3.0 * log(mt / q) - 2.0 * log(displayPhys().mGluino / q));
  
  int i,j;
  for (i=1; i<=2; i++)
    for (j=1; j<=3; j++)
      deltaAlphas = deltaAlphas - alphasMSbar / (12.0 * PI) * 
	(log(displayPhys().mu(i, j) / q) + log(displayPhys().md(i, j) /
						q));
  return alphasMSbar / (1.0 - deltaAlphas); 
}

// Does SUSY (and other) threshold corrections to alphaEm - returns alpha in
// DRbar scheme at scale Q. From hep-ph/9606211. Input empirical value of
// alpha at 0 momentum....
double MssmSoftsusy::qedSusythresh(double alphaEm, double q) const {

  if (displayPhys().mhiggs(4) < EPS) return 0.0;

  double mtpole = displayDataSet().displayPoleMt();
  double deltaAlpha = 0.0682 -  alphaEm / (2 * PI) * 
    (-7.0 * log(MW / q) + 16.0 / 9.0 * log(mtpole / q)
     + log(displayPhys().mhiggs(4) / q) / 3.0 + 4.0 / 9.0 * 
     (log(displayPhys().mu(1,1) / q) + log(displayPhys().mu(2,1) / q) + 
      log(displayPhys().mu(1,2) / q) + log(displayPhys().mu(2,2) / q) + 
      log(displayPhys().mu(1,3) / q) + log(displayPhys().mu(2,3) / q)) + 
     (log(displayPhys().md(1,1) / q) + log(displayPhys().md(2,1) / q) + 
      log(displayPhys().md(1,2) / q) + log(displayPhys().md(2,2) / q) + 
      log(displayPhys().md(1,3) / q) + log(displayPhys().md(2,3) / q)) 
     / 9.0 +
     (log(displayPhys().me(1,1) / q) + log(displayPhys().me(2,1) / q) + 
      log(displayPhys().me(1,2) / q) + log(displayPhys().me(2,2) / q) + 
      log(displayPhys().me(1,3) / q) + log(displayPhys().me(2,3) / q)) 
     / 3.0 
     + (log(fabs(displayPhys().mch(1)) / q) 
	+ log(fabs(displayPhys().mch(2)) / q)) 
     * 4.0 / 3.0);

  return alphaEm / (1.0 - deltaAlpha);
}


// Prints out what the lsp is
void recogLsp(int temp, int posj) {
  switch(temp) {
  case 0: cout << "neutralino"; break;
  case 1: 
    switch(posj)
      {
      case 3: cout << "stop"; break;
      case 2: cout << "scharm"; break;
      case 1: cout << "sup"; break;
      } break;
  case 2:
    switch(posj)
      {
      case 3: cout << "sbottom"; break;
      case 2: cout << "sstange"; break;
      case 1: cout << "sdown"; break;
      } break;
  case 3:
    switch(posj)
      {
      case 3: cout << "stau"; break;
      case 2: cout << "smu"; break;
      case 1: cout << "selectron"; break;
      } break;
  case 4: cout << "chargino"; break;
  case 5: cout << "sneutrino"; break;
  case 6: cout << "gluino"; break;
  default:
    cout << "Wrong input to lsp printing routine\n";
    exit(1); break;
  }
  
}

// Returns lsp mass in mass and function return labels which particle is lsp:
// 0 is neutralino posi = #, posj = 0
int MssmSoftsusy::lsp(double & mass, int & posi, int & posj) const {
  int temp = 0, pos1, pos2;
  sPhysical s(displayPhys());
  
  double minmass = s.mneut.apply(fabs).min(pos1); posi = pos1; posj = 0;
  
  // up squarks 1
  double lightest = s.mu.apply(fabs).min(pos1, pos2);
  if (lightest < minmass) { 
    minmass = lightest; posi = pos1; posj = pos2; temp = 1; }
  
  // down squarks 2
  lightest = s.md.apply(fabs).min(pos1, pos2);
  if (lightest < minmass) { 
    minmass = lightest; posi = pos1; posj = pos2; temp = 2; }
  
  // sleptons 3
  lightest = s.me.apply(fabs).min(pos1, pos2);
  if (lightest < minmass) { 
    minmass = lightest; posi = pos1; posj = pos2; temp = 3; }
  
  // charginos 4
  lightest = s.mch.apply(fabs).min(pos1);
  if (lightest < minmass) { 
    minmass = lightest; posi = pos1; posj = 0; temp = 4; }
  
  // sneutrinos 5
  lightest = s.msnu.apply(fabs).min(pos1);
  if (lightest < minmass) { 
    minmass = lightest; posi = pos1; posj = 0; temp = 5; }
  
  // gluino 6
  lightest = s.mGluino;
  if (lightest < minmass) { 
    minmass = lightest; posi = 0; posj = 0; temp = 6; }
  
  mass = minmass;
  return temp;
}

// physpars should be defined already for this
double MssmSoftsusy::calcMs() const {
  return maximum(sqrt(displayPhys().mu(2, 3) * displayPhys().mu(1, 3)), MZ);
}

// Provides the first guess at a SUSY object at mt, inputting tanb and oneset
// (should be at MZ) - it's very crude, doesn't take radiative corrections
// into account etc. 
MssmSusy guessAtSusyMt(double tanb, const QedQcd & oneset) {
  if (oneset.displayMu() != MZ) {
    cout << 
      "WARNING:guessAtSusymt in softsusy.cc called with oneset at scale" <<
      oneset.displayMu() << "\ninstead of " << MZ << endl;
  }
  
  // This bit gives a guess at a SUSY object
  QedQcd leAtMt(oneset);

  DoubleVector a(3), g(3);
  double sinth2 = 1.0 - sqr(MW / MZ);
  // Gauge couplings at mt
  a = leAtMt.getGaugeMu(oneset.displayPoleMt(), sinth2);
  
  MssmSusy t; 
  t.setTanb(tanb);
  double beta = atan(tanb);
  
  // Yukawa couplings
  double vev = 246.22;
  double ht = leAtMt.displayMass(mTop) * sqrt(2.0) / (vev * sin(beta));
  double hb =  ht * tanb * leAtMt.displayMass(mBottom) /
    leAtMt.displayMass(mTop);
  double htau =  hb * leAtMt.displayMass(mTau) /
    leAtMt.displayMass(mBottom); 
  
  const double pi = 3.141592653589793;
  int i; for(i=1; i<=3; i++) g(i) = sqrt(4.0 * pi * a(i));
  t.setAllGauge(g);
  
  t.setYukawaElement(YU, 3, 3, ht);
  t.setYukawaElement(YD, 3, 3, hb);
  t.setYukawaElement(YE, 3, 3, htau);
  
  t.setMu(oneset.displayPoleMt());   
  
  return t;
}


// Returns low energy softsusy object consistent with BC's m0 etc at MGUT.
// This version iterates from the GUT scale.
// If minV < 0 there's a problem with the ufb constraints on return
// epsilon is accuracy
// susyAtMt is just for initialisation, but it should be at the top mass
// oneset should be at MZ and contains the SM data to fit the model to
// if the running runs into difficulty, eg if a Landau pole is reached, it
// returns a ZERO object: no result is possible!
// Boundary condition is the theoretical condition on parameters at the high
// energy scale mx: the parameters themselves are contained within the vector.
// The parameters should be ordered in a manner in which its reasonable for
// the first to be an initial guess for mu(mx), eg m0 in mSUGRA
void MssmSoftsusy::lowOrg
(void (*boundaryCondition)(MssmSoftsusy &, const DoubleVector &),
 double mx, const DoubleVector & pars, int sgnMu, double tanb, const QedQcd &
 oneset, int thresholds, double epsilon) {

  if (oneset.displayMu() != MZ) {
    cout << "WARNING: lowOrg in softsusy.cc called with oneset at scale" <<
      oneset.displayMu() << "\ninstead of " << MZ << endl;
  }
  
  const int maxtries = 50; 
  double tol = epsilon;
  
  MssmSusy t(guessAtSusyMt(tanb, oneset));
  t.setLoops(2); t.runto(mx); // 2 loops should protect against ht Landau pole
  
  setSusy(t); 
  setData(oneset); 
  
  // Initial guess: B=0, mu=1st parameter, need better guesses
  boundaryCondition(*this, pars);
  setSusyMu(sgnMu * 1.0);
  setB(0.0);
  
  run(mx, MZ);
  
  rewsbTreeLevel(sgnMu); 
  
  physical(0);

  setThresholds(thresholds); setLoops(2);
  
  // Following returns the most accurate iteration at MZ: in=out
  itLowsoft(maxtries, mx, sgnMu, tol, tanb, boundaryCondition, pars);
  
  if (displayProblem().nonperturbative || displayProblem().noConvergence)
    return;
  
  if (PRINTOUT) cout << " end of iteration" << endl;
  
  if (PRINTOUT > 0) {
    MssmSoftsusy out(*this);
    out.runto(calcMs());
    double tb, MZp = sqrt(fabs(out.predMzsq(tb))); 
    cout << "MZ is calculated to be " <<  MZp <<
      " and tan beta(MS)=" << tb << endl;
  }
    
  // check to see if there are any negative mass squared scalars: if there
  // are, flag scalarMinimum
  int posi, posj, lspCode; double minmass;
  lspCode = lsp(minmass, posi, posj); 
  if (lspCode > 0 && lspCode < 6 && lspCode !=4 && minmass < 0.0)
    { flagTachyon(true); }
}


// You should evaluate this at a scale MSusy average of stops.
// Depth of electroweak minimum:
double MssmSoftsusy::realMinMs() const {
  MssmSusy temp(displaySusy());
  temp.runto(calcMs(), EPS);
  
  double beta = atan(temp.displayTanb());
  return - 1.0 / 16.0 * 
    (0.6 * sqr(temp.displayGauge()(1)) + 
     sqr(temp.displayGauge()(2))) * 
    sqr(displayHiggsVevMs() * cos(2.0 * beta));
}

// Difference between two SOFTSUSY objects in and out: EWSB terms only
double sumTol(const MssmSoftsusy & in, const MssmSoftsusy & out) {
  
  double muin = fabs(in.displaySusyMu()), 
    muout = fabs(out.displaySusyMu()),
    bin = fabs(in.displayB()), 
    bout = fabs(out.displayB());
  /*    mH1sqin = fabs(in.displayMh1Squared()),
    mH1sqout = fabs(out.displayMh1Squared()),
    mH2sqin = fabs(in.displayMh2Squared()),
    mH2sqout = fabs(out.displayMh2Squared()); */
  
  DoubleVector sT(2);
  sT(1) = fabs(1.0 - minimum(muin, muout) / maximum(muin, muout));
  sT(2) = fabs(1.0 - minimum(bin, bout) / maximum(bin, bout));
  //  sT(3) = fabs(1.0 - minimum(mH1sqin, mH1sqout) / maximum(mH1sqin, mH1sqout));
  //  sT(4) = fabs(1.0 - minimum(mH2sqin, mH2sqout) / maximum(mH2sqin, mH2sqout));
  
  return sT.max();
}

// Yukawa coupling at any scale - should give it vev at correct scale
double MssmSoftsusy::calcHt(double vev) const {
  double mtrun = calcRunningMt();
  double beta = atan(displayTanb());
  double ht = mtrun * sqrt(2.0) / (vev * sin(beta));
  return ht;
}

double MssmSoftsusy::calcHb(double vev) const {
  double mbrun = calcRunningMb();
  double beta = atan(displayTanb());
  double hb = mbrun * sqrt(2.0) / (vev * cos(beta));
  return hb;
}

double MssmSoftsusy::calcHtau(double vev) const {
  double mtaurun = calcRunningMtau();
  double beta = atan(displayTanb());
  double htau = mtaurun * sqrt(2.0) / (vev * cos(beta));
  return htau;
}

// Calculates sin theta at the current scale
double MssmSoftsusy::calcSinthdrbar() const {
  double alphaDrbar = qedSusythresh(ALPHAEM0, displayMu());
  double eDrbar = sqrt(4.0 * PI * alphaDrbar);
  
  return eDrbar / displayGaugeCoupling(2); // from hep-ph/9501277
}

//VEV at current scale
double MssmSoftsusy::getVev() const {
  double piZzT = piZZT(MZ, displayMu());

  double vsquared = 4.0 * (sqr(MZ) + piZzT) /
    (sqr(displayGaugeCoupling(2)) +
     sqr(displayGaugeCoupling(1)) * 0.6); 

	if (vsquared < 0.0) return 246.22;

  return sqrt(vsquared);
}

// It'll set the important SUSY couplings: supposed to be applied at MZ
// You should set up an iteration here since Yuk's depend on top mass which
// depends on Yuk's etc. 
void MssmSoftsusy::sparticleThresholdCorrections(double tb) {
  if (displayMu() != MZ) {
    cout << "Called MssmSoftsusy::sparticleThresholdCorrections "
	 << "with scale" << displayMu() << endl;
    exit(1);
  }
  
  setTanb(tb);
  
  double alphaDrbar = qedSusythresh(ALPHAEM0, displayMu());
  double alphasMZDRbar =
    qcdSusythresh(displayDataSet().displayAlpha(ALPHAS), displayMu());
  
  // Do gauge couplings
  double outrho = 1.0, outsin = 0.5, tol = EPS * 1.0e-3; 
   int maxTries = 20;
   double pizztMZ = piZZT(MZ, displayMu());
   rhohat(outrho, outsin, alphaDrbar, pizztMZ, tol, maxTries);
   if (problem.noRhoConvergence) outsin = sqrt(1.0 - sqr(MW / MZ)); 
  
  double eDR = sqrt(4.0 * PI * alphaDrbar), costhDR = cos(asin(outsin));

  DoubleVector newGauge(3);
  newGauge(1) = eDR / costhDR * sqrt(5.0 / 3.0);
  newGauge(2) = eDR / outsin;
  newGauge(3) = sqrt(4.0 * PI * alphasMZDRbar);

  setGaugeCoupling(1, newGauge(1));
  setGaugeCoupling(2, newGauge(2));
  setGaugeCoupling(3, newGauge(3));

  double ht, hb, htau;
  double vev = getVev();
  ht = calcHt(vev); hb = calcHb(vev); htau = calcHtau(vev);
  
  if (MIXING < -1 || MIXING > 2) {
    cout << "In MssmSoftsusy::sparticleThresholdCorrections(double tb) ";
    cout << "\n MIXING=" << MIXING << " is out of range (-1 -> 2)\n";
    exit(1);
  }

  if (MIXING > -1) setDiagYukawas(displayDataSet(), vev);
  setYukawaElement(YU, 3, 3, ht);
  setYukawaElement(YD, 3, 3, hb);
  setYukawaElement(YE, 3, 3, htau);

  // 3-family mixed-up Yukawa couplings 
  static DoubleMatrix ckm(3, 3);
  if (ckm(1, 1) == 0.0) {
    ckm(1, 1) = 0.9752; ckm(1, 2) = -0.2205, ckm(1, 3) = -0.0031;
    ckm(2, 1) = 0.2205; ckm(2, 2) = 0.9745, ckm(2, 3) = -0.0390;
    ckm(3, 1) = 0.0085; ckm(3, 2) = 0.0385, ckm(3, 3) = -0.9993;
  }
  
  if (MIXING > 0) quarkMixing(ckm, MIXING); 
}

// Pass low-scale objects in, out and oneset (MSbar, at MZ)
// it will try to acheive tol fractional difference in mu and B from the
// iteration.
// returns 1 if cannot converge, 2 for no rewsb
// in and out are the latest iterations on return
void MssmSoftsusy::itLowsoft
(int maxTries, double mx, int sgnMu, double tol, 
double tanb, void (*boundaryCondition)(MssmSoftsusy &, const DoubleVector &), 
		const DoubleVector & pars) {

  static MssmSoftsusy old;

  static int numTries = 0;
  
  if (numTries != 0 && sqr(displayMu() / MZ - 1.0) > EPS) {
    cout << "WARNING: itLowsoft called at inappropriate";
    cout << " scale:" << displayMu() << endl; 
    cout << "whereas it should be " << MZ << endl; 
  }
  
  if (numTries - 1 > maxTries) {// Iterating too long: bail out
    flagNoConvergence(true);
    if (PRINTOUT) cout << "itLowsoft reached maxtries\n"; 
    numTries = 0;
    return;
  }
  double c = sumTol(*this, old);    
  
  if (c < tol) {// Accuracy achieved: bail out
    numTries = 0; // Reset the number of iterations for the next time
    if (PRINTOUT > 0) cout << " sT=" << c << " " << flush; 
    if (displayProblem().test() && PRINTOUT > 0) 
      cout << " ***problem point***: " << displayProblem() << ".";
    return; 
  }

  if (PRINTOUT > 1) cout << displayProblem(); 

  // All problems should be reset since only the ones of the final iteration
  // should count (sometimes problems disappear). This can mean that problems
  // only show up as no rho convergence....
  flagAllProblems(false);
  
  static double mtpole, mtrun;
  
  mtpole = displayDataSet().displayPoleMt();
  // On first iteration, don't bother with finite corrections
  
  numTries = numTries + 1;
  
  // Old iteration is 'old': these are the parameters by which convergence is
  // measured. 
  old.setSusyMu(displaySusyMu());
  old.setB(displayB());
  old.setMh1Squared(displayMh1Squared());
  old.setMh2Squared(displayMh2Squared());
  sparticleThresholdCorrections(tanb); 

  if (problem.noRhoConvergence) {
    if (PRINTOUT) cout << "No convergence in rhohat\n"; 
    //    numTries = 0; // DEBUG
    //    return;
  }
  
  // precision of running/RGE integration: start off low and increase
  double eps = maximum(exp(double(-1 - numTries) * log(10.0)), tol * 0.01); 
  
  // if we're having trouble converging, increase accuracy of the running
  if (numTries > 9) eps = eps / double(numTries);
  
  double ms = calcMs();

  int err = run(MZ, ms, eps);
  if (err) {
    // problem with running: bail out 
    if (PRINTOUT) cout << "itLowsoft gone non-perturbative on way to MX(u)\n"; 
    if (PRINTOUT > 1) cout << *this;
    numTries = 0;
    return;
  }
  
  mtrun = mtpole;

  if (numTries != 1) {
    double beta = atan(displayTanb());
    double vev = displayHiggsVevMs();
    mtrun = displayYukawaElement(YU, 3, 3) * vev * sin(beta) /
      sqrt(2.0); 
  }
  
  double MZp;
  if (PRINTOUT > 0) {
    double tb; MZp = sqrt(fabs(predMzsq(tb)));
  }

  physical(3);

  if (PRINTOUT > 0) {
    cout << "\nsT=" << c << " mu=" <<
      displaySusyMu() <<  " B=" <<
      displayB() 
	 << " MZp=" << MZp << flush;
  }
  
  err = run(ms, mx, eps);

  if (err || problem.noMuConvergence) {
    // problem with running: bail out 
    flagNonperturbative(true);
    if (PRINTOUT) cout << "itLowsoft gone non-perturbative\n"; 
    if (PRINTOUT > 1) cout << *this;
    numTries = 0;
    return;
  }
  
  boundaryCondition(*this, pars); 
  
  err = run(mx, ms, eps);
  ms = calcMs();

  if (err) {
    // problem with running: bail out 
    flagNonperturbative(true);
    if (PRINTOUT) 
      cout << "itLowsoft gone non-perturbative on way down to MS\n"; 
    if (PRINTOUT > 1) cout << *this;
    numTries = 0;
    return;
  }

  rewsb(sgnMu, mtrun);

  // If there is a big problem, you should really bail out
  if (displayProblem().b) return;

  err = runto(MZ, eps);
  if (err) {
    // problem with running: bail out 
    flagNonperturbative(true);
    if (PRINTOUT) cout << "itLowsoft gone non-perturbative\n"; 
    if (PRINTOUT > 1) cout << *this;
    numTries = 0;
    return;
  }

  itLowsoft(maxTries, mx, sgnMu, tol, tanb, boundaryCondition, pars);
}

// Transverse part of Z self-energy: has been checked
double MssmSoftsusy::piZZT(double p, double q) const {

  // fermions: these are valid at MZ
  double    mtop =  displayDataSet().displayPoleMt() ;
  double    mb   =  displayDataSet().displayMass(mBottom) ;
  double    mtau =  displayDataSet().displayMass(mTau) ;
  double    ms   =  displayDataSet().displayMass(mStrange) ;
  double    mc   =  displayDataSet().displayMass(mCharm) ;
  double    mmu  =  displayDataSet().displayMass(mMuon) ;
  double    mE  =  displayDataSet().displayMass(mElectron) ;
  double    mD  =  displayDataSet().displayMass(mDown) ;
  double    mU  =  displayDataSet().displayMass(mUp);
  double    thetaWDRbar = asin(calcSinthdrbar());
  double    cw2DRbar    = sqr(cos(thetaWDRbar));
  double    sw2DRbar    = 1.0 - cw2DRbar;
  double    g       = displayGauge()(2);
  double    alpha   = displayPhys().thetaH ;
  double    beta    = atan(displayTanb());
  double    thetat = displayPhys().thetat ;
  double    thetab = displayPhys().thetab;
  double    thetatau= displayPhys().thetatau ;
  double    st      = sin(thetat) ;
  double    sb      = sin(thetab) ;
  double    stau    = sin(thetatau) ;
  double    ct      = cos(thetat) ;
  double    cb      = cos(thetab) ;
  double    ctau    = cos(thetatau);
  double    ct2     = sqr(ct) ;
  double    cb2     = sqr(cb) ;
  double    ctau2   = sqr(ctau) ;
  double    st2     = (1.0 - ct2);
  double    sb2     = (1.0 - cb2);
  double    stau2   = (1.0 - ctau2);

  double rhs = 0.0;
  
  double higgs1 = 0.0, higgs2 = 0.0, higgs3 = 0.0, charginos = 0.0, 
    neutralinos = 0.0, squarks = 0.0, thirdFamily = 0.0, sneutrinos = 0.0;

  higgs1 = 
    - sqr(sin(alpha - beta)) *
    (b22bar(p, displayPhys().mhiggs(2), displayPhys().mhiggs(3), q) +
     b22bar(p, MZ, displayPhys().mhiggs(1), q) - 
     sqr(MZ) * b0(p, MZ, displayPhys().mhiggs(1), q));
 

  higgs2 =
    - sqr(cos(alpha - beta)) * 
    (b22bar(p, MZ, displayPhys().mhiggs(3), q) +
     b22bar(p, displayPhys().mhiggs(2), displayPhys().mhiggs(1), q) -
     sqr(MZ) * b0(p, MZ, displayPhys().mhiggs(3), q));
  
  higgs3 = 
    - 2.0 * sqr(cw2DRbar) * (2 * sqr(p) + sqr(MW) - sqr(MZ) *
			     sqr(sw2DRbar) / cw2DRbar)
    * b0(p, MW, MW, q)
    - (8.0 * sqr(cw2DRbar) + sqr(cos(2.0 * thetaWDRbar))) * 
    b22bar(p, MW, MW, q) - 
    sqr(cos(2.0 * thetaWDRbar)) * b22bar(p, displayPhys().mhiggs(4),
					 displayPhys().mhiggs(4), q);
  
  //static 
  DoubleVector vu(2), vd(2), ve(2);
  //static 
  DoubleMatrix vt(2, 2), vb(2, 2), vtau(2, 2);
    
  vu(1) = guL;
  vu(2) = guR;
  
  vt(1, 1) = guL * ct2 - guR * st2;
  vt(1, 2) = (guL + guR) * ct * st;
  vt(2, 2) = guR * ct2 - guL * st2;
  vt(2, 1) = vt(1, 2);
  
  vd(1) = gdL;
  vd(2) = gdR;
  
  vb(1, 1) = gdL * cb2 - gdR * sb2;
  vb(1, 2) = (gdL + gdR) * cb * sb;
  vb(2, 2) = gdR * cb2 - gdL * sb2;
  vb(2, 1) = vb(1, 2);
  
  ve(1) = geL;
  ve(2) = geR;
  
  vtau(1, 1) = geL * ctau2 - geR * stau2;
  vtau(1, 2) = (geL + geR) * ctau * stau;
  vtau(2, 2) = geR * ctau2 - geL * stau2;
  vtau(2, 1) = vtau(1, 2);
  
  int i, j, family;
  for (family = 1; family<=2; family++) {      
    // sneutrinos
    sneutrinos = sneutrinos - 3.0 * 
      0.25 * b22bar(p, displayPhys().msnu(family),
		    displayPhys().msnu(family), q);
    for (i=1; i<=2; i++) {
      // up squarks
      squarks = squarks - 12.0 * sqr(vu(i)) * 
	b22bar(p, displayPhys().mu(i, family), 
	       displayPhys().mu(i, family), q); 
      // down squarks
      squarks = squarks - 12.0 * sqr(vd(i)) * 
	b22bar(p, displayPhys().md(i, family), 
	       displayPhys().md(i, family), q);
      // selectrons
      squarks = squarks - 4.0 * sqr(ve(i)) * 
	b22bar(p, displayPhys().me(i, family), 
	       displayPhys().me(i, family), q); 
    }
  }
  family = 3;
  
  // THIRD FAMILY
  // sneutrinos
  thirdFamily = thirdFamily - 3.0 * 0.25 * 
    b22bar(p, displayPhys().msnu(family), displayPhys().msnu(family), q);
  for (i=1; i<=2; i++)
    for (j=1; j<=2; j++) {
      // up squarks
      thirdFamily = thirdFamily - 12.0 * sqr(vt(i, j)) * 
	b22bar(p, displayPhys().mu(i, family), 
	       displayPhys().mu(j, family), q); 
      
      // down squarks
      thirdFamily = thirdFamily - 12.0 * sqr(vb(i, j)) * 
	b22bar(p, displayPhys().md(i, family), 
	       displayPhys().md(j, family), q);
      // selectrons
       thirdFamily = thirdFamily - 4.0 * sqr(vtau(i, j)) * 
	b22bar(p, displayPhys().me(i, family), 
	       displayPhys().me(j, family), q); 
    }
  
  double quarks = 0.0;

  quarks = quarks + 3.0 * hfn(p, mU, mU, q) * (sqr(guL) + sqr(guR)); 
  quarks = quarks + 3.0 * hfn(p, mc, mc, q) * (sqr(guL) + sqr(guR)); 
  quarks = quarks + 3.0 * (hfn(p, mtop, mtop, q) * 
		     (sqr(guL) + sqr(guR)) - 4.0 * guL * guR * sqr(mtop) * 
		     b0(p, mtop, mtop, q)); 
  
  quarks = quarks + 3.0 * hfn(p, mD, mD, q) * (sqr(gdL) + sqr(gdR)); 
  quarks = quarks + 3.0 * hfn(p, ms, ms, q) * (sqr(gdL) + sqr(gdR)); 
  quarks = quarks + 3.0 * (hfn(p, mb, mb, q) * (sqr(gdL) + sqr(gdR)) - 4.0 * gdL * gdR
		     * sqr(mb) * b0(p, mb, mb, q)); 
  
  quarks = quarks + hfn(p, mE, mE, q) * (sqr(geL) + sqr(geR)); 
  quarks = quarks + hfn(p, mmu, mmu, q) * (sqr(geL) + sqr(geR)); 
  quarks = quarks + hfn(p, mtau, mtau, q) * 
    (sqr(geL) + sqr(geR)) - 4.0 * geL * geR
    * sqr(mtau) * b0(p, mtau, mtau, q) ; 
  
  // Neutralinos
  //static 
  ComplexMatrix aPsi(4, 4), bPsi(4, 4), aChi(4, 4), bChi(4, 4);
  ComplexMatrix n(4, 4); DoubleVector mneut(4);
  mpzNeutralinos(n, mneut); // gets to Matchev+Pierce's conventions  
   
  aPsi(3, 3) = g / (2 * cos(thetaWDRbar)); aPsi(4, 4) = 0. - aPsi(3, 3);
  bPsi = 0. - aPsi;
  
  aChi = n.complexConjugate() * aPsi * n.transpose();
  bChi = n * bPsi * n.hermitianConjugate();
  
  for (i=1; i<=4; i++)
    for (j=1; j<=4; j++) {
      neutralinos = neutralinos + cw2DRbar / (2.0 * sqr(g)) * 
	((sqr(aChi(i, j).mod()) + sqr(bChi(i, j).mod())) * 
	 hfn(p, mneut(i), mneut(j), q)
	 + 4.0 * (bChi(i, j).conj() * aChi(i, j)).real() *
	 mneut(i) * mneut(j) * b0(p, mneut(i), mneut(j), q)); 
    }
  
  // Charginos
  //  static 
  ComplexMatrix aPsiCh(2, 2), aCh(2, 2), bCh(2, 2);
  aPsiCh(1, 1) = g * cos(thetaWDRbar);
  aPsiCh(2, 2) = g * cos(2 * thetaWDRbar) / (2 * cos(thetaWDRbar));
  
  ComplexMatrix u(2, 2), v(2, 2); DoubleVector mch(2);
  mpzCharginos(u, v, mch);

  aCh = v.complexConjugate() * aPsiCh * v.transpose();
  bCh = u * aPsiCh * u.hermitianConjugate();
  
  for (i=1; i<=2; i++)
    for(j=1; j<=2; j++) {	
      charginos = charginos + cw2DRbar / sqr(g) * 
	((sqr(aCh(i, j).mod()) + sqr(bCh(i, j).mod())) * 
	 hfn(p, mch(i), mch(j), q) 
	 + 4.0 * (bCh(i, j).conj() * aCh(i, j)).real() * mch(i) *
	 mch(j) * b0(p, mch(i), mch(j), q));
    }

  rhs = higgs1 + higgs2 + higgs3 + charginos + neutralinos + squarks + 
    thirdFamily + sneutrinos + quarks;
  double pi = rhs * sqr(g) / (cw2DRbar * 16 * sqr(PI));

  if (pi < -sqr(MZ)) return 0.0; // DEBUG
	
  return pi;
}

//W propagator to 1 loop in MSSM 
// It's all been checked
double MssmSoftsusy::piWWT(double p, double q) const {
 
  double    beta    = atan(displayTanb());
  double    alpha   = displayPhys().thetaH ;
  double    mH = displayPhys().mhiggs(3); 
  double    mh0 = displayPhys().mhiggs(1);
  double    mHc = displayPhys().mhiggs(4);
  double    mA = displayPhys().mhiggs(2);
  // fermions: these are valid at MZ
  double    mtop =  displayDataSet().displayPoleMt() ;
  double    mb   =  displayDataSet().displayMass(mBottom) ;
  double    mtau =  displayDataSet().displayMass(mTau) ;
  double    ms   =  displayDataSet().displayMass(mStrange) ;
  double    mc   =  displayDataSet().displayMass(mCharm) ;
  double    mmu  =  displayDataSet().displayMass(mMuon) ;
  double    mE  =  displayDataSet().displayMass(mElectron) ;
  double    mD  =  displayDataSet().displayMass(mDown) ;
  double    mU  =  displayDataSet().displayMass(mUp);
  double    thetat = displayPhys().thetat ;
  double    thetab = displayPhys().thetab;
  double    thetatau= displayPhys().thetatau ;
  double    st      = sin(thetat) ;
  double    sb      = sin(thetab) ;
  double    stau    = sin(thetatau) ;
  double    ct      = cos(thetat) ;
  double    cb      = cos(thetab) ;
  double    ctau    = cos(thetatau);
  double    thetaWDRbar = asin(calcSinthdrbar());
  double    cw2DRbar    = sqr(cos(thetaWDRbar));
  double    sw2DRbar    = 1.0 - cw2DRbar;
  double    g       = displayGauge()(2);

  double ans = 0.0;

  double higgs = 0.0;
  
  higgs = higgs -
    sqr(sin(alpha - beta)) * 
    (b22bar(p, mH, mHc, q) + b22bar(p, mh0, MW, q) 
     - sqr(MW) * b0(p, mh0, MW, q));

  higgs = higgs -
    sqr(cos(alpha - beta)) *
    (b22bar(p, mh0, mHc, q) + b22bar(p, mH, MW, q) 
     - sqr(MW) * b0(p, mH, MW, q));

  higgs = higgs -
    b22bar(p, mA, mHc, q) - (1.0 + 8.0 * cw2DRbar) * b22bar(p, MZ, MW, q)
    - sw2DRbar * (8.0 * b22bar(p, MW, 0.0, q) + 4.0 * sqr(p) * 
		  b0(p, MW, 0.0, q))  
    - ((4.0 * sqr(p) + sqr(MZ) + sqr(MW)) * cw2DRbar - sqr(MZ)  *
       sqr(sw2DRbar)) * b0(p, MZ, MW, q);
  
  ans = ans + higgs;

  // fermions
  double fermions =
    1.5 * (hfn(p, mU, mD, q) + hfn(p, mc, ms, q) + hfn(p, mtop, mb, q)) + 0.5
    * (hfn(p, 0.0, mE, q) + hfn(p, 0.0, mmu, q) + hfn(p, 0.0, mtau, q));     
  ans = ans + fermions;
  
  // sfermions
  double sfermions = - 6.0 *
    (b22bar(p, displayPhys().mu(1, 1), displayPhys().md(1, 1), q) +
     b22bar(p, displayPhys().mu(1, 2), displayPhys().md(1, 2), q)) -
    2.0 * (b22bar(p, displayPhys().msnu(1), displayPhys().me(1, 1), q) +
	   b22bar(p, displayPhys().msnu(2), displayPhys().me(1, 2), q));
  ans = ans + sfermions;
  
  // stop/sbottom
  DoubleMatrix w(2, 2);
  double stopBot = 0.0;
  w(1, 1) = ct * cb; w(1, 2) = ct * sb; w(2, 1) = st * cb; w(2, 2) = st * sb;
  int i, j; 
  for(i=1; i<=2; i++)
    for(j=1; j<=2; j++)
      stopBot = stopBot - 6.0 * sqr(w(i, j)) * 
	b22bar(p, displayPhys().mu(i, 3), displayPhys().md(j, 3), q);
  ans = ans + stopBot;
  
  // LH slepton
  double slepton = - 2.0 * 
    (sqr(ctau) * 
     b22bar(p, displayPhys().msnu(3), displayPhys().me(1, 3), q)
     + sqr(stau) * b22bar(p, displayPhys().msnu(3), displayPhys().me(2, 3),
			  q));
  ans = ans + slepton;
  
  ComplexMatrix aPsi0PsicW(4, 2), bPsi0PsicW(4, 2), aChi0ChicW(4, 2),
    bChi0ChicW(4, 2);
  DoubleMatrix fW(4, 2), gW(4, 2);
  
  aPsi0PsicW(2, 1) = - g;
  bPsi0PsicW(2, 1) = - g;
  aPsi0PsicW(4, 2) = g / sqrt(2.0);		     
  bPsi0PsicW(3, 2) = -g / sqrt(2.0);		     
  
  ComplexMatrix aPsi(4, 4), bPsi(4, 4), aChi(4, 4), bChi(4, 4);
  ComplexMatrix n(4, 4); DoubleVector mneut(4);
  mpzNeutralinos(n, mneut); // gets to Matchev+Pierce's conventions 
  ComplexMatrix u(2, 2), v(2, 2); DoubleVector mch(2);
  mpzCharginos(u, v, mch);    

  // These ought to be in physpars
  aChi0ChicW = n.complexConjugate() * aPsi0PsicW * v.transpose();
  bChi0ChicW = n * bPsi0PsicW * u.hermitianConjugate();

  double gauginos = 0.0;

  for(i=1;i<=4;i++)
    for(j=1;j<=2;j++) {
      fW(i, j) = sqr(aChi0ChicW(i, j).mod()) + sqr(bChi0ChicW(i, j).mod());
      gW(i, j) = 2.0 * (bChi0ChicW(i, j).conj() * aChi0ChicW(i, j)).real(); 
      gauginos = gauginos + 
	(fW(i, j) * hfn(p, mneut(i), mch(j), q)
	 + 2.0 * gW(i, j) * mneut(i) * mch(j) * b0(p, mneut(i), mch(j), q)) 
	/ sqr(g);
    }
  
  ans = ans + gauginos;

  double pi = ans * sqr(g) / (16.0 * sqr(PI));
	
	//if (pi < -sqr(MW)) return 0.0; // DEBUG

  return pi;
}

double MssmSoftsusy::piAA(double p, double q) const {// checked 19.01.2001

  double    mtop =  displayDataSet().displayPoleMt() ;
  double    mb   =  displayDataSet().displayMass(mBottom) ;
  double    ms   =  displayDataSet().displayMass(mStrange) ;
  double    mc   =  displayDataSet().displayMass(mCharm) ;
  double    mD  =  displayDataSet().displayMass(mDown) ;
  double    mU  =  displayDataSet().displayMass(mUp);
  double    thetaWDRbar = asin(calcSinthdrbar());
  double    cw2DRbar    = sqr(cos(thetaWDRbar));
  double    sw2DRbar    = 1.0 - cw2DRbar;
  double    thetat  = displayPhys().thetat ;
  double    thetab  = displayPhys().thetab;
  double    st      = sin(thetat) ;
  double    sb      = sin(thetab) ;
  double    ct      = cos(thetat) ;
  double    cb      = cos(thetab) ;
  double    beta    = atan(displayTanb());
  double    g       = displayGauge()(2);
  double    mh0     = displayPhys().mhiggs(1);
  double    mHc     = displayPhys().mhiggs(4);
  double    mH     = displayPhys().mhiggs(3);
  double    mA      = displayPhys().mhiggs(2);
  double    gp      = displayGaugeCoupling(1) * sqrt(0.6);
  double cosb = cos(beta), cosb2 = sqr(cosb), cos2b = cos(2.0 * beta),
    sinb = sin(beta), sinb2 = sqr(sinb), sin2b = sin(2.0 * beta);

  double ans = 0.0;
  
  // Up fermions
  ans = ans + cosb2 * 
    (3.0 * sqr(displayYukawaElement(YU, 3, 3)) * 
     (sqr(p) * b0(p, mtop, mtop, q) - 2.0 * a0(mtop, q)) +
     3.0 * sqr(displayYukawaElement(YU, 2, 2)) * 
     (sqr(p) * b0(p, mc, mc, q) - 2.0 * a0(mc, q)) +
     3.0 * sqr(displayYukawaElement(YU, 1, 1)) * 
     (sqr(p) * b0(p, mU, mU, q) - 2.0 * a0(mU, q)));
  
  ans = ans +
    3.0 * (sqr(displayYukawaElement(YU, 1, 1)) * cosb2 - 
	   sqr(g) / (cw2DRbar) * cos2b * 0.5 * (0.5 - 2.0 / 3.0 * sw2)) *
    (a0(displayPhys().mu(1, 1), q)) +
    3.0 * (sqr(displayYukawaElement(YU, 2, 2)) * cosb2 - 
	   sqr(g) / (cw2DRbar) * cos2b * 0.5 * (0.5 - 2.0 / 3.0 * sw2)) *
    (a0(displayPhys().mu(1, 2), q)) +
    3.0 * (sqr(displayYukawaElement(YU, 3, 3)) * cosb2 - 
	   sqr(g) / (cw2DRbar) * cos2b * 0.5 * (0.5 - 2.0 / 3.0 * sw2)) *
    (sqr(ct) * a0(displayPhys().mu(1, 3), q) + 
     sqr(st) * a0(displayPhys().mu(2, 3), q));
  
  ans = ans +
    3.0 * (sqr(displayYukawaElement(YU, 1, 1)) * cosb2 - 
	   sqr(g) / (cw2DRbar) * cos2b * 0.5 * 2.0 / 3.0 * sw2) *
    (a0(displayPhys().mu(2, 1), q)) +
    3.0 * (sqr(displayYukawaElement(YU, 2, 2)) * cosb2 - 
	   sqr(g) / (cw2DRbar) * cos2b * 0.5 * 2.0 / 3.0 * sw2) *
    (a0(displayPhys().mu(2, 2), q)) +
    3.0 * (sqr(displayYukawaElement(YU, 3, 3)) * cosb2 - 
	   sqr(g) / (cw2DRbar) * cos2b * 0.5 * 2.0 / 3.0 * sw2) *
    (sqr(st) * a0(displayPhys().mu(1, 3), q) + 
     sqr(ct) * a0(displayPhys().mu(2, 3), q));
  
  ans = ans +
    3.0 * (sqr(displayYukawaElement(YU, 1, 1) * 
	       (-displaySusyMu() * sinb - 
		displaySoftA(UA, 1, 1) * cosb))) * 0.5 * 
    (b0(p, displayPhys().mu(1, 1), displayPhys().mu(2, 1), q) +
     b0(p, displayPhys().mu(2, 1), displayPhys().mu(1, 1), q)) + 
    3.0 * (sqr(displayYukawaElement(YU, 2, 2) * 
	       (-displaySusyMu() * sinb - 
		displaySoftA(UA, 2, 2) * cosb))) * 0.5 * 
    (b0(p, displayPhys().mu(1, 2), displayPhys().mu(2, 2), q) +
     b0(p, displayPhys().mu(2, 2), displayPhys().mu(1, 2), q)) + 
    3.0 * (sqr(displayYukawaElement(YU, 3, 3) * 
	       (-displaySusyMu() * sinb - 
		displaySoftA(UA, 3, 3) * cosb))) * 0.5 * 
    (b0(p, displayPhys().mu(1, 3), displayPhys().mu(2, 3), q) +
     b0(p, displayPhys().mu(2, 3), displayPhys().mu(1, 3), q));
  
  // Other Higgs'
  ans = ans +
    sqr(g) * 0.25 * 
    (2.0 * ffn(p, displayPhys().mhiggs(4), MW, q) + 
     sqr(sin(displayPhys().thetaH - beta)) / cw2DRbar *
     ffn(p, displayPhys().mhiggs(3), MZ, q) + 
     sqr(cos(displayPhys().thetaH - beta)) / cw2DRbar * 
     ffn(p, displayPhys().mhiggs(1), MZ, q));
  
  // trilinear Higgs coupling Feynman rules
  DoubleVector lAas(2), lAah(2);
  lAas(1) = -cos2b * cosb; lAas(2) = cos2b * sinb;
  lAas = lAas * (g * MZ / (2.0 * cos(thetaWDRbar)));
  lAah = rot2d(displayPhys().thetaH) * lAas;
  
  // quartic Higgs coupling Ferynman rules: check
  DoubleMatrix lAa12(2, 2), lAahh(2, 2);
  lAa12(1, 1) = -cos(2.0 * beta);
  lAa12(2, 2) = cos(2.0 * beta);
  lAahh = rot2d(displayPhys().thetaH) * lAa12 *
    rot2d(-displayPhys().thetaH);
  lAahh = lAahh * sqr(g) / (4.0 * cw2DRbar);
  double lAaaa = sqr(g) / (4.0 * cw2DRbar) * 3.0 * sqr(cos2b),
    lAagg = (3.0 * sqr(sin2b) - 1.0) * sqr(g) / (4.0 * cw2DRbar);
  
  ans = ans +
    0.5 * (sqr(lAah(2)) * b0(p, mA, mh0, q) + 
	   sqr(lAah(1)) * b0(p, mA, mH, q) + 
	   lAahh(1, 1) * a0(mH, q) + lAahh(2, 2) * a0(mh0, q) + 
	   lAaaa * a0(mA, q) + lAagg * a0(MZ, q)) +             
    sqr(g) * sqr(MW) * 0.5 * b0(p, MW, mHc, q) + 
    sqr(g) / (4.0 * cw2DRbar)
    * ((cw2DRbar * (1.0 + sqr(sin2b)) - sw2DRbar * sqr(cos2b)) * a0(MW, q) +
       sqr(cos2b) * a0(mHc, q));    
  
  ans = ans + sqr(g) * (2.0 * a0(MW, q) + a0(MZ, q) / cw2DRbar);
  
  DoubleMatrix aPsiP1(4, 4), aPsiP2(4, 4), bPsiP1(4, 4), 
    bPsiP2(4, 4);
  aPsiP1(1, 3) = - gp * 0.5;   aPsiP1(3, 1) = - gp * 0.5; 
  aPsiP1(2, 3) = g * 0.5;      aPsiP1(3, 2) = g * 0.5; 
  aPsiP2(1, 4) = - gp * 0.5;   aPsiP2(4, 1) = - gp * 0.5; 
  aPsiP2(2, 4) = g * 0.5;      aPsiP2(4, 2) = g * 0.5; 
  
  bPsiP1 = -1.0 * aPsiP1; bPsiP2 = -1.0 * aPsiP2;

  ComplexMatrix aPsi(4, 4), bPsi(4, 4), aChi(4, 4), bChi(4, 4);
  ComplexMatrix n(4, 4); DoubleVector mneut(4);
  mpzNeutralinos(n, mneut); // gets to Matchev+Pierce's conventions 
  ComplexMatrix u(2, 2), v(2, 2); DoubleVector mch(2);
  mpzCharginos(u, v, mch);      
  
  // Rotate to A neutralino basis
  ComplexMatrix aChiChiA(4, 4), bChiChiA(4, 4);
  aChiChiA = -sinb * n.complexConjugate() * aPsiP1 * n.hermitianConjugate() +
    cosb * n.complexConjugate() * aPsiP2 * n.hermitianConjugate();
  bChiChiA = -sinb * n * bPsiP1 * n.transpose() + cosb * n * bPsiP2 *
    n.transpose();
  
  DoubleMatrix fChiChiA(4, 4), gChiChiA(4, 4);
  int i, j;
  for (i=1; i<=4; i++)
    for (j=1; j<=4; j++) {
      fChiChiA(i, j) = sqr(aChiChiA(i, j).mod()) + sqr(bChiChiA(i, j).mod());
      gChiChiA(i, j) = 2.0 * (bChiChiA(i, j).conj() * aChiChiA(i, j)).real();
      ans = ans + 
	0.5 * fChiChiA(i, j) * 
	gfn(p, mneut(i), mneut(j), q) -
	gChiChiA(i, j) * mneut(i) * mneut(j ) * 
	b0(p, mneut(i), mneut(j), q);
    }
  
  ComplexMatrix aChPsiP1(2, 2), aChPsiP2(2, 2), bChPsiP1(2, 2),
    bChPsiP2(2, 2), aChPsiA(2, 2), bChPsiA(2, 2); 
  aChPsiP1(1, 2) = g / sqrt(2.0);   aChPsiP2(2, 1) = - g / sqrt(2.0);
  bChPsiP1 = - 1.0 * aChPsiP1.transpose(); 
  bChPsiP2 = - 1.0 * aChPsiP2.transpose();
  
  // To A-mass basis
  aChPsiA = -sinb * aChPsiP1 + cosb * aChPsiP2;
  bChPsiA = -sinb * bChPsiP1 + cosb * bChPsiP2;
  
  ComplexMatrix aChChiA(2, 2), bChChiA(2, 2);
  DoubleMatrix  fChChiA(2, 2), gChChiA(2, 2);
  aChChiA = v.complexConjugate() * aChPsiA * u.hermitianConjugate();
  bChChiA = u * bChPsiA * v.transpose();
  
  for (i=1; i<=2; i++)
    for (j=1; j<=2; j++) {
      fChChiA(i, j) = sqr(aChChiA(i, j).mod()) + sqr(bChChiA(i, j).mod());
      gChChiA(i, j) = 2.0 * (bChChiA(i, j).conj() * aChChiA(i, j)).real();
      ans = ans +
	fChChiA(i, j) * 
	gfn(p, mch(i), mch(j), q) -
	2.0 * gChChiA(i, j) * mch(i) * mch(j) * b0(p, mch(i), mch(j), q);
    }
  
  // Down fermions			
  ans = ans + sinb2 * 
    (3.0 * sqr(displayYukawaElement(YD, 3, 3)) * 
     (sqr(p) * b0(p, mb, mb, q) - 2.0 * a0(mb, q)) +
     3.0 * sqr(displayYukawaElement(YD, 2, 2)) * 
     (sqr(p) * b0(p, ms, ms, q) - 2.0 * a0(ms, q)) +
     3.0 * sqr(displayYukawaElement(YD, 1, 1)) * 
     (sqr(p) * b0(p, mD, mD, q) - 2.0 * a0(mD, q)));
  
  ans = ans +
    3.0 * (sqr(displayYukawaElement(YD, 1, 1)) * sinb2 +
	   sqr(g) / (cw2DRbar) * cos2b * 0.5 * (-0.5 + 1.0 / 3.0 * sw2)) *
    (a0(displayPhys().md(1, 1), q)) +
    3.0 * (sqr(displayYukawaElement(YD, 2, 2)) * sinb2 + 
	   sqr(g) / (cw2DRbar) * cos2b * 0.5 * (-0.5 + 1.0 / 3.0 * sw2)) *
    (a0(displayPhys().md(1, 2), q)) +
    3.0 * (sqr(displayYukawaElement(YD, 3, 3)) * sinb2 + 
	   sqr(g) / (cw2DRbar) * cos2b * 0.5 * (-0.5 + 1.0 / 3.0 * sw2)) *
    (sqr(cb) * a0(displayPhys().md(1, 3), q) + 
     sqr(sb) * a0(displayPhys().md(2, 3), q));
  
  ans = ans +
    3.0 * (sqr(displayYukawaElement(YD, 1, 1)) * sinb2 -
	   sqr(g) / (cw2DRbar) * cos2b * 0.5 * 1.0 / 3.0 * sw2) *
    (a0(displayPhys().md(2, 1), q)) +
    3.0 * (sqr(displayYukawaElement(YD, 2, 2)) * sinb2 - 
	   sqr(g) / (cw2DRbar) * cos2b * 0.5 * 1.0 / 3.0 * sw2) *
    (a0(displayPhys().md(2, 2), q)) +
    3.0 * (sqr(displayYukawaElement(YD, 3, 3)) * sinb2 - 
	   sqr(g) / (cw2DRbar) * cos2b * 0.5 * 1.0 / 3.0 * sw2) *
    (sqr(sb) * a0(displayPhys().md(1, 3), q) + 
     sqr(cb) * a0(displayPhys().md(2, 3), q));
  
  ans = ans +
    3.0 * (sqr(displayYukawaElement(YD, 1, 1) * 
	       (-displaySusyMu() * cosb - 
		displaySoftA(DA, 1, 1) * sinb))) * 0.5 * 
    (b0(p, displayPhys().md(1, 1), displayPhys().md(2, 1), q) +
     b0(p, displayPhys().md(2, 1), displayPhys().md(1, 1), q)) + 
    3.0 * (sqr(displayYukawaElement(YD, 2, 2) * 
	       (-displaySusyMu() * cosb - 
		displaySoftA(DA, 2, 2) * sinb))) * 0.5 * 
    (b0(p, displayPhys().md(1, 2), displayPhys().md(2, 2), q) +
     b0(p, displayPhys().md(2, 2), displayPhys().md(1, 2), q)) + 
    3.0 * (sqr(displayYukawaElement(YD, 3, 3) * 
	       (-displaySusyMu() * cosb - 
		displaySoftA(DA, 3, 3) * sinb))) * 0.5 * 
    (b0(p, displayPhys().md(1, 3), displayPhys().md(2, 3), q) +
     b0(p, displayPhys().md(2, 3), displayPhys().md(1, 3), q));

  return ans / (16.0 * sqr(PI));
}


// Has been checked
/*double MssmSoftsusy::piZGT(double p, double q) const {
#include "softbrevity.h"
  
  double ans = 0.0;
  
  ans = ans + 
    (12.0 * sw2DRbar - 10.0) * b22bar(p, MW, MW, q) - 
    2.0 * (sqr(MW) + 2.0 * cw2DRbar * sqr(p)) * b0(p, MW, MW, q);
  
  double guL = 0.5 - 2.0 * sw2 / 3.0; 
  double gdL = -0.5 + sw2 / 3.0; 
  double geL = -0.5 + sw2; 
  double guR = 2.
0 * sw2 / 3.0; 
  double gdR = -sw2 / 3.0; 
  double geR = -sw2; 
  
  ans = ans + 2.0 * (guL - guR) * (4.0 * b22bar(p, mU, mU, q) +
				   sqr(p) * b0(p, mU, mU, q));
  ans = ans + 2.0 * (guL - guR) * (4.0 * b22bar(p, mc, mc, q) +
				   sqr(p) * b0(p, mc, mc, q));
  ans = ans + 2.0 * (guL - guR) * (4.0 * b22bar(p, mtop, mtop, q) +
				   sqr(p) * b0(p, mtop, mtop, q));
  ans = ans - (gdL - gdR) * (4.0 * b22bar(p, mD, mD, q) +
			     sqr(p) * b0(p, mD, mD, q));
  ans = ans - (gdL - gdR) * (4.0 * b22bar(p, ms, ms, q) +
			     sqr(p) * b0(p, ms, ms, q));
  ans = ans - (gdL - gdR) * (4.0 * b22bar(p, mb, mb, q) +
			     sqr(p) * b0(p, mb, mb, q));
  ans = ans - (geL - geR) * (4.0 * b22bar(p, mE, mE, q) +
			     sqr(p) * b0(p, mE, mE, q));
  ans = ans - (geL - geR) * (4.0 * b22bar(p, mmu, mmu, q) +
			     sqr(p) * b0(p, mmu, mmu, q));
  ans = ans - (geL - geR) * (4.0 * b22bar(p, mtau, mtau, q) +
			     sqr(p) * b0(p, mtau, mtau, q));
  
  ans = ans - 2.0 * cos(2.0 * thetaWDRbar) * b22bar(p, mHc, mHc, q);
  
  DoubleMatrix v(2, 2), u(2, 2);
  v = rot2d(physpars.thetaR); u = rot2d(physpars.thetaL); // conventions
  
  int i; for (i=1; i<=2; i++) {
    ans = ans + 0.5 * (sqr(v(i, 1)) + sqr(u(i, 1)) 
		       + 2.0 * cos(2.0 * thetaWDRbar))
      * (4.0 * b22bar(p, displayPhys().mch(i), displayPhys().mch(i), q) +
	 sqr(p) * b0(p, displayPhys().mch(i), displayPhys().mch(i), q));
    
    ans = ans - 
      8.0 * 
      (guL * b22bar(p, displayPhys().mu(1, i), displayPhys().mu(1, i), q) 
       - guR * 
       b22bar(p, displayPhys().mu(2, i), displayPhys().mu(2, i), q)) +
      4.0 * 
      (gdL * b22bar(p, displayPhys().md(1, i), displayPhys().md(1, i), q) 
       - gdR * 
       b22bar(p, displayPhys().md(2, i), displayPhys().md(2, i), q)) +
      4.0 * 
      (geL * b22bar(p, displayPhys().me(1, i), displayPhys().me(1, i), q) 
       - geR * 
       b22bar(p, displayPhys().me(2, i), displayPhys().me(2, i), q));
  }
  
  ans = ans -  8.0 * 
    ((guL * ct2 - guR * st2) * b22bar(p, mstop1, mstop1, q) + 
     (guL * st2 - guR * ct2) * b22bar(p, mstop2, mstop2, q));
  ans = ans +  4.0 * 
    ((gdL * cb2 - gdR * sb2) * b22bar(p, msbot1, msbot1, q) + 
     (gdL * sb2 - gdR * cb2) * b22bar(p, msbot2, msbot2, q));
  ans = ans +  4.0 * 
    ((geL * ctau2 - geR * stau2) * b22bar(p, mstau1, mstau1, q) + 
     (geL * stau2 - geR * ctau2) * b22bar(p, mstau2, mstau2, q));
  
  double e = sqrt(ALPHAEM0 * 4.0 * PI);
  
  return ans * e * g / (16 * sqr(PI) * cos(thetaWDRbar)); 
} */

double rho2(double r) {// checked 
  if (r <= 1.9)
    return 19.0 - 16.5 * r + 43.0 * sqr(r) / 12.0 + 7.0 / 120.0 * sqr(r) * r -
      PI * sqrt(r) * (4.0 - 1.5 * r + 3.0 / 32.0 * sqr(r) + sqr(r) * r /
		      256.0) - sqr(PI) * (2.0 - 2.0 * r + 0.5 * sqr(r)) -
      log(r) * (3.0 * r - 0.5 * sqr(r));
  else {
    double rm1 = 1.0 / r, rm2 = sqr(rm1), rm3 = rm2 * rm1, rm4 = rm3 * rm1,
      rm5 = rm4 * rm1;
    return sqr(log(r)) * (1.5 - 9.0 * rm1 - 15.0 * rm2 - 48.0 * rm3 - 168.0
			  * rm4 - 612.0 * rm5) -
      log(r) * (13.5 + 4.0 * rm1 - 125.0 / 4.0 * rm2 - 558.0 / 5.0 * rm3 -
		8307.0 / 20.0 * rm4 - 109321.0 / 70.0 * rm5)
      + sqr(PI) * (1.0 - 4.0 * rm1 - 5.0 * rm2 - 16.0 * rm3 -
		   56.0 * rm4 - 204.0 * rm5)
      + 49.0 / 4.0 + 2.0 / 3.0 * rm1 + 1613.0 / 48.0 * rm2 + 87.57 * rm3 +
      341959.0 / 1200.0 * rm4 + 9737663.0 / 9800.0 * rm5;
  }
}

// Checked 20.11.00
// Flags noconvergence if there's trouble...then don't believe outrho and
// outsin produced - they are fudged!
void MssmSoftsusy::rhohat(double & outrho, double & outsin, double alphaDRbar,
			  double pizztMZ, double tol, int maxTries) {

  static double oldrho = 0.23, oldsin = 0.8;

  if (displayMu() != MZ) {
    cout << "Called MssmSoftsusy::rhohat "
	 << "with scale" << displayMu() << endl;
    exit(1);
  }
  
  static int numTries = 0;
  
  if ((outrho < EPS || outsin < EPS) || (numTries - 1 > maxTries)) { 
    oldrho = 0.23, oldsin = 0.8;
    numTries = 0;
    flagNoRhoConvergence(true);
    if (PRINTOUT) cout << flush << "rhohat reached maxtries\n"; 
    return;
  }
  
  // Difference to last iteration
  double sumTol;
  sumTol = fabs(oldrho / outrho - 1.0) + fabs(oldsin / outsin - 1.0);
  
  if (numTries != 0 && sumTol < tol) {
    numTries = 0;
    oldrho = 0.23, oldsin = 0.8;
    if (PRINTOUT > 2) cout << "sin rho converged\n";
    return;
  }

  double    beta    = atan(displayTanb());
  double    g       = displayGauge()(2);
  double    gp      = displayGaugeCoupling(1) * sqrt(0.6);
  double    costh   = (MW / MZ);
  double    cw2     = sqr(costh) ;
  double    sw2     = (1.0 - cw2);

  ComplexMatrix n(4, 4); DoubleVector mneut(4);
  ComplexMatrix u(2, 2), v(2, 2); DoubleVector mch(2);

  mpzNeutralinos(n, mneut); // gets to Matchev+Pierce's conventions 
  mpzCharginos(u, v, mch);          

  numTries = numTries + 1;
  
  oldrho = outrho; oldsin = outsin; 
  double outcos = cos(asin(outsin));
  
  if (PRINTOUT > 2) cout << " st2=" << sumTol;
  
  // 2 loop SM contribution
  double mt = displayDataSet().displayPoleMt(), q = displayMu();
  double sinb = sin(beta);
  
  double xt = 3.0 * GMU * sqr(mt) / (8.0 * sqr(PI) * sqrt(2.0));
  
  double deltaRho2LoopSm = alphaDRbar * sqr(displayGaugeCoupling(3)) / 
    (16.0 * PI * sqr(PI) * sqr(outsin) * sqr(outcos)) * 
    (-2.145 * sqr(mt) / sqr(MW) + 1.262 * log(mt / MZ) - 2.24 - 0.85 * sqr(MZ)
     / sqr(mt)) + sqr(xt) * sqr(cos(displayPhys().thetaH)) / sqr(sinb) *
    rho2(displayPhys().mhiggs(1) / mt) / 3.0;
  
  double deltaRhoOneLoop = pizztMZ / (outrho * sqr(MZ))
    - piWWT(MW, displayMu()) / sqr(MW);
  
  double deltaRho = deltaRhoOneLoop + deltaRho2LoopSm;
  
  double deltaVbSm = outrho * alphaDRbar / (4.0 * PI * sqr(outsin)) *
    (6.0 + log(cw2) / sw2 * 
     (3.5 - 2.5 * sw2 - sqr(outsin) * (5.0 - 1.5 * cw2 / sqr(outcos))));
  
  DoubleVector bPsi0NuNul(4), bPsicNuSell(2);
  DoubleVector bPsi0ESell(4), aPsicESnul(2);
  ComplexVector bChi0NuNul(4), bChicNuSell(2);
  ComplexVector bChi0ESell(4), aChicESnul(2);
  
  bPsicNuSell(1) = g;
  bPsi0NuNul(2) = sqrt(2.0) * g * 0.5;
  bPsi0NuNul(1) = - gp / sqrt(2.0);
  aPsicESnul(1) = g;
  bPsi0ESell(1) = -gp / sqrt(2.0);
  bPsi0ESell(2) = -g * sqrt(2.0) * 0.5;
  
  bChicNuSell = u * bPsicNuSell;
  bChi0ESell =  n * bPsi0ESell;
  bChi0NuNul = n * bPsi0NuNul;

  aChicESnul = v.complexConjugate() * aPsicESnul;
  
  double deltaZnue = 0.0, deltaZe = 0.0;
  double mselL = displayPhys().me(1, 1), msnue = displayPhys().msnu(1);
  int i; for(i=1; i<=4; i++) {
    if (i < 3) {
      deltaZnue = deltaZnue -
	sqr(bChicNuSell(i).mod()) * b1(0.0, mch(i), mselL, q);
      deltaZe = deltaZe -
	sqr(aChicESnul(i).mod()) * b1(0.0, mch(i), msnue, q);
    }
    deltaZnue = deltaZnue -
      sqr(bChi0NuNul(i).mod()) * b1(0.0, mneut(i), msnue, q);
    deltaZe = deltaZe -
      sqr(bChi0ESell(i).mod()) * b1(0.0, mneut(i), mselL, q);
  }
  
  DoubleVector bPsicNuSmul(2);
  DoubleVector bPsi0MuSmul(4), aPsicMuSnul(2);
  ComplexVector bChicNuSmul(2);
  ComplexVector bChi0MuSmul(4), aChicMuSnul(2);
  
  double hmu = displayYukawaMatrix(YE)(2, 2);
  bPsicNuSmul(1) = g;
  bPsicNuSmul(2) = -hmu;
  aPsicMuSnul(1) = g;
  aPsicMuSnul(2) = -hmu;
  bPsi0MuSmul(1) = -gp / sqrt(2.0);
  bPsi0MuSmul(2) = -g * sqrt(2.0) * 0.5;

  bChicNuSmul = u * bPsicNuSmul;
  bChi0MuSmul =  n * bPsi0MuSmul;
  bChi0NuNul = n * bPsi0NuNul;
  aChicMuSnul = v.complexConjugate() * aPsicMuSnul;
  
  double deltaZnumu = 0.0, deltaZmu = 0.0;
  double msnumu = displayPhys().msnu(1), msmuL = displayPhys().me(1, 2);
  for(i=1; i<=4; i++) {
    if (i < 3) {
      deltaZnumu = deltaZnumu -
	sqr(bChicNuSmul(i).mod()) * b1(0.0, mch(i), msmuL, q);
      deltaZmu = deltaZmu -
	sqr(aChicMuSnul(i).mod()) * b1(0.0, mch(i), msnumu, q);
    }
    deltaZnumu = deltaZnumu -
      sqr(bChi0NuNul(i).mod()) * b1(0.0, mneut(i), msnumu, q);
    deltaZmu = deltaZmu -
      sqr(bChi0MuSmul(i).mod()) * b1(0.0, mneut(i), msmuL, q);
  }
  
  DoubleMatrix aPsi0PsicW(4, 2), bPsi0PsicW(4, 2), fW(4, 2), gW(4, 2);
  ComplexMatrix aChi0ChicW(4, 2), bChi0ChicW(4, 2);
  
  aPsi0PsicW(2, 1) = - g;
  bPsi0PsicW(2, 1) = - g;
  aPsi0PsicW(4, 2) = g / sqrt(2.0);		     
  bPsi0PsicW(3, 2) = -g / sqrt(2.0);		     
  
  // These ought to be in physpars
  aChi0ChicW = n.complexConjugate() * aPsi0PsicW * v.transpose();
  bChi0ChicW = n * bPsi0PsicW * u.hermitianConjugate();
  
  Complex deltaVE = 0.0;
  int j; for(i=1; i<=2; i++)
    for(j=1; j<=4; j++) {
      deltaVE = deltaVE - bChicNuSell(i) * bChi0ESell(j).conj() *
	(- sqrt(2.0) / g * aChi0ChicW(j, i) * mch(i) * mneut(j) *
	 c0(mselL, mch(i), mneut(j)) + 1.0 / (sqrt(2.0) * g) *
	 bChi0ChicW(j, i) *
	 (b0(0.0, mch(i), mneut(j), q) + sqr(mselL) * 
	  c0(mselL, mch(i), mneut(j)) - 0.5));
      deltaVE = deltaVE - aChicESnul(i) * bChi0NuNul(j) *
	(- sqrt(2.0) / g * bChi0ChicW(j, i) * mch(i) * mneut(j) *
	 c0(msnue, mch(i), mneut(j)) + 1.0 / (sqrt(2.0) * g) *
	 aChi0ChicW(j, i) * 
	 (b0(0.0, mch(i), mneut(j), q) + sqr(msnue) * 
	  c0(msnue, mch(i), mneut(j)) - 0.5));
      if (i == 1)
	deltaVE = deltaVE +
	  0.5 * bChi0ESell(j).conj() * bChi0NuNul(j) *
	  (b0(0.0, mselL, msnue, q) + sqr(mneut(j)) * c0(mneut(j), mselL,
							 msnue) + 0.5);
    }
  
  Complex deltaVMu = 0.0;
  for(i=1; i<=2; i++)
    for(j=1; j<=4; j++) {
      deltaVMu = deltaVMu - bChicNuSmul(i) * bChi0MuSmul(j).conj() *
	(- sqrt(2.0) / g * aChi0ChicW(j, i) * mch(i) * mneut(j) *
	 c0(msmuL, mch(i), mneut(j)) + 1.0 / (sqrt(2.0) * g) *
	 bChi0ChicW(j, i) *
	 (b0(0.0, mch(i), mneut(j), q) + sqr(msmuL) * 
	  c0(msmuL, mch(i), mneut(j)) - 0.5));
      deltaVMu = deltaVMu - aChicMuSnul(i) * bChi0NuNul(j) *
	(- sqrt(2.0) / g * bChi0ChicW(j, i) * mch(i) * mneut(j) *
	 c0(msnumu, mch(i), mneut(j)) + 1.0 / (sqrt(2.0) * g) *
	 aChi0ChicW(j, i) * 
	 (b0(0.0, mch(i), mneut(j), q) + sqr(msnumu) * 
	  c0(msnumu, mch(i), mneut(j)) - 0.5));
      if (i == 1)
	deltaVMu = deltaVMu +
	  0.5 * bChi0MuSmul(j).conj() * bChi0NuNul(j) *
	  (b0(0.0, msmuL, msnumu, q) + sqr(mneut(j)) * c0(mneut(j), msmuL,
							  msnumu) + 0.5);
    }
  
  ComplexMatrix Debug(2, 4);
  Complex a1(0.0, 0.0);
  for(i=1; i<=2; i++)
    for(j=1; j<=4; j++) {
      a1 = a1 + 0.5 * aChicMuSnul(i) * bChicNuSell(i).conj() *
	bChi0NuNul(j) * bChi0ESell(j) * mch(i) * mneut(j) * 
	d0(mselL, msnumu, mch(i), mneut(j));
      a1 = a1 + 0.5 * aChicESnul(i).conj() * bChicNuSmul(i) *
	bChi0NuNul(j).conj() * bChi0MuSmul(j).conj() * mch(i) * mneut(j) * 
	d0(msmuL, msnue, mch(i), mneut(j));
      a1 = a1 + bChicNuSmul(i) * bChicNuSell(i).conj() *
	bChi0MuSmul(j).conj() * bChi0ESell(j) * 
	d27(msmuL, mselL, mch(i), mneut(j));
      a1 = a1 + aChicMuSnul(i).conj() * aChicESnul(i) *
	bChi0NuNul(j) * bChi0NuNul(j).conj() * 
	d27(msnumu, msnue, mch(i), mneut(j));
    } 

  double deltaVbSusy = 
    (-sqr(outsin) * sqr(outcos) / (2.0 * PI * alphaDRbar) * sqr(MZ)
     * a1.real() + deltaVE.real() + deltaVMu.real() + 
     0.5 * (deltaZe + deltaZnue + deltaZmu + deltaZnumu) ) /
    (sqr(PI) * 16.0); 

  double deltaVb = deltaVbSusy + deltaVbSm;

  double deltaR =  outrho * piWWT(0.0, displayMu()) / sqr(MW) 
    - pizztMZ / sqr(MZ) + deltaVb; 

  // Dominant two-loop SM term
  double deltaR2LoopSm = alphaDRbar * sqr(displayGaugeCoupling(3)) / 
    (16.0 * sqr(PI) * PI * sqr(outsin) * sqr(outcos)) *
    (2.145 * sqr(mt) / sqr(MZ) + 0.575 * log(mt / MZ) - 0.224 
     - 0.144 * sqr(MZ) / sqr(mt)) - 
    sqr(xt) * sqr(cos(displayPhys().thetaH)) / sqr(sinb) *
    rho2(displayPhys().mhiggs(1) / mt) * (1.0 - deltaR) * outrho / 3.0;
  
  deltaR = deltaR + deltaR2LoopSm; 
  
  double sin2thetasqO4 = PI * alphaDRbar / 
    (sqrt(2.0) * sqr(MZ) * GMU * (1.0 - deltaR)); 

  if (sin2thetasqO4 >= 0.25) sin2thetasqO4 = 0.25;
  if (sin2thetasqO4 < 0.0) sin2thetasqO4 = 0.0;

  double sin2theta = sqrt(4.0 * sin2thetasqO4);

  double theta = 0.5 * asin(sin2theta);
  
  outsin = sin(theta); 

  if (fabs(deltaRho) < 1.0) 
    outrho = 1.0 / (1.0 - deltaRho);
  else
    outrho = 1.0;

  if (PRINTOUT > 2) cout << " drho=" << deltaRho << " sw=" << outsin << endl;

  rhohat(outrho, outsin, alphaDRbar, pizztMZ, tol, maxTries);
}

// Returns mixing matrix n and neutralino masses mneut in the MPZ convention
// (hep-ph/9606211), n is 4 by 4 and mneut is 1->4.
void MssmSoftsusy::mpzNeutralinos(ComplexMatrix & n, DoubleVector & mneut)
  const { 
  // We want to change the PHASES of the neutralino mixing matrix in order to
  // produce POSITIVE neutralino masses, a la Matchev, Pierce and Zhang

  DoubleMatrix temp(displayPhys().mneut);
  
  ComplexMatrix K(4, 4);
  int i; for (i=1; i<=4; i++) 
    if (displayPhys().mneut(i) < 0.0) K(i, i) = Complex(0.0, 1.0);
    else
      K(i, i) = Complex(1.0, 0.0);
  
  mneut = displayPhys().mneut.apply(fabs);
  n = K.hermitianConjugate() * displayPhys().mixNeut.transpose();
}

// Returns mixing matrices u,v and neutralino masses mneut in the MPZ
// convention (hep-ph/9606211),  u+v are (2,2) and mch is 1->2.
void MssmSoftsusy::mpzCharginos(ComplexMatrix & u, ComplexMatrix & v,
				DoubleVector & mch) const {
  // We want to change the PHASES of the neutralino mixing matrix in order to
  // produce POSITIVE neutralino masses, a la Matchev, Pierce and Zhang
  positivise(displayPhys().thetaL, displayPhys().thetaR, displayPhys().mch,
	     u, v);
  mch = displayPhys().mch.apply(fabs); 
}

void MssmSoftsusy::methodBoundaryCondition(const DoubleVector & pars) {
  cout << "Should only use MssmSoftsusy::methodBoundaryCondition in derived"
       << " objects.\n";
  exit(1);
}

// Works out how best to fit the isajet numbers to the spectrum.
// There are problems with the Higgs and sbottoms because ISAJET assumes
// certain tree-level relations between masses that are broken by SOFTSUSY's
// higher accuracy. The differences get large for high tan beta around 50, at
// around 10 they're typically only a percent.
void MssmSoftsusy::isajetNumbers751 
(double & mtopPole, double & mGPole, double & smu, double & mA, double & tanb, 
 double & mq1l, double & mdr, double & mur, double & meL, double & meR, 
 double & mql3, double & mdr3, double & mur3, double &  mtauL, 
 double & mtauR, double & at, double & ab, double & atau, double & mq2l, 
 double & msr, double & mcr, double & mmuL, double & mmuR, double & m1, 
 double & m2) 
  const {

  // Store a copy of the current object
  MssmSoftsusy store(*this);

  // Run to MSUSY 
  store.runto(calcMs(), EPS * 0.01);

  double pizztMS = store.piZZT(MZ, displayMu());

  DoubleMatrix mNeut(4, 4);
  
  // Store current object in vector
  DoubleVector storeObject(store.display());
  double muInitial = store.displayMu();

  // In order to get M2(mch1), mu(mch2)
  store.runto(store.displayGaugino()(1));
  m1 = store.displayGaugino()(1);
  store.runto(store.displayGaugino()(2));
  m2 = store.displayGaugino()(2); 
  smu = store.displaySusyMu();
  store.runto(fabs(smu));
  smu = store.displaySusyMu(); 

  // restore object at MSUSY
  store.setMu(muInitial);
  store.set(storeObject);

  // tree level
  mNeut(1, 1) = m1;
  mNeut(2, 2) = m2;
  mNeut(3, 4) = - smu;

  // This is basically a fix: ISAJET currently calculates tree-level 
  // masses for charginos/neutralinos, so we use M1, M2, mu as I have done
  // in the one-loop corrections. Off-diagonal terms will be different.
  store.addNeutralinoLoop(mNeut);
  m1 = mNeut(1, 1);
  m2 = mNeut(2, 2);
  smu = -mNeut(3, 4);

  double beta = atan(store.displayTanb());
  double sinthDRbarMS = store.calcSinthdrbar();

  // Add most important radiative corrections
  DoubleMatrix mass(2, 2);
  mass(1, 1) = store.displaySoftMassSquared(mQl)(1, 1);
  mass(2, 2) = store.displaySoftMassSquared(mDr)(1, 1);
  store.addSquarkCorrection(mass); 
  mq1l = sqrt(mass(1, 1)); mdr = sqrt(mass(2 ,2));

  mass(1, 1) = store.displaySoftMassSquared(mUr)(1, 1);
  mass(2, 2) = store.displaySoftMassSquared(mQl)(2, 2);
  store.addSquarkCorrection(mass); 
  mur = sqrt(mass(1, 1)); mq2l = sqrt(mass(2, 2));

  mass(1, 1) = store.displaySoftMassSquared(mDr)(2, 2);
  mass(2, 2) = store.displaySoftMassSquared(mUr)(2, 2);
  store.addSquarkCorrection(mass); 
  msr = sqrt(mass(1, 1)); mcr = sqrt(mass(2, 2));

  // Needs more work: another fiddle
  double vev = store.displayHiggsVevMs();
  const double isaCorr = 4.3;
  double mtrun = store.displayYukawaElement(YU, 3, 3) * vev * sin(beta) 
    / sqrt(2.0) + isaCorr; // fiddle factor on end because ISAJET ignores some
		       // contributions   

  DoubleMatrix mStopSquared(2, 2);
  store.treeStop(mStopSquared, mtrun, pizztMS, sinthDRbarMS, 3);
  store.addStopCorrection(mStopSquared, mtrun);  

  double mzIsajet = 91.17, sw2Isajet = 0.232;
  DoubleMatrix mSbotSquared(2, 2);
  double mbrun = store.displayYukawaElement(YD, 3, 3) * vev * cos(beta) 
    / sqrt(2.0);
  store.treeSbot(mSbotSquared, displayDataSet().displayMass(mBottom), 
	   pizztMS, sinthDRbarMS, 3);
  store.addSbotCorrection(mSbotSquared, mtrun - isaCorr);

  // We're fiddling these variables to reproduce inside isajet the squark mass
  // matrices that we've found in softsusy
  mql3 = sqrt(mStopSquared(1, 1) - sqr(mtrun) - 
	      (0.5 - 2.0 / 3.0 * sw2Isajet) * sqr(mzIsajet) * 
	      cos(2.0 * beta));
  mdr3 = sqrt(mSbotSquared(2, 2) - sqr(mbrun) 
	      + 1.0 / 3.0 * sw2Isajet * sqr(mzIsajet) * cos(2.0 * beta));
  mur3 = sqrt(mStopSquared(2, 2) - sqr(mtrun) -
	      2.0 / 3.0 * sw2Isajet * sqr(mzIsajet) * cos(2.0 * beta));
  mtauL = sqrt(store.displaySoftMassSquared(mLl)(3, 3) + 
	       0.5 * pizztMS * cos(2.0 * beta));
  mtauR = sqrt(store.displaySoftMassSquared(mEr)(3, 3));

  at  = mStopSquared(1, 2) / mtrun + smu / tan(beta);
  ab  = mSbotSquared(1, 2) / mbrun + smu * tan(beta);
  atau = store.displaySoftA(EA, 3 ,3);

  mmuL = sqrt(store.displaySoftMassSquared(mLl)(2, 2) + 
	      0.5 * pizztMS * cos(2.0 * beta));
  mmuR = sqrt(store.displaySoftMassSquared(mEr)(2, 2));

  mtopPole = store.displayDataSet().displayPoleMt();
  mGPole   = store.displayPhys().mGluino;
  mA       = store.displayPhys().mhiggs(2);
  meL      = sqrt(store.displaySoftMassSquared(mLl)(1, 1) + 
		  0.5 * pizztMS * cos(2.0 * beta));
  meR      = sqrt(store.displaySoftMassSquared(mEr)(1, 1));
  
  tanb = store.displayTanb();
}

void MssmSoftsusy::ssrunInterface751(char fname [80]) const {

  double mtopPole, mGPole, smu, mA, tanb, mq1l, mdr, mur, 
    meL, meR, mql3, mdr3, mur3,  mtauL, mtauR, at, ab, atau, 
    mq2l, msr, mcr, mmuL, mmuR, m1, m2;

  isajetNumbers751(mtopPole, mGPole, smu, mA, tanb, mq1l, mdr, mur, 
		   meL, meR, mql3, mdr3, mur3,  mtauL, mtauR, at, ab, atau, 
		   mq2l, msr, mcr, mmuL, mmuR, m1, m2);

  cout << "'" << fname << "'" << endl;

  cout << mtopPole << endl;
  
  cout << mGPole  << "," << smu << "," << mA << "," << tanb
       << endl;

  cout << mq1l    << "," << mdr << "," << mur << "," 
       << meL     << "," << meR << endl;

  cout << mql3    << "," << mdr3  << "," << mur3 << "," 
       << mtauL   << "," << mtauR << "," 
       << at      << "," << ab    << "," << atau
       << endl;

  cout << mq2l << "," << msr << "," << mcr << ","  
       << mmuL << "," << mmuR
       << endl;

  cout << m1 << "," << m2 << endl << "/" << endl;

}

void MssmSoftsusy::isawigInterface751(char herwigInputFile [80], 
				      char isajetOutputFile [80])  
  const {

  ssrunInterface751(isajetOutputFile);

  cout << "n" << endl << "n" << endl;

  cout << "'" << herwigInputFile << "'" << endl;
}

// For input into isajet parameter file
void MssmSoftsusy::isajetInterface751() const {

  double mtopPole, mGPole, smu, mA, tanb, mq1l, mdr, mur, 
    meL, meR, mql3, mdr3, mur3,  mtauL, mtauR, at, ab, atau, 
    mq2l, msr, mcr, mmuL, mmuR, m1, m2;

  isajetNumbers751(mtopPole, mGPole, smu, mA, tanb, mq1l, mdr, mur, 
		   meL, meR, mql3, mdr3, mur3,  mtauL, mtauR, at, ab, atau, 
		   mq2l, msr, mcr, mmuL, mmuR, m1, m2);

  // PRINT out the fiddle in ISAJET PARAMETER file format
  cout << "TMASS"  << endl;
  cout << mtopPole << "/" << endl;

  cout << "MSSMA" << endl;
  cout << mGPole  << "," << smu << "," << mA << "," << tanb
       << "/"     << endl;

  cout << "MSSMB" << endl;
  cout << mq1l    << "," << mdr << "," << mur << "," 
       << meL     << "," << meR << "/" << endl;

  cout << "MSSMC" << endl;
  cout << mql3    << "," << mdr3  << "," << mur3 << "," 
       << mtauL   << "," << mtauR << "," 
       << at      << "," << ab    << "," << atau
       << "/" << endl;

  cout << "MSSMD" << endl;
  cout << mq2l << "," << msr << "," << mcr << ","  
       << mmuL << "," << mmuR
       << "/" << endl;

  cout << "MSSME" << endl;
  cout << m1 << "," << m2 << "/" << endl;
}
 
